<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief generates comment feed from the site
 *
 * A feed is generated to inform feed readers about the last comments in the site.
 * <h3>format</h3>
 * The generated feed can be in 3 formats :
 * <ul>
 *  <li>RSS 0.91</li>
 *  <li>RSS 1.0</li>
 *  <li>ATOM</li>
 * </ul>
 * the format is controlled with the HTML GET variable "feed_type". RSS 1.0 is the default format.
 * <h3>content</h3>
 * The feed can contains the whole content or just a small sentence indicating the changement.
 * This is controlled by the HTML GET variable "full_content".
 * <h3>site wide or related to a particular page</h3>
 * The generated feed can be site wide, or related to a particular page. In that case, all comments of the page
 * is served as a feed.
 * This is controlled by the HTML GET variable "title".
 * <h3>lang</h3>
 * Finally the lang can be controlled. The lang controlled in one of the lang available for the content.
 * We're not talking about ui lang here (it doesn't make sense).
 * It is controlled by the HTML GET variable "lang".
 * <h3>Url example</h3>
 * <ul>
 *   <li>recent comments for fr pages : http://server/comment_feed.php?lang=fr</li>
 *   <li>recent comments for fr pages in atom : http://server/comment_feed.php?lang=fr&feed_type=atom</li>
 *   <li>all comments of page Bugs in default lang : http://server/comment_feed.php?title=Bugs</li>
 *   <li>all comments for page Bugs in default lang with full content: http://server/comment_feed.php?title=Bugs&full_content=1</li>
 * </ul>
 */

error_reporting(E_ALL);
@ini_set('display_errors','on');

$file_path = __FILE__;
$path      = dirname(__FILE__).'/';
// let's use a constant instead of a variable so that hacker can't use it to inject something using a require
// on an unprotected directory
define('wc_inc_path', $path);
unset($path);

require wc_inc_path.'/inc/lib/constants.lib.php';

// conf
require wc_inc_path.'/inc/classes/WiclearIni.conf.php';
require wc_inc_path.'/inc/classes/ini.class.php';
$ini = new ini(wc_inc_path.'/conf/config.ini');
$ini->importConstants();

@ini_set('default_charset',wc_charset);
@ini_set('mbstring.func_overload','7');
@ini_set('mbstring.internal_encoding',wc_charset);
@ini_set('mbstring.internal_encoding',wc_charset);

// variable
require wc_inc_path.'/inc/lib/variables.lib.php';

// rss
require wc_inc_path.'/inc/classes/date_helper.class.php';
require wc_inc_path.'/inc/classes/feed.class.php';

function generateFeed($wiki, $title, $lang, $fullContent, $feedType)
{
  global $app_path;

  // Set number of items in feed
  $feedLimit = 15;

  $feedTitle = wc_wiki_name.' - '.tr('Recent comments');
  $feedDescription = tr('Recent comments');
  if ($title != '')
  {
    $feedTitle = $title.' - '.tr('comments feed for').' '.$title;
    $feedDescription = $title.' '.tr('comments');
  }

  $feedTitle       = formatHtml($feedTitle);
  $feedDescription = formatHtml($feedDescription);

  // retrieve data
  if ($title == '')
  {
    $items = $wiki->recentComments(0, $feedLimit , true, $fullContent);
  }
  else
  {
    $items = $wiki->recentCommentsByTitle($title, $lang, 0, $feedLimit , true, $fullContent);
  }

  // create feed object and header
  switch($feedType)
  {
    case 'atom' :
      require wc_inc_path.'/inc/classes/atom.class.php';
      $feed = new atom();
    break;
    case 'rss_2' :
      require wc_inc_path.'/inc/classes/rss_2.class.php';
      $feed = new rss_2();
    break;
    case 'rss_1' :
    default :
      require wc_inc_path.'/inc/classes/rss_1.class.php';
      $feed = new rss_1();
    break;
  }

  // Setup core feed details
  $feed->setCharset(wc_charset);
  $feed->setLang($lang);
  $feed->setTitle($feedTitle);
  $feed->setDescription($feedDescription);
  $feed->setSiteUrl($app_path);
  $feed->setFeedUrl($app_path.'comment_feed.php?'.htmlspecialchars( $_SERVER["QUERY_STRING"]) );

  // Add feed items
  foreach ($items as $item)
  {
    $content_title = $item[0];
    $comment_id    = $item[1];
    $comment_title = $item[2];
    $date          = $item[3];
    $lang          = $item[4];
    $author        = $item[5].' '.$item[6];

    if ($fullContent)
    {
      $description = array('content' => nl2br(htmlspecialchars($item[7])), 'type' => 'xhtml');
    }
    else
    {
      $description = array('content' => tr('Modified on').' '.formatDate($date), 'type' => 'text');
    }

    $url = createURL($content_title, array('lang'=>$lang, 'anchor'=>'co'.$comment_id));

    $feed->addItem(formatHtml($comment_title), $author, $date, $lang, $description, $url);
  }

  return $feed->generate();
}

// extract variables from url
$title       = isset($_GET['title']) ? varGet('title') : '';
$fullContent = isset($_GET['full_content']) ? true : false;
$feedType    = isset($_GET['feed_type']) ? varGet('feed_type') : wc_feed_format;
$lang        = isset($_GET['lang']) ? varGet('lang') : wc_default_lang;

// generate unique id from there
$id = 'comment_feed-'.$title.'-'.$lang.'-'.$fullContent.'-'.$feedType;

// cache
require wc_inc_path.'/inc/classes/PEAR/Cache_Lite.class.php';

$cacheDir = wc_inc_path.wc_cache_path.'/';

if (!file_exists($cacheDir))
{
  @mkdir($cacheDir);
}

// Set a few options
$options = array(
  'cacheDir' => $cacheDir,
  'lifeTime' => 60*60
);
$cacheLite = new Cache_Lite($options);

if (wc_activate_cache == "true" && ($feed = $cacheLite->get($id, cache_global_pages_name)) !== false)
{
  // cache hit
}
else
{
  // for some reason, including everything inside generateFeed provokes errors
  // => includes them here

  // translation
  require wc_inc_path.'/inc/classes/translation.class.php';
  if (wc_default_lang != 'en')
  {
    $translation = new translation(wc_inc_path.'/i18n/', wc_default_lang, 'en', '.lang');
  }

  // url
  require wc_inc_path.'/inc/lib/url.lib.php';
  $app_path = createWebAppPath();

  require wc_inc_path.'/inc/classes/chrono.class.php';

  // db
  require wc_inc_path.'/inc/classes/db_connection.class.php';
  require wc_inc_path.'/inc/classes/mysql.class.php';
  $con = new mysql(wc_db_host, wc_db_login, wc_db_password, wc_db_database);

  if (!$con->connected())
  {
    return;
  }

  // wiki
  require wc_inc_path.'/inc/classes/wiki.class.php';
  $wiki = new wiki($con, wc_db_prefix);

  // format
  require wc_inc_path.'/inc/lib/format.lib.php';

  $feed = generateFeed($wiki, $title, $lang, $fullContent, $feedType);
  $cacheLite->save($feed, $id, cache_global_pages_name);
}

// send header
switch($feedType)
{
  case 'atom':
    require_once wc_inc_path.'/inc/classes/atom.class.php';
    atom::headers(wc_charset);
  break;
  case 'rss_2':
    require_once wc_inc_path.'/inc/classes/rss_2.class.php';
    rss_2::headers(wc_charset);
  break;
  case 'rss_1':
  default:
    require_once wc_inc_path.'/inc/classes/rss_1.class.php';
    rss_1::headers(wc_charset);
  break;
}

echo $feed;
?>
