<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief construct a check list
 *
 * a check list is used when performing several actions that need report to the user when everything's finished.
 * you use it by constructing it with 2 different styles.
 * Then for each actions, you call addItem() with a boolean indicating the success or failure of the operation,
 * and 2 strings.
 * Finally when you're done and want to display result to user, you call htmlReport()
 */
class check_list
{
  var $ok_style;   //!< success style
  var $nok_style;  //!< failure style
  var $warn_style; //!< warning style

  var $style;      //!< list of styles
  var $text;       //!< list of text

  var $hasWarning; //!< there was at least a warning, but we should be able to proceed
  var $allOK;      //!< there was at least a failure

  /**
   * \brief constructs a check_list
   *
   * \param ok_style success style to use for report
   * \param nok_style failure style to use for report
   * \param warn_style warn style to use for report
   */
  function check_list($ok_style, $nok_style, $warn_style='')
  {
    $this->ok_style   = $ok_style;
    $this->nok_style  = $nok_style;
    $this->warn_style = $warn_style;

    $this->style = array();
    $this->text  = array();

    $this->hasWarning = false;
    $this->allOK      = true;
  }

  /**
   * \brief add a new report item
   *
   * \param result result of the operation
   * \param ok_string the string to display if result was true
   * \param nok_string the string to display if result was false
   * \param is_warning indicates if the error must be handled as a fatal error or a warning
   */
  function addItem($result, $ok_string, $nok_string, $is_warning = false)
  {
    array_push($this->style, $result ? $this->ok_style  : ($is_warning ? $this->warn_style : $this->nok_style));
    array_push($this->text,  $result ? $ok_string       : $nok_string);

    if (!$result)
    {
      if ($is_warning)
      {
        $this->hasWarning = true;
      }
      else
      {
        $this->allOK = false;
      }
    }
  }

  /**
   * \brief display an html report as an unordered list
   *
   * \return html display
   */
  function htmlReport()
  {
    $blob = '<ul>';
    $nbReports = sizeof($this->style);
    for ($i = 0; $i < $nbReports; ++$i)
    {
      $blob .= '<li class="'.$this->style[$i].'">'.$this->text[$i].'</li>';
    }
    $blob .= '</ul>';

    return $blob;
  }

  /**
   * \brief were all operations successfull ?
   *
   * \brief return true if all operations were successfull
   */
  function allOK()
  {
    return $this->allOK;
  }

  /**
   * \brief has one operation generated a warning ?
   *
   * \brief return true if at least one operation was a warning
   */
  function hasWarning()
  {
    return $this->hasWarning;
  }
}
?>