<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief interface connection to a db
 */
class db_connection
{
  var $debug_mode;                   //!< debug flag (will output each query before executing it if true)
  var $no_execute;                   //!< won't execute any statements if true

  var $cumulated_sql_execution_time; //!< cumulated time taken to perform sql queries

  var $error;                        //!< last error

  /**
   * \brief return if the connection is up with the base
   *
   * \return state of the connection
   */
  function connected() { return false; }

  /**
   * \brief escape a string so that it is safe to include it in a query
   *
   * should be called by user code to escape all user provided variables
   *
   * \param str the string to escape
   * \return escaped string
   */
  function escapeString($str) { return addslashes($str); }

  /**
   * \brief use debug mode
   *
   * \param debug_mode true (debug mode on), or false (debug mode off)
   */
  function setDebug($debug_mode)
  {
    $this->debug_mode = $debug_mode;
  }

  /**
   * \brief use no execute mode
   *
   * \param no_execute true (no execute mode on), or false (no execute mode off)
   */
  function setNoExecute($no_execute)
  {
    $this->no_execute = $no_execute;
  }

  /**
   * \brief retrieve last error from database
   */
  function setError() {}

  /**
   * \brief executes an sql query
   *
   * \param query the query to execute (INSERT, UPDATE, DELETE)
   */
  function execute($query) {}

  /**
   * \brief do an insert and retrieve generated id
   *
   * \param query the insert query
   * \return generated id
   */
  function insertAndGetId($query) { return 0; }

  /**
   * \brief do a select that yields one row
   *
   * \param query the select that returns only one row
   * \return row as an array, or false if there was an error
   */
  function select1($query) { return false; }

  /**
   * \brief do a select that yields multiple row
   *
   * \param query the select that returns only one row
   * \return rows as an array of array, or false if there was an error
   */
  function select($query) { return false; }

  /**
   * \brief close this connection
   */
  function close() {}

  /**
   * \brief checks if a table exists
   */
  function tableExists($table_name) { return false; }

  /**
   * \brief creates a table from an sql file
   *
   * \param file path to sql file containing create sql statements
   * \param prefix if the sql statement contains {prefix} string, this is the string to use as replacement
   * \param charset charset to use when creating table
   */
  function createTableFromFile($file, $prefix, $charset)
  {
    if (file_exists($file))
    {
      $query = file_get_contents($file);
      $query = str_replace('{prefix}', $prefix, $query);
      if ($this->createTable($query, $charset) === true)
      {
        return true;
      }
      else
      {
        $this->setError();
      }
    }
    else
    {
      $this->error = tr('Error : sql file').' '.$file.' '.tr('not found');
    }

    return false;
  }

  /**
   * \brief creates a table
   *
   * \param query query that contains the create table sql statement
   * \param charset default charset of MySql table
   */
  function createTable($query, $charset)
  {
    return $this->execute($query);
  }

  var $knownDbCharset = array('utf8', 'latin1');

  /**
   * \brief convert a web charset to a MySql charset
   *
   * \param charset web charset to convert
   * \return MySql charset
   */
  function htmlCharsetToMySqlCharset($charset)
  {
    if ($charset == 'UTF-8')
    {
      return 'utf8';
    }
    else
    if ($charset == 'ISO-8859-1')
    {
      return 'latin1';
    }

    return '';
  }

  /**
   * \brief convert a web charset to a MySql charset
   *
   * \param charset web charset to convert
   * \return MySql charset
  */
  function mySqlCharsetToHtmlCharset($charset)
  {
    if ($charset == 'utf8')
    {
      return 'UTF-8';
    }
    else
    if ($charset == 'latin1')
    {
      return 'ISO-8859-1';
    }

    return '';
  }
}

?>