<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief is this variable an associative array
 *
 * \param var variable to test
 * \return true if variable is an associative array, false otherwise
 */
function is_assoc($var)
{
  return is_array($var) && array_keys($var) !== range(0, sizeof($var) - 1);
}

/**
 * \brief helper class to generate xhtml valid form
 */
class form
{
  var $blob;           //!< cached output
  var $openedFieldset; //!< do we have an opened field set

  /**
   * \brief constructs a new form
   *
   * \param action action target url
   * \param method method to use ("post", "get")
   * \param class optional class attribute
   * \param encoding optional encoding to use
   */
  function form($action, $method, $class = '', $encoding='')
  {
    $ext = '';
    if (!empty($class))
    {
      $ext = 'class="'.$class.'"';
    }
    if (!empty($encoding))
    {
      $ext = ' enctype="'.$encoding.'"';
    }

    $this->blob = '<form action="'.$action.'" method="'.$method.'" accept-charset="'.wc_charset.'" '.$ext.'>';
    $this->openedFieldset = false;
  }

  function getAndIncTabIndex()
  {
    static $tabIndex = 0;

    return ++$tabIndex;
  }

  /**
   * \brief add a fieldset
   *
   * this method may be called several times.
   * When calling method toHtml(), the fieldset will be automatically closed
   */
  function addFieldset()
  {
    if ($this->openedFieldset)
    {
      $this->blob .= '</fieldset>';
    }

    $this->blob .= '<fieldset>';
    $this->openedFieldset = true;
  }

  /**
   * \brief add a legend
   *
   * \param legend the legend to add to form
   */
  function addLegend($legend)
  {
    $this->blob .= '<legend>'.$legend.'</legend>';
  }

  /**
   * \brief output a drop down
   *
   * \param options list of option can be an array or an associative array
   * \param default_value the default value the control must bear
   * \param name the name the control is attached to
   * \param id optional id
   */
  function addDropDown($options, $default_value, $name, $id = '')
  {
    $ext = '';

    // id
    if ($id != '')
    {
      $ext .= 'id="'.$id.'" ';
    }

    // tabIndex
    $ext .= 'tabindex="'.$this->getAndIncTabIndex().'" ';

    $is_assoc = is_assoc($options);

    $this->blob .= '<select name="'.$name.'" '.$ext.'>';
    foreach ($options as $k=>$v)
    {
      if (!$is_assoc)
      {
        // this is not an associative array => display array value both in the control
        // and in the "value" field of option
        $k = $v;
      }

      $selected = "";
      if ($v == $default_value)
      {
        $selected = 'selected="selected"';
      }

      $this->blob .= '<option '.$selected.' value="'.$v.'">'.$k.'</option>';
    }
    $this->blob .= '</select>';
  }

  /**
   * \brief output a list box
   *
   * \param options list of option can be an array or an associative array
   * \param selected_values array of selected values
   * \param name the name the control is attached to
   * \param size size of the list box
   * \param id optional id
   */
  function addListBox($options, $selected_values, $name, $size, $id = '')
  {
    $ext = '';

    // size
    if ($size != '')
    {
      $ext .= 'size="'.$size.'" ';
    }

    // id
    if ($id != '')
    {
      $ext .= 'id="'.$id.'" ';
    }

    // tabIndex
    $ext .= 'tabindex="'.$this->getAndIncTabIndex().'" ';

    // multiple
    $ext .= 'multiple="true"';

    $is_assoc = is_assoc($options);

    $this->blob .= '<select name="'.$name.'[]" '.$ext.'>';
    foreach ($options as $k=>$v)
    {
      if (!$is_assoc)
      {
        // this is not an associative array => display array value both in the control
        // and in the "value" field of option
        $k = $v;
      }

      $selected = '';
      if (in_array($v, $selected_values))
      {
        $selected = 'selected="selected"';
      }

      $this->blob .= '<option '.$selected.' value="'.$v.'">'.$k.'</option>';
    }
    $this->blob .= '</select>';
  }

  /**
   * \brief add a label
   *
   * \param text text to put in label
   * \param id optional id
   */
  function addLabel($text, $id = '')
  {
    $ext = '';
    if ($id != '')
    {
      $ext = ' for="'.$id.'"';
    }
    $this->blob .= '<label'.$ext.'>'.$text.'</label>';
  }

  /**
   * \brief not to be used directly
   *
   * this method is private and is called from other generateInput() like methods
   *
   * \param type type of html control (text, password, hidden, checkbox, radio, submit ...)
   * \param name name of control
   * \param alt alt tag (optional : maybe empty)
   * \param value initial value (optional : maybe empty)
   * \param id css id (optional : maybe empty)
   * \param ext extra attributes (optional : maybe empty)
   * \param class css class (optional : maybe empty)
   * \return html code
   */
  function generateInputRaw($type, $name, $alt, $value, $id, $ext, $class)
  {
    if (!empty($alt))
    {
      $ext .= 'alt="'.$alt.'" ';
    }
    if (!empty($value))
    {
      $ext .= 'value="'.$value.'" ';
    }
    if (!empty($id))
    {
      $ext .= 'id="'.$id.'" ';
    }
    if (!empty($class))
    {
      $ext .= 'class="'.$class.'" ';
    }
    // tabIndex
    $ext .= 'tabindex="'.$this->getAndIncTabIndex().'" ';

    return '<input type="'.$type.'" name="'.$name.'" '.$ext.'/>';
  }

  /**
   * \brief not to be used directly
   *
   * this method is private and is called from other addInput() like methods
   *
   * \param type type of html control (text, password, hidden, checkbox, radio, submit ...)
   * \param name name of control
   * \param alt alt tag (optional : maybe empty)
   * \param value initial value (optional : maybe empty)
   * \param id css id (optional : maybe empty)
   * \param ext extra attributes (optional : maybe empty)
   * \param class css class (optional : maybe empty)
   */
  function addInputRaw($type, $name, $alt, $value, $id, $ext, $class)
  {
    $this->blob .= $this->generateInputRaw($type, $name, $alt, $value, $id, $ext, $class);
  }

  /**
   * \brief add a text input
   *
   * \param name name of control
   * \param alt alt tag
   * \param value initial value (optional)
   * \param id css id (optional)
   */
  function addInput($name, $alt, $value = '', $id = '')
  {
    $this->addInputRaw('text', $name, $alt, $value, $id, '', '');
  }

  /**
   * \brief add a password input
   *
   * \param name name of control
   * \param value initial value (optional)
   * \param id css id (optional)
   */
  function addPassword($name, $value = '', $id = '')
  {
    $this->addInputRaw('password', $name, '', $value, $id, '', '');
  }

  /**
   * \brief add an hidden field
   *
   * \param name name of control
   * \param value initial value (optional)
   */
  function addHidden($name, $value)
  {
    $this->addInputRaw('hidden', $name, '', $value, '', '', '');
  }

  /**
   * \brief add a checkbox
   *
   * \param name name of control
   * \param checked is checkbox initially checked
   * \param id id of checkbox to add
   */
  function addCheckBox($name, $checked = false, $id = '')
  {
    $ext = '';
    if ($checked)
    {
      $ext = 'checked="checked" ';
    }

    $this->addInputRaw('checkbox', $name, '', '', $id, $ext, '');
  }

  /**
   * \brief generates html to represent a radio button
   *
   * \param name name of control
   * \param value initial value (optional)
   * \param checked is radiobutton initially checked
   * \return html code
   */
  function generateRadioButton($name, $value, $checked = false)
  {
    $ext = '';
    if ($checked)
    {
      $ext = 'checked="checked" ';
    }

    return $this->generateInputRaw('radio', $name, '', $value, '', $ext, '');
  }

  /**
   * \brief add a radio button
   *
   * \param name name of control
   * \param value initial value (optional)
   * \param checked is radiobutton initially checked
   */
  function addRadioButton($name, $value, $checked = false)
  {
    $this->blob .= $this->generateRadioButton($name, $value, $checked);
  }

  /**
   * \brief add a button
   *
   * \param name name of control
   * \param value text button
   * \param class css class
   * \param disabled is button disabled ?
   */
  function addButton($name, $value, $class = '', $disabled = false)
  {
    $ext = '';
    if ($disabled)
    {
      $ext = 'disabled="disabled" ';
    }

    $this->addInputRaw('submit', $name, '', $value, '', $ext, $class);
  }

  /**
   * \brief add a textarea
   *
   * \param name name of control
   * \param rows number of rows
   * \param cols number of columns
   * \param value text
   * \param id id of textarea to add
   */
  function addTextarea($name, $rows, $cols, $value='', $id='')
  {
    // tabIndex
    $ext = 'tabindex="'.$this->getAndIncTabIndex().'" ';
    if (!empty($id))
    {
      $ext .= 'id="'.$id.'" ';
    }

    $this->blob .= '<textarea name="'.$name.'" rows="'.$rows.'" cols="'.$cols.'" '.$ext.'>'.$value.'</textarea>';
  }

  /**
   *  \brief add an upload control
   *
   * \param name name of control
   */
  function addFileUploadButton($name)
  {
    // tabIndex
    $ext = 'tabindex="'.$this->getAndIncTabIndex().'" ';

    $this->addInputRaw('file', $name, '', '', '', '', '');
  }

  /**
   * \brief add raw html
   *
   * \param html raw html to add
   */
  function addHtml($html)
  {
    $this->blob .= $html;
  }

  /**
   * \brief return complete form as html
   *
   * opened field set is closed
   */
  function toHtml()
  {
    if ($this->openedFieldset)
    {
      $this->blob .= '</fieldset>';
    }
    $this->blob .= '</form>';

    return $this->blob;
  }
}

?>
