<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief handles a configuration file with a very simple syntax keyword = value
 */
class ini
{
  var $config;         //!< description of fields, types, default_value
  var $file;           //!< file containing this configuration file
  var $conf = array(); //!< array<string> containing the configuration
  var $file_exists;    //!< boolean to report success to the user

  /**
   * \brief read conf from file
   *
   * the file will be read, and each constants will be exported as php constants
   */
  function ini($file, $config=null)
  {
    if (is_null($config))
    {
      $this->config = & new WiclearIniConfig;
    }
    else
    {
      $this->config = $config;
    }

    $this->file = $file;
    if ($this->readFile())
    {
      $this->file_exists = true;
    }
    else
    {
      $this->file_exists = false;
    }

    $this->completeFromDefault();
  }

  /**
   * \brief add a new constant to current conf
   */
  function addConstant($name, $value)
  {
    $this->conf[$name] = $value;
  }

  /**
   * \brief delete file before saving it
   */
  function forceSaveFile()
  {
    if (@unlink($this->file))
    {
      return $this->saveFile();
    }
    else
    {
      return false;
    }
  }

  /**
   * \brief save this configuration to disk
   */
  function saveFile()
  {
    $h = @fopen($this->file, 'w');
    if (!$h)
    {
      return false;
    }

    // iterate over fields of meta conf
    foreach ($this->config->fields as $field=>$default_value)
    {
      // if field is defined in current conf
      if (isset($this->conf[$field]))
      {
        $value = $this->conf[$field];
        // and different from default value
        if ($value != $default_value)
        {
          // save it
          $line = $field.' = '.$value."\n";
          @fwrite($h, $line);
        }
      }
    }

    @fclose($h);

    return true;
  }

  /**
   * \brief read configuration file
   */
  function readFile()
  {
    if (file_exists($this->file) && is_file($this->file) && is_readable($this->file))
    {
      $content = file($this->file);
      foreach ($content as $line )
      {
        $elem  = split('=', $line);
        $name  = trim($elem[0]);
        unset($elem[0]);
        $value = trim(join('=', $elem));

        $this->conf[$name] = $value;
      }

      return true;
    }

    return false;
  }

  /**
   * \brief dump constants to STDOUT
   */
  function dumpConstants()
  {
    $blob = '<ul>';
    foreach ($this->conf as $name => $value)
    {
      $blob += '<li>'.$name.' = '.$value.'</li>';
    }
    $blob += '</ul>';

    return $blob;
  }

  /**
   * \brief import constants as php constant
   */
  function importConstants()
  {
    if (empty($this->conf))
      return;

    foreach ($this->conf as $name => $value)
    {
      define($name, $value);
    }
  }

  /**
   * \brief read configuration, deleting unknown variables, and creating missing one
   */
  function completeFromDefault()
  {
    // create list of variables to create
    $to_create = array();

    // all variables are to be created by default
    foreach ($this->config->fields as $field=>$field_desc)
    {
      $to_create[$field] = true;
    }

    // loop over read values, completing list of values to delete (unknown in meta conf)
    // and marking already created values
    $to_delete = array();
    foreach ($this->conf as $name=>$value)
    {
      if (isset($to_create[$name]))
      {
        $to_create[$name] = false;
      }
      else
      {
        $to_delete[] = $name;
      }
    }

    // delete unknown fields in meta conf
    foreach ($to_delete as $name)
    {
      unset($this->conf[$name]);
    }

    // create missing fields from meta conf with default values
    foreach ($to_create as $name=>$value)
    {
      if ($value)
      {
        $this->conf[$name] = $this->config->fields[$name];
      }
    }
  }
}
?>
