<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief helper to handle nested set model
 */
class NestedSetModelHelper
{
  /**
   * \brief return left and right value of hierarchy table
   *
   * private method used in this class.
   * the left and right value are the nested set model left and right values
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param id the id we want to retrieve left and right value from
   * \param oid optional id we want to retrieve left and right value from
   *
   * \return array[left, right] if oid == 0 or array[0..1][left, right] if oid != 0
   */
  function getLeftRight(&$wiki, $id, $oid = 0)
  {

    $query = <<<SQL
      SELECT left_n, right_n
      FROM {prefix}content_hierarchy
      WHERE content_fk='$id'
SQL;
    if ($oid != 0)
    {
      $otherQuery = <<<SQL
        UNION
        SELECT left_n, right_n
        FROM {prefix}content_hierarchy
        WHERE content_fk='$oid'
SQL;

      $query .= ' '.$otherQuery;
    }

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    if ($oid == 0)
    {
      return $wiki->con->select1($query);
    }
    else
    {
      return $wiki->con->select($query);
    }
  }

  /**
   * \brief returns a triplet (parent, left, right) from a master node id
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param id database id of a master node
   * \return array[parent, left, right]
   */
  function getNodeHierarchyById(&$wiki, $id)
  {
    $id = $wiki->con->escapeString($id);

    $query = <<<SQL
      SELECT parent_content_fk, left_n, right_n FROM {prefix}content_hierarchy
      WHERE content_fk='$id'
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    return $wiki->con->select1($query);
  }

  /**
   * \brief returns a triplet (parent, left, right) from root node
   *
   * \param wiki the controller so that we have access to configuration/parameters
   *
   * \return array[parent, left, right]
   */
  function getRootNodeHierarchy(&$wiki)
  {
    $query = <<<SQL
      SELECT parent_content_fk, left_n, right_n FROM {prefix}content_hierarchy
      WHERE content_fk=parent_content_fk
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    return $wiki->con->select1($query);
  }

  /**
   * \brief move a node 
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param node_id the id we want to move
   * \param old_parent_node_id current parent node of this page
   * \param left left value of node
   * \param right right value of node
   * \param parent_node_id new parent node of this page
   */
  function moveNode(&$wiki, $node_id, $old_parent_node_id, $left, $right, $parent_node_id)
  {
    // change of parent_id : let's rebuild nested set model
    $lr = NestedSetModelHelper::getLeftRight($wiki, $parent_node_id, $old_parent_node_id);
    $new_left  = $lr[0][0];
    $new_right = $lr[0][1];
    $old_left  = $lr[1][0];
    $old_right = $lr[1][1];

    if ($left < $new_right)
    {
      // right translation
      $query = <<<SQL
        UPDATE {prefix}content_hierarchy
        SET left_n           =IF(content_fk='$node_id', '$new_right'-2,    IF(left_n> '$left',      left_n-2,  left_n)),
            right_n          =IF(content_fk='$node_id', '$new_right'-1,    IF(right_n<'$new_right', right_n-2, right_n)),
            parent_content_fk=IF(content_fk='$node_id', '$parent_node_id', parent_content_fk)
        WHERE left_n BETWEEN '$left' AND '$new_right'
        OR    content_fk='$old_parent_node_id'
SQL;
    }
    else
    if ($left > $new_right)
    {
      // left translation
      $query = <<<SQL
        UPDATE {prefix}content_hierarchy
        SET left_n           =IF(content_fk='$node_id', '$new_right',      IF(left_n> '$new_left', left_n+2,  left_n)),
            right_n          =IF(content_fk='$node_id', '$new_right'+1,    IF(right_n<'$right',    right_n+2, right_n)),
            parent_content_fk=IF(content_fk='$node_id', '$parent_node_id', parent_content_fk)
        WHERE left_n BETWEEN '$new_right' AND '$left'
        OR    content_fk='$parent_node_id'
SQL;
    }

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);
  }

  /**
   * \brief insert a node 
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param id id of newly inserted node
   * \param parent_node_id parent node of newly inserted page
   */
  function insertNode(&$wiki, $id, $parent_node_id)
  {
    $lr = NestedSetModelHelper::getLeftRight($wiki, $parent_node_id);
    $left  = $lr[0];
    $right = $lr[1];

    // update nested set model first
    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET left_n =IF(left_n> '$left', left_n+2,  left_n),
          right_n=IF(right_n>'$left', right_n+2, right_n)
      WHERE right_n>='$right'
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);

    $l = $right;
    $r = $l+1;

    // insert new relationship
    $query = <<<SQL
      INSERT INTO {prefix}content_hierarchy
      SET parent_content_fk='$parent_node_id', content_fk='$id', left_n ='$l', right_n='$r'
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);
  }

  /**
   * \brief delete a node
   *
   * The difference with removeNode is that removeNode modifies parent relationship
   * so that removedNode has no parent and no children, but the 'entry' is not removed
   * deleteNode also removes the entry
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param id id of node to delete
   * \param left  left_n value of node to delete
   * \param right right_n value of node to delete
   * \param $parent_content_fk parent of node being deleted that will be the new parent for the children of deleted node
   */
  function deleteNode(&$wiki, $id, $left, $right, $parent_content_fk)
  {
    NestedSetModelHelper::removeNode($wiki, $id, $left, $right, $parent_content_fk);

    // delete content_hierarchy's content_fk lines
    $query = <<<SQL
      DELETE FROM {prefix}content_hierarchy
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);
  }

  /**
   * \brief remove a node
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param id id of node to remove
   * \param left  left_n value of node to remove
   * \param right right_n value of node to remove
   * \param new_parent parent of node being removed that will be the new parent for the children of removed node
   */
  function removeNode(&$wiki, $id, $left, $right, $new_parent)
  {
    // rebuild nested set model : step1 : modify nodes after right
    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET left_n =IF(left_n> '$right', left_n-2,  left_n),
          right_n=right_n-2
      WHERE right_n>'$right'
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);

    // rebuild nested set model : step2 : modify nodes between left and right
    // also updating content_hierarchy parent_content_fk by attaching childs to the parent of the node we're removing
    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET left_n =IF(left_n> '$left', left_n-1,  left_n),
          right_n=IF(right_n<'$right', right_n-1, right_n),
          parent_content_fk=IF(parent_content_fk='$id', '$new_parent', parent_content_fk)
      WHERE left_n>'$left'
      AND   right_n<'$right'
SQL;

      $query = str_replace('{prefix}', $wiki->prefix, $query);
      $wiki->con->execute($query);
  }

  /**
   * \brief set a new root node
   *
   * \param wiki the controller so that we have access to configuration/parameters
   * \param id id of the new root node
   * \param root_right right value of previous root node
   */
  function setNewRootNode(&$wiki, $id, $root_right)
  {
    // rebuild nested set model : step3 : make room for new root node
    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET left_n=left_n+1,
          right_n=right_n+1
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);

    // rebuild nested set model : step4 : set parent_content_fk of old root node
    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET parent_content_fk='$id'
      WHERE content_fk=parent_content_fk
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);

    // rebuild nested set model : step5 : set left_n/right_n/parent_content_fk of new root node
    $query = <<<SQL
      UPDATE {prefix}content_hierarchy
      SET left_n=0, right_n='$root_right', parent_content_fk=content_fk
      WHERE content_fk='$id'
SQL;

    $query = str_replace('{prefix}', $wiki->prefix, $query);
    $wiki->con->execute($query);
  }
}
?>