<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****
define ('STATE_NONE', 0);
define ('STATE_CONTENT', 1);

/**
 * \brief handles i18n : UI strings translation
 *
 * in our world, there is a global translation object that is used by global function tr()
 */
class translation
{
  var $lang;

  /**
   * \brief creates a new translation object
   *
   * tries to load a translation file from parameter, if no files could be loaded,
   * each string will remain untranslated
   *
   * \param path path to i18n files
   * \param user_lang translation code we want to have (fr, en, es ...)
   * \param default_lang lang to default to if user_lang is not found (fr, en, es ...)
   * \param ext extension of files (.lang)
   */
  function translation($path, $user_lang, $default_lang, $ext)
  {
    $this->lang = array();
    $this->completeFrom($path, $user_lang, $default_lang, $ext);
  }

  /**
   * \brief complete current object with i18n strings read from files
   *
   * tries to load a translation file from parameter, if no files could be loaded,
   * each string will remain untranslated
   *
   * \param path path to i18n files
   * \param user_lang translation code we want to have (fr, en, es ...)
   * \param default_lang lang to default to if user_lang is not found (fr, en, es ...)
   * \param ext extension of files (.lang)
   */
  function completeFrom($path, $user_lang, $default_lang, $ext)
  {
    $file = $path.'/'.$user_lang.$ext;
    if (file_exists($file))
    {
      $this->readFile($file);
    }
    else
    {
      $file = $path.'/'.$default_lang.$ext;
      if (file_exists($file))
      {
        $this->readFile($file);
      }
    }
  }

  /**
   * \brief return translation for a key
   *
   * if no translation exist, then return key
   *
   * \param key the string to translate
   * \return translated string
   */
  function get($key)
  {
    if (array_key_exists($key, $this->lang))
    {
      return $this->lang[$key];
    }
    else
    {
      return $key;
    }
  }

  /**
   * \brief reads a translation file
   *
   * translation files have a very simple BNF.
   * Lines starting with '#' are comments
   * Lines starting with '[' and ending with ']' define a key
   * Lines following a key line are considered translation (thus translation can be multi lines)
   * A translation must be followed by an empty line
   *
   * \param file file to read
   */
  function readFile($file)
  {
    $content = file($file);
    $state = STATE_NONE;

    foreach ($content as $line )
    {
      $line = trim($line);
      $len  = strlen($line);

      if (strpos($line, '#') === 0)
      {
        // if we find a comment, continue
        continue;
      }

      if ($state == STATE_NONE && $len > 0)
      {
        // searching for a key
        if ($line[0] == '[' && $line[$len - 1] == ']')
        {
          $key = substr($line, 1, $len - 2);
          $content = '';
          $state = STATE_CONTENT;
        }
      }
      else
      if ($state == STATE_CONTENT)
      {
        if ($len == 0)
        {
          $this->lang[$key] = $content;
          $state = STATE_NONE;
        }
        else
        {
          $content .= $line;
        }
      }
    }

    if ($state == STATE_CONTENT)
    {
      $this->lang[$key] = $content;
    }
  }

  /**
   * \brief dump content of translation file for debugging purpose
   *
   * The content is dumped direclty on STDOUT with pre tags
   */
  function dumpContent()
  {
    $blob = '<pre>';

    foreach ($this->lang as $key => $content)
    {
      $blob .= $key.":\n".$content."\n\n";
    }
    $blob .= '</pre>';

    return $blob;
  }
}

/**
 * \brief translates a string from global variable translation
 *
 * \param string the string to translate
 * \return translation if it exist, otherwise the string itself
 */
function tr($string)
{
  global $translation;

  if (!isset($translation))
  {
    return $string;
  }
  else
  {
    // the entry can exist but has not been translated yet (will be empty)
    $translated = $translation->get($string);

    if (empty($translated))
      return $string;
    else
      return $translated;
  }
}
?>