<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief utf8 helper functions
 */
class utf8_helper
{
  function is_utf8($str)
  {
    $re = 
    '/^([\x00-\x7f]|'.
    '[\xc2-\xdf][\x80-\xbf]|'.
    '\xe0[\xa0-\xbf][\x80-\xbf]|'.
    '[\xe1-\xec][\x80-\xbf]{2}|'.
    '\xed[\x80-\x9f][\x80-\xbf]|'.
    '[\xee-\xef][\x80-\xbf]{2}|'.
    'f0[\x90-\xbf][\x80-\xbf]{2}|'.
    '[\xf1-\xf3][\x80-\xbf]{3}|'.
    '\xf4[\x80-\x8f][\x80-\xbf]{2})*$/';

    return (preg_match($re, $str) === 1);
  }

  /**
   * \brief returns number of character in an utf8 encoded string
   *
   * \param str the string to check
   * \return number of utf8 characters
   */
  function strlen($str)
  {
    preg_match_all('/./su', $str, $ar);
    return count($ar[0]);
  }

  /**
   * \brief extract part of an utf8 string
   *
   * \param str the string to extract
   * \param start where to begin substring
   * \param end where to end substring (optional)
   *
   * \return utf8 substring
   */
  function substr($str, $start, $end = -1)
  {
    preg_match_all('/./su', $str, $ar);

    if ($end != -1)
    {
        $end = func_get_arg(2);
        return join('', array_slice($ar[0], $start, $end));
    }
    else
    {
        return join('', array_slice($ar[0], $start));
    }
  }

  /**
   * \brief Convert a Latin1 string into a UTF-8 string and translate litigious character.
   * Taken from DotClear
   */
  function latin1ToUTF8($str)
  {
    $conv = array(
      chr(194).chr(128) => chr(226).chr(130).chr(172),
      chr(194).chr(130) => chr(226).chr(128).chr(154),
      chr(194).chr(131) => chr(198).chr(146),
      chr(194).chr(132) => chr(226).chr(128).chr(158),
      chr(194).chr(133) => chr(226).chr(128).chr(166),
      chr(194).chr(134) => chr(226).chr(128).chr(160),
      chr(194).chr(135) => chr(226).chr(128).chr(161),
      chr(194).chr(136) => chr(203).chr(134),
      chr(194).chr(137) => chr(226).chr(128).chr(176),
      chr(194).chr(138) => chr(197).chr(160),
      chr(194).chr(139) => chr(226).chr(128).chr(185),
      chr(194).chr(140) => chr(197).chr(146),
      chr(194).chr(145) => chr(226).chr(128).chr(152),
      chr(194).chr(146) => chr(226).chr(128).chr(153),
      chr(194).chr(147) => chr(226).chr(128).chr(156),
      chr(194).chr(148) => chr(226).chr(128).chr(157),
      chr(194).chr(149) => chr(226).chr(128).chr(162),
      chr(194).chr(150) => chr(226).chr(128).chr(147),
      chr(194).chr(151) => chr(226).chr(128).chr(148),
      chr(194).chr(152) => chr(203).chr(156),
      chr(194).chr(153) => chr(226).chr(132).chr(162),
      chr(194).chr(154) => chr(197).chr(161),
      chr(194).chr(155) => chr(226).chr(128).chr(186),
      chr(194).chr(156) => chr(197).chr(147),
      chr(194).chr(159) => chr(197).chr(184)
    );

    $str = utf8_encode($str);

    return str_replace(array_keys($conv), array_values($conv), $str);
  }

  /**
   * \brief Truncate a UTF-8-encoded string safely to a number of bytes. 
   *
   * If the end position is in the middle of a UTF-8 sequence, it scans backwards 
   * until the beginning of the byte sequence. 
   *
   * Use this function whenever you want to chop off a string at an unsure 
   * location. On the other hand, if you're sure that you're splitting on a 
   * character boundary (e.g. after using strpos() or similar), you can safely use 
   * substr() instead. 
   *
   * code taken from Drupal CMS (www.drupal.org)
   *
   * \param $string 
   *   The string to truncate.
   * \param $len
   *   An upper limit on the returned string length.
   * \param $wordsafe
   *   Flag to truncate at nearest space. Defaults to FALSE.
   *
   * \return
   *   The truncated string.
   */
  function truncate($string, $len, $wordsafe = false, $dots = false)
  {
    $slen = strlen($string);
    if ($slen <= $len)
    {
      return $string;
    }
    if ($wordsafe)
    {
      $end = $len;
      while (($string[--$len] != ' ') && ($len > 0)) {};
      if ($len == 0)
      {
        $len = $end;
      }
    }
    if ((ord($string[$len]) < 0x80) || (ord($string[$len]) >= 0xC0))
    {
      return substr($string, 0, $len) . ($dots ? ' ...' : '');
    }
    while (--$len >= 0 && ord($string[$len]) >= 0x80 && ord($string[$len]) < 0xC0) {};
    return substr($string, 0, $len) . ($dots ? ' ...' : '');
  }
}

?>
