<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \brief the controller
 *
 * this class act has a controller for all datas stored in the wiki.
 * this class is used from all views and all views must use this class.
 * it contains user handling, data handling (nodes, comments), lang handling, group handling
 * ...
 */
class wiki
{
  var $con;     //!< connection to the database
  var $prefix;  //!< prefix table to use when building query

  /**
   * \brief constructs a new instance from a connection object and a prefix
   *
   * \param con the connection object to a database
   * \param prefix prefix used for all tables
   */
  function wiki(&$con, $prefix)
  {
    $this->con = &$con;
    $this->prefix = $prefix;
  }

  /**
   * \brief retrieve all langs available in the wiki
   *
   * \returns a list of lang objects
   */
  function getLangs()
  {
    $query =<<<SQL
      SELECT id, code, description, flag
      FROM   {prefix}lang;
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $langs = array();
    if ($result !== false)
    {
      foreach ($result as $r)
      {
        $langs[] = new lang($r[0], $r[1], $r[2], $r[3]);
      }
    }

    return $langs;
  }

  /**
   * \brief retrieve a lang by id
   *
   * \param id the database id of the lang
   * \returns a lang object
   */
  function getLangById($id)
  {
    $id = $this->con->escapeString($id);

    $query =<<<SQL
      SELECT id, code, description, flag
      FROM   {prefix}lang
      WHERE  id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $r = $this->con->select1($query);

    return new lang($r[0], $r[1], $r[2], $r[3]);
  }

  /**
   * \brief add a new unconfirmed user
   *
   * When a user subscribe, as we want to be sure of its email address, we send back to
   * him an email with an hidden link. The user remain 'unconfirmed' while he has not
   * visited the link provided in the email and cannot log in the system.
   *
   * \param login choosen login
   * \param fname first name
   * \param lname last name
   * \param password password
   * \param email email
   * \param lang_fk choosen lang for content (one available in table lang)
   * \param lang_ui choosen lang for ui (one available in i18n/)
   *
   * \return randomly generated key associated to user or empty if there was an error
   */
  function addUnconfirmedUser($login, $fname, $lname, $password, $email, $lang_fk, $lang_ui)
  {
    $login      = $this->con->escapeString($login);
    $fname      = $this->con->escapeString($fname);
    $lname      = $this->con->escapeString($lname);
    $password   = $this->con->escapeString($password);
    $email      = $this->con->escapeString($email);
    $lang_fk    = $this->con->escapeString($lang_fk);
    $lang_ui    = $this->con->escapeString($lang_ui);
    $user_value = user_value;

    $time = time();
    $key = md5($time.$password.mt_rand());

    $query =<<<SQL
      INSERT INTO {prefix}user
      (login, firstname, lastname, passwd, email, admin, lang_fk, lang_ui, timestamp, temp_key)
      VALUES('$login', '$fname', '$lname', MD5('$password'), '$email', '$user_value', '$lang_fk', '$lang_ui', '$time', '$key');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    if ($this->con->execute($query))
    {
      return $key;
    }
    else
    {
      return;
    }
  }

  /**
   * \brief confirm a user
   *
   * when the user click on the link provided to him in the email that was sent to him when he registered
   * this function is called. The key is a randomly generated key that is passed in the url and must
   * match a value in column temp_key.
   *
   * \param key the temporary key that was assigned to him when he registered
   * \return array[id, password]
   */
  function confirmUser($key)
  {
    $key = $this->con->escapeString($key);

    $query =<<<SQL
      SELECT id, passwd
      FROM {prefix}user
      WHERE temp_key='$key';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    if (!empty($result))
    {
      $query =<<<SQL
          UPDATE {prefix}user
          SET temp_key=NULL
          WHERE temp_key='$key';
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $this->con->execute($query);
    }

    return $result;
  }

  /**
   * \brief does the user exists ?
   *
   * \param login the login to check
   * \return true if user exists, false otherwise
   */
  function userExists($login)
  {
    $login = $this->con->escapeString($login);

    $query =<<<SQL
      SELECT id
      FROM {prefix}user
      WHERE login='$login';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    return !empty($result);
  }

  /**
   * \brief creates an anonymous user object
   */
  function anonymousUser()
  {
    // $id, $login, $password, $admin, $first_name, $last_name, $email, $lang
    return new user(user_anonymous_id, 'anonymous', '', user_value, '', '', '',
      wc_default_lang, wc_default_lang, 0
    );
  }

  /**
   * \brief returns a user from its id and md5 encoded password
   *
   * the cookie sent to the browser contains two fields : id and encoded password
   * this enables to quickly find a user from the cookie
   *
   * \param id database id of user
   * \param md5password md5 encoded password
   *
   * \return user object representing found user, or anonymous user if not found
   */
  function getUser($id, $md5password)
  {
    $id          = $this->con->escapeString($id);
    $md5password = $this->con->escapeString($md5password);

    $query =<<<SQL
      SELECT {prefix}user.login, {prefix}user.admin, {prefix}user.firstname, {prefix}user.lastname,
             {prefix}user.email, {prefix}user.lang_ui, {prefix}user.timestamp,
             {prefix}lang.code
      FROM   {prefix}user
      JOIN   {prefix}lang on {prefix}lang.id={prefix}user.lang_fk
      WHERE  {prefix}user.id='$id'
      AND    {prefix}user.passwd='$md5password'
      AND    {prefix}user.temp_key IS NULL
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    if (empty($result))
    {
      return $this->anonymousUser();
    }
    else
    {
      // $id, $login, $password, $admin, $first_name, $last_name, $email, $lang, $ui_lang, $timestamp
      $user = new user($id, $result[0], $md5password, $result[1], $result[2], $result[3], $result[4], $result[7], $result[5], $result[6]);

      $query =<<<SQL
        SELECT content_fk
        FROM   {prefix}content_user_email
        WHERE  user_fk='$id';
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $result = $this->con->select($query);

      $node_ids = array();
      foreach ($result as $r)
      {
        $node_ids[] = $r[0];
      }

      $user->setWatches($node_ids);

      return $user;
    }
  }

  /**
   * \brief find a user by login and plain text password
   *
   * When the user register in, login and password are sent as plain text.
   * we then try to find the user in the database and returns it back
   *
   * \param login login of user
   * \param password password of user
   */
  function findUser($login, $password)
  {
    $login    = $this->con->escapeString($login);
    $password = $this->con->escapeString($password);

    $query =<<<SQL
      SELECT {prefix}user.id, {prefix}user.passwd, {prefix}user.admin, {prefix}user.firstname,
             {prefix}user.lastname, {prefix}user.email, {prefix}user.lang_ui, {prefix}user.timestamp,
             {prefix}lang.code
      FROM   {prefix}user
      JOIN   {prefix}lang on {prefix}lang.id={prefix}user.lang_fk
      WHERE  {prefix}user.login='$login'
      AND    {prefix}user.passwd=MD5('$password')
      AND    {prefix}user.temp_key IS NULL
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    if (empty($result))
    {
      return $this->anonymousUser();
    }
    else
    {
      // $id, $login, $password, $admin, $first_name, $last_name, $email, $lang, $timestamp
      return new user($result[0], $login, $result[1], $result[2], $result[3], $result[4], $result[5], $result[8], $result[6], $result[7]);
    }
  }

  /**
   * \brief update a user
   *
   * update a user from the user preference dialog
   *
   * \param id database id of user
   * \param login login of user
   * \param fname first name of user
   * \param lname last name of user
   * \param password password of user (can be empty if user do not want to update it)
   * \param email email of user
   * \param lang_fk foreign key to lang table (for content)
   * \param lang_ui lang code corresponding to an i18n file (for ui)
   */
  function updateUser($id, $login, $fname, $lname, $password, $email, $lang_fk, $lang_ui)
  {
    $id       = $this->con->escapeString($id);
    $login    = $this->con->escapeString($login);
    $fname    = $this->con->escapeString($fname);
    $lname    = $this->con->escapeString($lname);
    $password = $this->con->escapeString($password);
    $email    = $this->con->escapeString($email);
    $lang_fk  = $this->con->escapeString($lang_fk);
    $lang_ui  = $this->con->escapeString($lang_ui);

    $query =<<<SQL
      UPDATE {prefix}user
      SET    login='$login', firstname='$fname', lastname='$lname', email='$email', lang_fk='$lang_fk', lang_ui='$lang_ui'
SQL;

    if (!empty($password))
    {
      $query .= ", passwd=MD5('$password')";
    }

    $query .= "WHERE  id='$id';";

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->execute($query);
  }

  /**
   * \brief get users
   *
   * get all users (including anonymous)
   *
   * \return array<id, login>
  */
  function getUsers()
  {
    // get rights
    $query = <<<SQL
      SELECT id, login
      FROM   {prefix}user
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief returns emails of admins
   *
   * \return list of emails
   */
  function getAdminEmails()
  {
    $admin_value = admin_value;

    $query =<<<SQL
      SELECT email
      FROM   {prefix}user
      WHERE  admin='$admin_value'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $emails = array();
    foreach ($result as $r)
    {
      $emails[] = $r[0];
    }

    return $emails;
  }

  /**
   * \brief returns all available groups
   *
   * \return list of group objects
   */
  function getGroups()
  {
    // the outer join means we want result from the left part of the join (table groups)
    // even if there is no result in the right part (table user_group)
    // in that case, user_group values will appear as NULL
    $query = <<<SQL
     SELECT DISTINCT {prefix}groups.id, {prefix}groups.name, {prefix}user.id, {prefix}user.login
     FROM      {prefix}groups
     LEFT JOIN {prefix}user_group ON {prefix}groups.id={prefix}user_group.group_fk
     LEFT JOIN {prefix}user       ON {prefix}user.id={prefix}user_group.user_fk
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $groups = array();
    foreach ($result as $r)
    {
      if (!array_key_exists($r[0], $groups))
      {
        $groups[$r[0]] = new group($r[0], $r[1]);
      }

      if ($r[2] != 0)
      {
        $groups[$r[0]]->addUser($r[2], $r[3]);
      }
    }

    return $groups;
  }

  /**
   * \brief returns an ordered list of parent nodes from a related node and a lang id
   *
   * Look into hierarchy table for each node that are parent from the nested set model.
   * Once all possible parent nodes have been retrieved, keep only the one that are in
   * the desired lang if it exist, or default to node in site default lang
   *
   * \param related_id master node in site default lang
   * \param lang_id the desired lang of parent nodes
   *
   * \return array<id, title, alternate_title>
   */
  function getNodeParents($related_id, $lang_id)
  {
    $query = <<<SQL
      SELECT {prefix}content.id, {prefix}content.title, {prefix}content.alternate_title,
             {prefix}content_lang.related_content_fk, {prefix}content.lang_fk
      FROM   {prefix}content
      JOIN   {prefix}content_hierarchy as node on node.content_fk='$related_id'
      JOIN   {prefix}content_lang on {prefix}content_lang.content_fk={prefix}content.id
      JOIN   {prefix}content_hierarchy as pnode on pnode.content_fk={prefix}content.id
      WHERE  pnode.left_n < node.left_n
      AND    pnode.right_n > node.right_n
      ORDER BY pnode.left_n ASC
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    // because of the OR statement we can have duplicated results for one node,
    // one result with the desired lang and one result with the default lang
    $relatedToLang = array();
    foreach ($result as $r)
    {
      $related = $r[3];
      $lang    = $r[4];

      // if the node was not known, or if the previous lang stored is not the wanted one
      if (!isset($relatedToLang[$related]) || $relatedToLang[$related] != $lang_id)
      {
        $relatedToLang[$related] = $lang;
      }
    }

    $parents = array();
    foreach ($result as $r)
    {
      $related = $r[3];
      $lang    = $r[4];

      // filter on lang then with default lang
      if ($relatedToLang[$related] == $lang)
      {
        $parents[] = array($r[0], $r[1], $r[2]);
      }
    }

    return $parents;
  }

  function sortByAlternateTitle($link1, $link2)
  {
    $title1 = (empty($link1[2]) ? $link1[1] : $link1[2]);
    $title2 = (empty($link2[2]) ? $link2[1] : $link2[2]);

    return strcasecmp($title1, $title2);
  }

  function displaySqlResult($msg, $query)
  {
    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->setDebug(true);
    $result = $this->con->select($query);
    $this->con->setDebug(false);

    echo '<pre>'.$msg.' BEGIN&gt;&gt;&gt;</pre>';
    foreach ($result as $r)
    {
      echo '<pre>'.print_r($r).'</pre>';
    }
    echo '<pre>'.$msg.' END&lt;&lt;&lt;</pre>';
  }

  /**
   * \brief returns the list of sub nodes from a node
   *
   * Look into the hierarchy table for all nodes that have a parent
   * node equals to related_id. Once they have been retrieved, keep only
   * the one in desired lang if they exist, or default to node in site default lang
   *
   * \param related_id master node in site default lang
   * \param lang_id the desired lang of parent nodes
   * \param user_id filter nodes by returning only nodes that are readable by user identified by user_id
   *
   * \return array<id, title, alternate_title>
   */
  function getNodeSubLinks($related_id, $lang_id, $user_id=-1)
  {
    // get sub links for this node
    $query = <<<SQL
      SELECT DISTINCT
             {prefix}content.id, {prefix}content.title, {prefix}content.alternate_title,
             {prefix}content_lang.related_content_fk, {prefix}content.lang_fk,
             {prefix}lang.code
      FROM   {prefix}content
      JOIN   {prefix}content_hierarchy on {prefix}content_hierarchy.parent_content_fk='$related_id'
      JOIN   {prefix}content_lang on {prefix}content_hierarchy.content_fk={prefix}content_lang.related_content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
SQL;

    $query .= "\n".
      'WHERE  {prefix}content_lang.content_fk={prefix}content.id'."\n".
      "AND    {prefix}content_lang.related_content_fk!='$related_id'\n";

    if ($user_id != -1)
    {
      $acl_read = acl_read;
      $anonymous_id = user_anonymous_id;

      // create a subquery that returns all nodes containing a read acl
      $subQuery1 = <<<SQL
        SELECT {prefix}content_group.content_fk
        FROM {prefix}content_group
        WHERE {prefix}content_group.acl_type='$acl_read'
SQL;

      //$this->displaySqlResult('1', $subQuery1);

      // create a subquery that returns all nodes where user has explicit read access from groups
      $subQuery2 = <<<SQL
        SELECT {prefix}content_group.content_fk
        FROM   {prefix}content_group
        JOIN   {prefix}user_group on {prefix}user_group.group_fk={prefix}content_group.group_fk
        WHERE  ({prefix}user_group.user_fk='$user_id' OR {prefix}user_group.user_fk='$anonymous_id')
        AND    {prefix}content_group.acl_type='$acl_read'
SQL;

      //$this->displaySqlResult('2', $subQuery2);

      // create a subquery that returns all nodes where user has explicit read access from users
      $subQuery3 = <<<SQL
        SELECT {prefix}content_group.content_fk
        FROM   {prefix}content_group
        WHERE  ({prefix}content_group.user_fk='$user_id' OR {prefix}content_group.user_fk='$anonymous_id')
        AND    {prefix}content_group.acl_type='$acl_read'
SQL;

      //$this->displaySqlResult('3', $subQuery3);

      $query .=
        'AND ('.
          '{prefix}content.id NOT IN ('."\n".$subQuery1."\n".') OR '.
          '{prefix}content.id IN ('."\n".$subQuery2."\n".') OR '.
          '{prefix}content.id IN ('."\n".$subQuery3."\n".')'.
        ') ';
    }

    $query .=
      'ORDER BY {prefix}content.title ASC';

    //$this->displaySqlResult('final', $query);

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    // because of the OR statement we can have duplicated results for one node,
    // one result with the desired lang and one result with the default lang
    $relatedToLang = array();
    foreach ($result as $r)
    {
      $related = $r[3];
      $lang    = $r[4];

      // if the node was not known, or if the previous lang stored is not the wanted one
      if (!isset($relatedToLang[$related]) || $relatedToLang[$related] != $lang_id)
      {
        $relatedToLang[$related] = $lang;
      }
    }

    $links = array();
    foreach ($result as $r)
    {
      $related = $r[3];
      $lang    = $r[4];

      // filter on lang then with default lang
      if ($relatedToLang[$related] == $lang)
      {
        $links[] = array($r[0], $r[1], $r[2], $r[5]);
      }
    }

    usort($links, array('wiki', 'sortByAlternateTitle'));

    return $links;
  }

  /**
   * \brief returns the list of nodes at a level
   *
   * A little explanation needs to be done here...
   * Let's say you retrieve the parent or the grand parent of the Nth parent of a node
   * and you want to have all other nodes at same N level in the hierarchy. That's exactly what this
   * function is doing.
   * An example :
   * <table>
   * <tr><td>Level0</td><td colspan="8">Root</td></tr>
   * <tr><td>Level1</td><td colspan="4">C1.1</td><td colspan="4">C1.2</td></tr>
   * <tr><td>Level2</td><td>GC1.1.1</td><td></td><td>GC1.1.2</td><td></td><td>GC1.2.1</td><td>GC1.2.2</td><td>GC1.2.3</td><td></td></tr>
   * <tr><td>Level3</td><td></td><td>GGC1.1.2.1</td><td></td><td>GGC1.1.2.2</td><td></td><td>GGC1.2.3.1</td><td>GGC1.2.3.2</td><td>GC1.2.3.2</td></tr>
   * </table>
   *
   * Now you're displaying node GC1.2.3.2 and you want to display in a tree view for example
   * all nodes at level 2 (GC1.2.1 GC1.2.2 GC1.2.3)
   * Then you call getNodesAtUpperLevel(GC1.2.3.2, lang_id, 2)
   *
   * \param related_content_id master node in site default lang
   * \param lang_id the desired lang of parent nodes
   * \param level desired level for nodes
   *
   * \return array<id, title, alternate_title>
   */
  function getNodesAtUpperLevel($related_content_id, $lang_id, $level)
  {
    $parents = $this->getNodeParents($related_content_id, $lang_id);

    if ($level <= 0) // user wants root node (which is unique)
    {
      $root = array();
      $root[] = array($parents[0][0], $parents[0][1], $parents[0][2]);
      return $root;
    }

    $parent_id_of_nodes_at_level = $parents[$level - 1][0];
    return $this->getNodeSubLinks($parent_id_of_nodes_at_level, $lang_id);
  }

  /**
  * \brief get comments associated to a node
  *
  * \param node_id database if of node
  * \return list of comment objects
  */
  function getNodeComments($node_id)
  {
    // get comments for this node
    $query = <<<SQL
      SELECT {prefix}comment.id, {prefix}comment.title, {prefix}comment.content, {prefix}comment.timestamp,
             {prefix}comment.name, {prefix}comment.email,
             {prefix}user.id, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email,
             {prefix}comment.ip
      FROM   {prefix}comment
      JOIN   {prefix}user on {prefix}user.id={prefix}comment.user_fk
      WHERE  {prefix}comment.content_fk='$node_id'
      ORDER BY {prefix}comment.timestamp ASC
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $comments = array();
    $nbResult = sizeof($result);
    for ($i = 0; $i < $nbResult; ++$i)
    {
      $comment = new comment(
        $result[$i][0], $result[$i][1], $result[$i][2], $result[$i][3],
        $result[$i][4], $result[$i][5],
        $result[$i][6], $result[$i][7], $result[$i][8], $result[$i][9],
        $result[$i][10]
      );
      array_push($comments, $comment);
    }

    return $comments;
  }

  /**
   * \brief returns list of group having write access to node
   *
   * \param node_id id of node
   *
   * \return list of group objects
   */
  function getNodeGroups($node_id)
  {
    // get acls on groups and groups composition for this node
    $query1 =<<<SQL
      SELECT {prefix}groups.id, {prefix}groups.name, {prefix}content_group.acl_type, {prefix}user.id, {prefix}user.login
      FROM   {prefix}content_group
      -- get groups compositions defined on acl
      INNER JOIN {prefix}groups        ON {prefix}groups.id={prefix}content_group.group_fk
      LEFT JOIN {prefix}user_group     ON {prefix}user_group.group_fk={prefix}groups.id
      LEFT JOIN {prefix}user           ON {prefix}user.id={prefix}user_group.user_fk
      -- for all acls of this node
      WHERE {prefix}content_group.content_fk='$node_id' AND {prefix}content_group.group_fk IS NOT NULL
SQL;

    // get acls on single users (in special group 0)
    $query2 =<<<SQL
      SELECT {prefix}groups.id, {prefix}groups.name, {prefix}content_group.acl_type, {prefix}user.id, {prefix}user.login
      FROM   {prefix}content_group
      -- get single users defined on acl
      INNER JOIN {prefix}user         ON {prefix}user.id={prefix}content_group.user_fk
      LEFT JOIN {prefix}groups        ON {prefix}groups.id={prefix}content_group.group_fk
      -- for all acls of this node
      WHERE {prefix}content_group.content_fk='$node_id' AND {prefix}content_group.user_fk IS NOT NULL
SQL;

    $query = '('.$query1.")\n UNION (\n".$query2.')';
    //$query = $query1;
    //$query = $query2;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $groupsByAclType = array(); // array[acl_type=>$groups]
    $groupsByAclType[acl_write] = array();
    $groupsByAclType[acl_read] = array();
    $groupsByAclType[acl_comment] = array();

    foreach ($result as $r)
    {
      $gid      = $r[0];
      $gname    = $r[1];
      $acl_type = $r[2];
      $uid     = $r[3];
      $ulogin  = $r[4];

      if (!array_key_exists($acl_type, $groupsByAclType)) // for plugin defined acl_types
      {
        $groupsByAclType[$acl_type] = array();
      }

      $groups = &$groupsByAclType[$acl_type];

      if (empty($gid))
      {
        $gid = 0;
        $gname = '';
      }

      if (!array_key_exists($gid, $groups))
      {
        $groups[$gid] = new group($gid, $gname);
      }

      if ($uid != 0)
      {
        $groups[$gid]->addUser($uid, $ulogin);
      }
    }

    return $groupsByAclType;
  }

  /**
   * \brief return list of lang that translates a given node
   *
   * \param related_id master node
   *
   * \return map<lang_id, lang_code>
   */
  function getNodeLangs($related_id)
  {
    $query = <<<SQL
      SELECT {prefix}lang.id, {prefix}lang.code
      FROM   {prefix}lang
      JOIN   {prefix}content on {prefix}content.lang_fk={prefix}lang.id
      JOIN   {prefix}content_lang on {prefix}content.id={prefix}content_lang.content_fk
      WHERE  {prefix}content_lang.related_content_fk='$related_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $langs = array();
    $nbResult = sizeof($result);
    for ($i = 0; $i < $nbResult; ++$i)
    {
      $langs[$result[$i][0]] = $result[$i][1];
    }

    return $langs;
  }

  /**
   * \brief set related info of a node
   *
   * private method used in this class.
   * it will set fields parents, sublinks, comments, groups, langs
   *
   * \param node
   */
  function setRelatedNodeInfo_($node)
  {
    // groups set on this node for ACLs
    $groupsByAclType = $this->getNodeGroups($node->id);
    $node->setGroups($groupsByAclType);

    // translated versions
    $langs = $this->getNodeLangs($node->related_content_id);
    $node->setLangs($langs);

    return $node;
  }

  /**
   * \brief private method called from getNodeByIdAndLang or getNodeByTitleAndLang
   *
   * composition of result is not documented, look inside caller methods
   * The purpose is to filter out the most relevant node in all nodes contained in $result
   * according to the the desired lang. The fallback is the main lang.
   *
   * \return only one result
   */
  function filterNodeResult_($result, $lang)
  {
    $nbResult = sizeof($result);
    if ($nbResult == 1)
    {
      return $result[0];
    }

    // result contain all nodes related to node_id (=> equals all translation)
    // keep only node in site default lang and asked lang
    for ($i = 0; $i < $nbResult; ++$i)
    {
      if ($result[$i][9] == $lang)
      {
        $langResult = $result[$i];
      }
      else
      if ($result[$i][0] == $result[$i][12])
      {
        $defaultResult = $result[$i];
      }
    }

    if (!empty($langResult))
    {
      $result = $langResult;
    }
    else
    if (!empty($defaultResult))
    {
      $result = $defaultResult;
    }
    else
    {
      return;
    }

    return $result;
  }

  /**
   * \brief retrieve root node in required lang code
   *
   * if node does not exist in required lang, default to site default lang
   *
   * \param lang lang code
   *
   * \return complete node
   */
  function getRootNodeInLang($lang)
  {
    // get node by id
    $lang    = $this->con->escapeString($lang);

    // OK, it gets complicated, a little explanation here.
    // we set related_content_lang as the row which is related to the id of the node we want to get
    // we then say the result for the row content_lang must have its column related_content_fk
    // that match the one of the row related_content_lang
    // this gives us all the translations related to the same node
    // then we just add we wants either the lang the user has asked or the default node
    $query = <<<SQL
      SELECT {prefix}content.id, {prefix}content.title, {prefix}content.alternate_title, {prefix}content.content, {prefix}content.commit_msg, {prefix}content.minor_major, {prefix}content.ip, {prefix}content.timestamp,
             {prefix}lang.id, {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_hierarchy.parent_content_fk, {prefix}content_hierarchy.left_n, {prefix}content_hierarchy.right_n,
             {prefix}content_lang.related_content_fk,
             {prefix}user.id, {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email
      FROM   {prefix}content
      JOIN   {prefix}content_hierarchy on {prefix}content_hierarchy.content_fk={prefix}content_hierarchy.parent_content_fk
      JOIN   {prefix}content_lang as related_content_lang on related_content_lang.content_fk={prefix}content_hierarchy.content_fk
      JOIN   {prefix}lang on {prefix}content.lang_fk={prefix}lang.id
      JOIN   {prefix}user on {prefix}content.user_fk={prefix}user.id
      JOIN   {prefix}content_lang on {prefix}content_lang.related_content_fk=related_content_lang.related_content_fk
      WHERE  {prefix}content.id={prefix}content_lang.content_fk
      AND    (
                {prefix}lang.code='$lang'
             OR {prefix}content_lang.content_fk={prefix}content_lang.related_content_fk
             )
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $result = $this->filterNodeResult_($result, $lang);
    if (empty($result))
    {
      return;
    }

    // id, code, description, flag
    $lang = new lang($result[8], $result[9], $result[10], $result[11]);

    // $id, $title, $alternate_title, $content, $commit_msg, $major_minor, $ip, $timestamp, $lang,
    // $parent_id, $left, $right, $related_content
    // $user_id, $login, $first_name, $last_name, $email
    $node = new node(
      $result[0],  $result[1],  $result[2],  $result[3],  $result[4],  $result[5], $result[6], $result[7], $lang,
      $result[12], $result[13], $result[14], $result[15],
      $result[16], $result[17], $result[18], $result[19], $result[20]
    );

    return $this->setRelatedNodeInfo_($node);
  }

  /**
   * \brief retrieve a node from its id and its lang code
   *
   * if node does not exist in required lang, default to site default lang
   *
   * \param node_id database id of lang
   * \param lang lang code
   *
   * \return complete node
   */
  function getNodeByIdAndLang($node_id, $lang)
  {
    // get node by id
    $node_id = $this->con->escapeString($node_id);
    $lang    = $this->con->escapeString($lang);

    // OK, it gets complicated, a little explanation here.
    // we set related_content_lang as the row which is related to the id of the node we want to get
    // we then say the result for the row content_lang must have its column related_content_fk
    // that match the one of the row related_content_lang
    // this gives us all the translations related to the same node
    // then we just add we wants either the lang the user has asked or the default node
    $query = <<<SQL
      SELECT {prefix}content.id, {prefix}content.title, {prefix}content.alternate_title, {prefix}content.content, {prefix}content.commit_msg, {prefix}content.minor_major, {prefix}content.ip, {prefix}content.timestamp,
             {prefix}lang.id, {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_hierarchy.parent_content_fk, {prefix}content_hierarchy.left_n, {prefix}content_hierarchy.right_n,
             {prefix}content_lang.related_content_fk,
             {prefix}user.id, {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email
      FROM   {prefix}content
      JOIN   {prefix}content_lang as related_content_lang on related_content_lang.content_fk='$node_id'
      JOIN   {prefix}lang on {prefix}content.lang_fk={prefix}lang.id
      JOIN   {prefix}user on {prefix}content.user_fk={prefix}user.id
      JOIN   {prefix}content_hierarchy on {prefix}content_hierarchy.content_fk=related_content_lang.related_content_fk
      JOIN   {prefix}content_lang on {prefix}content_lang.related_content_fk=related_content_lang.related_content_fk
      WHERE  {prefix}content.id={prefix}content_lang.content_fk
      AND    (
                {prefix}lang.code='$lang'
             OR {prefix}content_lang.content_fk={prefix}content_lang.related_content_fk
             )
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $result = $this->filterNodeResult_($result, $lang);
    if (empty($result))
    {
      return;
    }

    // id, code, description, flag
    $lang = new lang($result[8], $result[9], $result[10], $result[11]);

    // $id, $title, $alternate_title, $content, $commit_msg, $minor_major, $ip, $timestamp, $lang,
    // $parent_id, $left, $right, $related_content
    // $user_id, $login, $first_name, $last_name, $email
    $node = new node(
      $result[0],  $result[1],  $result[2],  $result[3],  $result[4], $result[5], $result[6], $result[7], $lang,
      $result[12], $result[13], $result[14], $result[15],
      $result[16], $result[17], $result[18], $result[19], $result[20]
    );

    return $this->setRelatedNodeInfo_($node);
  }

  /**
   * \brief retrieve a node from its title and its lang code
   *
   * if node does not exist in required lang, default to site default lang
   *
   * \param title title of node
   * \param lang lang code
   *
   * \return complete node
   */
  function getNodeByTitleAndLang($title, $lang)
  {
    // get node by title
    $title   = $this->con->escapeString($title);
    $lang    = $this->con->escapeString($lang);

    // Same kind of reasoning here than method getNodeByIdAndLang except that we don't have directly
    // the good id so we have first to find the rows that match title (related_content)
    $query = <<<SQL
      SELECT {prefix}content.id, {prefix}content.title, {prefix}content.alternate_title, {prefix}content.content, {prefix}content.commit_msg, {prefix}content.minor_major, {prefix}content.ip, {prefix}content.timestamp,
             {prefix}lang.id, {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_hierarchy.parent_content_fk, {prefix}content_hierarchy.left_n, {prefix}content_hierarchy.right_n,
             {prefix}content_lang.related_content_fk,
             {prefix}user.id, {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email
      FROM   {prefix}content
      JOIN   {prefix}content as related_content on related_content.title='$title'
      JOIN   {prefix}lang on {prefix}content.lang_fk={prefix}lang.id
      JOIN   {prefix}user on {prefix}content.user_fk={prefix}user.id
      JOIN   {prefix}content_lang as related_content_lang on related_content_lang.content_fk=related_content.id
      JOIN   {prefix}content_hierarchy on {prefix}content_hierarchy.content_fk=related_content_lang.related_content_fk
      JOIN   {prefix}content_lang on {prefix}content_lang.related_content_fk=related_content_lang.related_content_fk
      WHERE  {prefix}content.id={prefix}content_lang.content_fk
      AND    (
                {prefix}lang.code='$lang'
             OR {prefix}content_lang.content_fk={prefix}content_lang.related_content_fk
             )
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $result = $this->filterNodeResult_($result, $lang);
    if (empty($result))
    {
      return;
    }

    $lang = new lang($result[8], $result[9], $result[10], $result[11]);

    // $id, $title, $alternate_title, $content, $commit_msg, $minor_major, $ip, $timestamp, $lang,
    // $parent_id, $left, $right, $related_content
    // $user_id, $login, $first_name, $last_name, $email
    $node = new node(
      $result[0],  $result[1],  $result[2],  $result[3],  $result[4], $result[5], $result[6], $result[7], $lang,
      $result[12], $result[13], $result[14], $result[15],
      $result[16], $result[17], $result[18], $result[19], $result[20]
    );

    return $this->setRelatedNodeInfo_($node);
  }

  /**
   * \brief returns the list of ALL direct and indirect sub nodes of node
   *
   * private method used in this class.
   *
   * \param id database id of node
   *
   * \return list of id
   */
  function getSubNodesId_($id)
  {
    $query = <<<SQL
      SELECT {prefix}content.id
      FROM  {prefix}content
      JOIN  {prefix}content as node on node.id='$id'
      JOIN  {prefix}content_lang as related_node on related_node.content_fk=node.id
      JOIN  {prefix}content_hierarchy as parent on parent.content_fk=related_node.related_content_fk
      JOIN  {prefix}content_hierarchy as subnodes on subnodes.left_n BETWEEN parent.left_n AND parent.right_n
      JOIN  {prefix}content_lang on {prefix}content_lang.related_content_fk=subnodes.content_fk
      WHERE {prefix}content.id={prefix}content_lang.content_fk
      AND   {prefix}content.lang_fk=node.lang_fk
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $r = $this->con->select($query);

    $nbResult = sizeof($r);
    for ($i = 0; $i < $nbResult; ++$i)
    {
      $r[$i] = $r[$i][0];
    }

    return $r;
  }

  /**
   * \brief look if a given node exists
   *
   * the lookup does not default to site default lang if not found.
   * => the match is exact
   *
   * \param title title of node to look for
   * \param lang_id foreign key to table lang
   *
   * \return true if content exist, false otherwise
   */
  function contentExists($title, $lang_id)
  {
    // get node by title
    $title   = $this->con->escapeString($title);
    $lang_id = $this->con->escapeString($lang_id);

    $query = <<<SQL
      SELECT id
      FROM   {prefix}content
      WHERE  {prefix}content.title='$title'
      AND    {prefix}content.lang_fk='$lang_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    return !empty($result);
  }

  /**
   * \brief look if a given node exists
   *
   * \param node_id id of node to look for
   *
   * \return true if content exist, false otherwise
   */
  function contentExistsById($node_id)
  {
    $node_id = $this->con->escapeString($node_id);

    $query = <<<SQL
      SELECT id
      FROM   {prefix}content
      WHERE  {prefix}content.id='$node_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    return !empty($result);
  }

  /**
   * \brief look if a given node exists and if this node is a master node
   *
   * \param node_id id of node to look for
   *
   * \return true if content exist and is a master node, false otherwise
   */
  function isMasterNode($node_id)
  {
    $node_id = $this->con->escapeString($node_id);

    $query = <<<SQL
      SELECT {prefix}content.id
      FROM   {prefix}content
      JOIN   {prefix}content_lang on {prefix}content_lang.content_fk={prefix}content.id
      WHERE  {prefix}content.id='$node_id'
      AND    {prefix}content_lang.content_fk={prefix}content_lang.related_content_fk
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    return !empty($result);
  }

  /**
   * \brief edit an existing content
   *
   * this is the heart of the wiki : it manages content updating
   * AND hierarchy updating by updating the nested set model
   *
   * \param node_id database if of node
   * \param old_parent_node_id the old parent id before the content was edited
   * \param parent_node_id new parent id (may be equals to old_parent_node_id)
   * \param title new title
   * \param alternate_title new alternate_title
   * \param content new content
   * \param commit_msg commit message
   * \param minor_major is it a major or a minor changement ?
   * \param related_content_id related_content_id !!should not be changed!!
   * \param user user object that modified this node
   * \param left left value in nested set model !!should not be changed!!
   * \param right right value in nested set model !!should not be changed!!
   */
  function editContent(
    $node_id, $old_parent_node_id, $parent_node_id, $title, $alternate_title, $content,
    $commit_msg, $minor_major,
    $related_content_id,
    $user,
    $left, $right
  )
  {
    $node_id            = $this->con->escapeString($node_id);
    $parent_node_id     = $this->con->escapeString($parent_node_id);
    $title              = $this->con->escapeString($title);
    $alternate_title    = $this->con->escapeString($alternate_title);
    $content            = $this->con->escapeString($content);
    $commit_msg         = $this->con->escapeString($commit_msg);
    $minor_major        = $this->con->escapeString($minor_major);
    $related_content_id = $this->con->escapeString($related_content_id);

    $ip = implode(',', auth::getIPs());
    $time = time();

    $this->con->execute('START TRANSACTION');

    $needGlobalCleaning = false;
    if ($related_content_id == $node_id)
    {
      // only allow edition of parent_node_id on master node

      if ($old_parent_node_id != $parent_node_id)
      {
        NestedSetModelHelper::moveNode($this, $node_id, $old_parent_node_id, $left, $right, $parent_node_id);
        $needGlobalCleaning = true;
      }
    }

    // and finally update content of node as well
    $query = <<<SQL
      UPDATE {prefix}content
      SET    title='$title', alternate_title='$alternate_title', content='$content',
             commit_msg='$commit_msg', minor_major ='$minor_major', timestamp='$time',
             ip='$ip', user_fk='$user->id'
      WHERE  {prefix}content.id='$node_id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->execute($query);

    $this->con->execute('COMMIT');

    global $cacheLite;
    if ($needGlobalCleaning)
    {
      $cacheLite->wcCleanAll();
    }
    else
    {
      $cacheLite->wcClean();
    }

    return $result;
  }

  /**
   * \brief add a new node to the wiki
   *
   * create a new node in the wiki. If this node is a master node (a node in site default lang)
   * then create an entry in table hierarchy and in table content_lang
   *
   * \param parent_node_id database id of parent node
   * \param title title of node
   * \param alternate_title alternate title of node
   * \param content content of node
   * \param commit_msg commit message
   * \param minor_major is it a major or a minor changement ?
   * \param lang_id database id of lang
   * \param related_content_id database id of related content (can be zero if this node is a master node)
   * \param user user object that is creating this node
   */
  function addContent($parent_node_id, $title, $alternate_title, $content, $commit_msg, $minor_major, $lang_id, $related_content_id, $user)
  {
    $parent_node_id     = $this->con->escapeString($parent_node_id);
    $title              = $this->con->escapeString($title);
    $alternate_title    = $this->con->escapeString($alternate_title);
    $content            = $this->con->escapeString($content);
    $commit_msg         = $this->con->escapeString($commit_msg);
    $minor_major        = $this->con->escapeString($minor_major);
    $lang_id            = $this->con->escapeString($lang_id);
    $related_content_id = $this->con->escapeString($related_content_id);

    $ip = implode(',', auth::getIPs());
    $time = time();

    $this->con->execute('START TRANSACTION');

    // insert new node
    $query = <<<SQL
      INSERT INTO {prefix}content
      SET title='$title',
          alternate_title='$alternate_title',
          content='$content',
          commit_msg='$commit_msg',
          minor_major='$minor_major',
          timestamp='$time',
          creation_time='$time',
          ip='$ip', user_fk='$user->id',
          lang_fk='$lang_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $id = $this->con->insertAndGetId($query);

    // if we're adding a master node
    if ($related_content_id == 0)
    {
      NestedSetModelHelper::insertNode($this, $id, $parent_node_id);
    }

    // insert new translation
    if ($related_content_id == 0)
    {
      // this is a new node : it is a translation of itself
      $related_content_id = $id;
    }
    $query = <<<SQL
      INSERT INTO {prefix}content_lang
      SET related_content_fk='$related_content_id', content_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    $this->con->execute('COMMIT');

    global $cacheLite;
    if (isset($cacheLite))
    {
      // install program use this method and has no cache
      $cacheLite->wcCleanAll();
    }

    return $id;
  }

  /**
   * \brief make a new root node out of an existing page
   *
   * \param id database id of new root node
   */
  function makeNewRootPage($id)
  {
    $id = $this->con->escapeString($id);

    $this->con->execute('START TRANSACTION');

    // retrieve hierarchy info BEFORE moving content_fk line
    $root_plr = NestedSetModelHelper::getRootNodeHierarchy($this, $id);
    $root_right = $root_plr[2];

    $plr = NestedSetModelHelper::getNodeHierarchyById($this, $id);
    if ($plr !== false)
    {
      // this is a master node (this is what we want, else we do nothing)
      $parent_content_fk = $plr[0];
      $left              = $plr[1];
      $right             = $plr[2];

      NestedSetModelHelper::removeNode($this, $id, $left, $right, $parent_content_fk);
      NestedSetModelHelper::setNewRootNode($this, $id, $root_right);
    }
    $this->con->execute('COMMIT');

    global $cacheLite;
    $cacheLite->wcCleanAll();
  }

  /**
   * \brief delete a node and all its related nodes (translations)
   *
   * history, comments, acl, notifications associated to the node.
   * If the node is a master node, all translated nodes associated to it will be deleted too.
   *
   * \param id database id of node to delete
   */
  function deleteContentById($id)
  {
    $id = $this->con->escapeString($id);

    $this->con->execute('START TRANSACTION');

    // retrieve hierarchy info BEFORE deleting content_fk line
    $plr = NestedSetModelHelper::getNodeHierarchyById($this, $id);
    if ($plr !== false)
    {
      // master node
      $parent_content_fk = $plr[0];
      $left              = $plr[1];
      $right             = $plr[2];

      NestedSetModelHelper::deleteNode($this, $id, $left, $right, $parent_content_fk);
    }
    else
    {
      // translated nodes
      // retrieve all translated node and delete them
      $query = <<<SQL
        SELECT content_fk FROM {prefix}content_lang
        WHERE related_content_fk='$id'
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      $ids = $this->con->select($query);

      foreach ($ids as $sid)
      {
        $this->deleteContentById($sid);
      }
    }

    // delete history
    $query = <<<SQL
      DELETE FROM {prefix}history
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // then delete comments
    $query = <<<SQL
      DELETE FROM {prefix}comment
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // then delete content_group
    $query = <<<SQL
      DELETE FROM {prefix}content_group
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // then delete content_user_email
    $query = <<<SQL
      DELETE FROM {prefix}content_user_email
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // then delete content_lang's content_fk line
    $query = <<<SQL
      DELETE FROM {prefix}content_lang
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // then delete metadata_content's content_fk line
    $query = <<<SQL
      DELETE FROM {prefix}metadata_content
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // then delete attachments's content_fk line
    $query = <<<SQL
      DELETE FROM {prefix}attachments
      WHERE content_fk='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    // and finally delete node
    $query = <<<SQL
      DELETE FROM {prefix}content
      WHERE id='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    $this->con->execute('COMMIT');

    global $cacheLite;
    $cacheLite->wcCleanAll();
  }

  /**
   * \brief save a new version in history
   *
   * History are a stripped down version of content : hierarchy, parent relationship,
   * translation relationship, comments, acls, notifications are not saved
   *
   * \param node the node to save
   */
  function saveHistory($node)
  {
    $title              = $this->con->escapeString($node->title);
    $alternate_title    = $this->con->escapeString($node->alternate_title);
    $content            = $this->con->escapeString($node->content);
    $commit_msg         = $this->con->escapeString($node->commit_msg);
    $minor_major        = $this->con->escapeString($node->minor_major);
    $ip                 = $this->con->escapeString($node->ip);
    $time               = $this->con->escapeString($node->timestamp);

    $query = <<<SQL
      INSERT INTO {prefix}history
      SET title='$title', alternate_title='$alternate_title', content='$content',
      commit_msg='$commit_msg', minor_major='$minor_major', timestamp='$time', ip='$ip',
      user_fk='$node->user_id', content_fk='$node->id', parent_content_fk='$node->parent_id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->execute($query);
  }

  /**
   * \brief returns history of a node
   *
   * return ordered list of versions of a node
   *
   * \param content_id database id of node to retrieve
   * \return array[node]
   */
  function getHistory($content_id)
  {
    $content_id = $this->con->escapeString($content_id);

    $query = <<<SQL
      SELECT {prefix}history.id, {prefix}history.title, {prefix}history.alternate_title, {prefix}history.content, {prefix}history.commit_msg, {prefix}history.minor_major, {prefix}history.ip, {prefix}history.timestamp, {prefix}history.parent_content_fk,
             {prefix}lang.id, {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_lang.related_content_fk,
             {prefix}user.id, {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email
      FROM   {prefix}history
      JOIN   {prefix}content on {prefix}content.id={prefix}history.content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      JOIN   {prefix}user on {prefix}user.id={prefix}history.user_fk
      JOIN   {prefix}content_lang on {prefix}content_lang.content_fk={prefix}history.content_fk
      WHERE  {prefix}history.content_fk='$content_id'
      ORDER BY {prefix}history.timestamp DESC;
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    // $id, $title, $alternate_title, $content, $commit_msg, $mintor_major, $ip, $timestamp,
    // $lang, $lang_id
    // $parent_id, $left, $right, $related_content
    // $user_id, $login, $first_name, $last_name, $email
    $result = $this->con->select($query);

    $history = array();
    $nbResult = sizeof($result);
    for ($i = 0; $i < $nbResult; ++$i)
    {
      $lang = new lang($result[$i][9], $result[$i][10], $result[$i][11], $result[$i][12]);

      $history[$i] = new node(
        $result[$i][0],  $result[$i][1], $result[$i][2], $result[$i][3], $result[$i][4], $result[$i][5], $result[$i][6], $result[$i][7], $lang,
        $result[$i][8],  0, 0, $result[$i][13],
        $result[$i][14], $result[$i][15], $result[$i][16], $result[$i][17], $result[$i][18]
      );
    }

    return $history;
  }

  /**
   * \brief return a particular version of the history from database id
   *
   * \param history_id database id of version to retrieve
   * \return node object representing wanted version
   */
  function getHistoryVersion($history_id)
  {
    $history_id = $this->con->escapeString($history_id);

    $query = <<<SQL
      SELECT {prefix}history.id, {prefix}history.title, {prefix}history.alternate_title, {prefix}history.content, {prefix}history.commit_msg, {prefix}history.minor_major, {prefix}history.ip, {prefix}history.timestamp, {prefix}history.parent_content_fk,
             {prefix}lang.id, {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_lang.related_content_fk,
             {prefix}user.id, {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email
      FROM   {prefix}history
      JOIN   {prefix}content on {prefix}content.id={prefix}history.content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      JOIN   {prefix}user on {prefix}user.id={prefix}history.user_fk
      JOIN   {prefix}content_lang on {prefix}content_lang.content_fk={prefix}history.content_fk
      WHERE  {prefix}history.id='$history_id'
      ORDER BY {prefix}history.timestamp;
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    // $id, $title, $alternate_title, $content, $commit_msg, $minor_major, $ip, $timestamp, $lang, $lang_id
    // $parent_id, $left, $right, $related_content
    // $user_id, $login, $first_name, $last_name, $email
    $result = $this->con->select1($query);

    $lang = new lang($result[9], $result[10], $result[11], $result[12]);

    return new node(
      $result[0],  $result[1], $result[2], $result[3], $result[4], $result[5], $result[6], $result[7], $lang,
      $result[8],  0, 0, $result[13],
      $result[14], $result[15], $result[16], $result[17], $result[18]
    );
  }

  /**
   * \brief get nearest history by date strictly below given date
   *
   * \param node_id database id of node
   * \param timestamp date to look at
   *
   * \return node object representing wanted version
   */
  function getHistoryByDate($node_id, $timestamp)
  {
    $node_id = $this->con->escapeString($node_id);

    $query = <<<SQL
      SELECT {prefix}history.id, {prefix}history.title, {prefix}history.alternate_title, {prefix}history.content, {prefix}history.commit_msg, {prefix}history.minor_major, {prefix}history.ip, {prefix}history.timestamp, {prefix}history.parent_content_fk,
             {prefix}lang.id, {prefix}lang.code, {prefix}lang.description, {prefix}lang.flag,
             {prefix}content_lang.related_content_fk,
             {prefix}user.id, {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname, {prefix}user.email
      FROM   {prefix}history
      JOIN   {prefix}content on {prefix}content.id={prefix}history.content_fk
      JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      JOIN   {prefix}user on {prefix}user.id={prefix}history.user_fk
      JOIN   {prefix}content_lang on {prefix}content_lang.content_fk={prefix}history.content_fk
      WHERE  {prefix}history.content_fk='$node_id'
      AND    {prefix}history.timestamp<'$timestamp'
      ORDER BY {prefix}history.timestamp DESC LIMIT 1;
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    // $id, $title, $alternate_title, $content, $commit_msg, $minor_major, $ip, $timestamp, $lang, $lang_id
    // $parent_id, $left, $right, $related_content
    // $user_id, $login, $first_name, $last_name, $email
    $result = $this->con->select1($query);

    $lang = new lang($result[9], $result[10], $result[11], $result[12]);

    return new node(
      $result[0],  $result[1], $result[2], $result[3], $result[4], $result[5], $result[6], $result[7], $lang,
      $result[8],  0, 0, $result[13],
      $result[14], $result[15], $result[16], $result[17], $result[18]
    );
  }

  /**
   * \brief return a list of version identified by title and lang
   *
   * the list includes also the current content
   *
   * \param title title of node
   * \param lang lang of node
   * \param offset what portion of the list to return
   * \param limit max number of versions to return
   * \param withAuthor add author information in result (firstname, lastname)
   * \param withContent add complete content in result
   * \param withId add id of history in result
   *
   * \return array[title, alt_title, date, lang, author, content]
   */
  function getHistoryByTitle($title, $lang, $offset, $limit, $withAuthor = false, $withContent = false, $withId = false)
  {
    $title = $this->con->escapeString($title);
    $lang  = $this->con->escapeString($lang);

    // titleDateLangAuthorContent

    // current content
    $q1 =
    'SELECT {prefix}content.title, {prefix}content.alternate_title, {prefix}content.timestamp, {prefix}lang.code ';

    if ($withAuthor)
    {
      $q1 .= ', {prefix}user.firstname, {prefix}user.lastname ';
    }
    if ($withContent)
    {
      $q1 .= ', {prefix}content.content ';
    }
    if ($withId)
    {
      $q1 .= ', {prefix}content.id ';
    }

    $q1 .=
    'FROM  {prefix}content '.
    'JOIN  {prefix}lang on {prefix}lang.id={prefix}content.lang_fk '.
    'JOIN  {prefix}user on {prefix}user.id={prefix}content.user_fk '.
    'WHERE {prefix}content.title='."'".$title."'".' '.
    'AND   {prefix}lang.code='."'".$lang."'";

    // history
    $q2 =
    'SELECT {prefix}history.title, {prefix}history.alternate_title, {prefix}history.timestamp, {prefix}lang.code';

    if ($withAuthor)
    {
      $q2 .= ', {prefix}user.firstname, {prefix}user.lastname ';
    }
    if ($withContent)
    {
      $q2 .= ', {prefix}history.content ';
    }
     if ($withId)
    {
      $q2 .= ', {prefix}history.id ';
    }

    $q2 .=
    'FROM  {prefix}history '.
    'JOIN  {prefix}content on {prefix}content.id={prefix}history.content_fk '.
    'JOIN  {prefix}lang on {prefix}lang.id={prefix}content.lang_fk '.
    'JOIN  {prefix}user on {prefix}user.id={prefix}history.user_fk '.
    'WHERE {prefix}history.title='."'".$title."'".' '.
    'AND   {prefix}lang.code='."'".$lang."'".' '.
    'ORDER BY {prefix}history.timestamp DESC '.
    'LIMIT '.$limit.' OFFSET '.$offset;

    $q1 = str_replace('{prefix}', $this->prefix, $q1);
    $q2 = str_replace('{prefix}', $this->prefix, $q2);
    $result = $this->con->select('('.$q1.') UNION ('.$q2.')');

    return $result;
  }

  /**
   * \brief delete an entry of history
   *
   * \param id database id of history to delete
   */
  function deleteHistory($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      DELETE FROM {prefix}history
      WHERE id='$id'
SQL;
    $query = str_replace('{prefix}', $this->prefix, $query);

    $result = $this->con->execute($query);

    return $result;
  }

  /**
   * \brief delete all history of a page
   *
   * \param id database id of page's history to delete
   */
  function deleteAllHistory($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      DELETE FROM {prefix}history
      WHERE content_fk='$id'
SQL;
    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->execute($query);
  }

  /**
   * \brief add an anonymous comment
   *
   * \param node_id database if of node to comment
   * \param author user given author information
   * \param email user given email
   * \param title title of comment
   * \param comment
   */
  function anonymousComment($node_id, $author, $email, $title, $comment)
  {
    $node_id = $this->con->escapeString($node_id);
    $author  = $this->con->escapeString($author);
    $email   = $this->con->escapeString($email);
    $title   = $this->con->escapeString($title);
    $comment = $this->con->escapeString($comment);

    $ip = implode(',', auth::getIPs());
    $time = time();

    $user_id = user_anonymous_id;

    $query = <<<SQL
      INSERT INTO {prefix}comment
      SET title='$title', content='$comment', timestamp='$time', ip='$ip', name='$author', email='$email', content_fk='$node_id', user_fk='$user_id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    $result = $this->con->insertAndGetId($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief add a comment from a logged user
   *
   * \param node_id database if of node to comment
   * \param user user
   * \param title title of comment
   * \param comment
   */
  function comment($node_id, $user, $title, $comment)
  {
    $node_id = $this->con->escapeString($node_id);
    $title   = $this->con->escapeString($title);
    $comment = $this->con->escapeString($comment);

    $ip = implode(',', auth::getIPs());
    $time = time();

    $query = <<<SQL
      INSERT INTO {prefix}comment
      SET title='$title', content='$comment', timestamp='$time', ip='$ip', content_fk='$node_id', user_fk='$user->id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    $result = $this->con->insertAndGetId($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief delete a comment
   *
   * \param id database id of comment to delete
   */
  function deleteComment($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      DELETE FROM {prefix}comment
      WHERE id='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    $result =  $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief returns list of nodes by lang
   *
   * the match is exact
   *
   * \param lang_id database id of a lang
   * \return array[array[id, lang]]
   */
  function getAllNodesIdByLang($lang_id)
  {
    $lang_id = $this->con->escapeString($lang_id);

    $query = <<<SQL
      SELECT id, title
      FROM   {prefix}content
      WHERE  lang_fk='$lang_id'
      ORDER BY title ASC
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief search pages by keywords (implicit AND)
   *
   * the pages are searched from content and title
   *
   * \param search space separated list of keywords
   * \param offset what portion of the list to return
   * \param limit max number of elements to return
   *
   * \return array[array[title, alt_title, code]]
   */
  function searchPages($search, $offset, $limit)
  {
    $search = $this->con->escapeString($search);
    $searchElements = explode(' ', $search);

    $query = 'SELECT DISTINCT {prefix}content.title, {prefix}content.alternate_title, {prefix}lang.code '.
             'FROM {prefix}content '.
             'JOIN {prefix}lang on {prefix}lang.id={prefix}content.lang_fk '.
             'WHERE  ';

    $query .= '(';
    $first = true;
    foreach ($searchElements as $el)
    {
      if (!$first)
      {
        $query .= ' AND ';
      }
      else
      {
        $first = false;
      }

      if (function_exists('mb_strtolower'))
      {
        $el = mb_strtolower($el);
      }
      else
      {
        $el = strtolower($el);
      }

      $query .= "LOWER(title) LIKE '%".$el."%'";
    }
    $query .= ')';

    $query .= ' OR ';

    $query .= '(';
    $first = true;
    foreach ($searchElements as $el)
    {
      if (!$first)
      {
        $query .= ' AND ';
      }
      else
      {
        $first = false;
      }

      if (function_exists('mb_strtolower'))
      {
        $el = mb_strtolower($el);
      }
      else
      {
        $el = strtolower($el);
      }

      $query .= "LOWER(content) LIKE '%".$el."%'";
    }
    $query .= ')';
    $query .= ' LIMIT '.$limit.' OFFSET '.$offset;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief returns the list of recently modified nodes
   *
   * \param offset what portion of the list to return
   * \param limit max number of elements to return
   * \param withAuthor includes also in result author information
   * \param withContent includes also full content in result
   * \param withUserForReadAcl filter nodes by returning only nodes that are readable by user identified by user_id
   *
   * \return array[title, alt_title, date, lang, author, content]
   */
  function recentChanges($offset, $limit, $withAuthor = false, $withContent = false, $withUserForReadAcl = -1)
  {
    $offset = $this->con->escapeString($offset);
    $limit  = $this->con->escapeString($limit);

    $query =
    'SELECT {prefix}content.title, {prefix}content.alternate_title, {prefix}content.timestamp, {prefix}lang.code ';

    if ($withAuthor)
    {
      $query .= ', {prefix}user.firstname, {prefix}user.lastname ';
    }

    if ($withContent)
    {
      $query .= ', {prefix}content.content ';
    }

    $query .= ", {prefix}parent.title AS parent_title";

    $query .= "\n".
    'FROM   {prefix}content '."\n".
    'JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk '."\n".
    'JOIN   {prefix}user on {prefix}user.id={prefix}content.user_fk '."\n".
    'JOIN   {prefix}content_hierarchy on {prefix}content_hierarchy.content_fk={prefix}content.id '."\n".
    'JOIN   {prefix}content AS {prefix}parent on {prefix}parent.id={prefix}content_hierarchy.parent_content_fk '."\n";

    if ($withUserForReadAcl != -1)
    {
      $acl_read = acl_read;
      $anonymous_id = user_anonymous_id;

      // create a subquery that returns all nodes containing a read acl
      $subQuery1 = <<<SQL
        SELECT {prefix}content_group.content_fk
        FROM {prefix}content_group
        WHERE {prefix}content_group.acl_type='$acl_read'
SQL;

      // create a subquery that returns all nodes where user has explicit read access
      $subQuery2 = <<<SQL
        SELECT {prefix}content_group.content_fk
        FROM   {prefix}content_group
        JOIN   {prefix}user_group on {prefix}user_group.group_fk={prefix}content_group.group_fk
        WHERE  ({prefix}user_group.user_fk='$withUserForReadAcl' OR {prefix}user_group.user_fk='$anonymous_id')
        AND    {prefix}content_group.acl_type='$acl_read'
SQL;

      // create a subquery that returns all nodes where user has explicit read access from users
      $subQuery3 = <<<SQL
        SELECT {prefix}content_group.content_fk
        FROM   {prefix}content_group
        WHERE  ({prefix}content_group.user_fk='$withUserForReadAcl' OR {prefix}content_group.user_fk='$anonymous_id')
        AND    {prefix}content_group.acl_type='$acl_read'
SQL;

     $query .=
       'WHERE {prefix}content.id NOT IN ('."\n".$subQuery1."\n".') OR '.
             '{prefix}content.id IN ('."\n".$subQuery2."\n".') OR '.
             '{prefix}content.id IN ('."\n".$subQuery3."\n".') ';
    }

    $query .=
    'ORDER BY timestamp DESC '."\n".
    'LIMIT '.$limit.' OFFSET '.$offset;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief returns the list of recent comments
   *
   * \param offset what portion of the list to return
   * \param limit max number of elements to return
   * \param withAuthor includes also in result author information
   * \param withContent includes also full content in result
   *
   * \return array[content_title, id, title, date, lang, author, content]
   */
  function recentComments($offset, $limit, $withAuthor = false, $withContent = false)
  {
    $offset = $this->con->escapeString($offset);
    $limit  = $this->con->escapeString($limit);

    $query =
    'SELECT {prefix}content.title, {prefix}comment.id, {prefix}comment.title, {prefix}comment.timestamp, {prefix}lang.code ';

    if ($withAuthor)
    {
      $query .= ', {prefix}user.firstname, {prefix}user.lastname ';
    }

    if ($withContent)
    {
      $query .= ', {prefix}comment.content ';
    }

    $query .=
    'FROM   {prefix}comment '.
    'JOIN   {prefix}content on {prefix}content.id={prefix}comment.content_fk '.
    'JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk '.
    'JOIN   {prefix}user on {prefix}user.id={prefix}comment.user_fk '.
    'ORDER BY timestamp DESC '.
    'LIMIT '.$limit.' OFFSET '.$offset;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief returns the list of recent comments from a page
   *
   * \param title title of page to look at
   * \param lang lang of page to look at
   * \param offset what portion of the list to return
   * \param limit max number of elements to return
   * \param withAuthor includes also in result author information
   * \param withContent includes also full content in result
   *
   * \return array[content_title, id, title, date, lang, author, content]
   */
  function recentCommentsByTitle($title, $lang, $offset, $limit, $withAuthor = false, $withContent = false)
  {
    $title  = $this->con->escapeString($title);
    $lang   = $this->con->escapeString($lang);
    $offset = $this->con->escapeString($offset);
    $limit  = $this->con->escapeString($limit);

    $query =
    'SELECT {prefix}content.title, {prefix}comment.id, {prefix}comment.title, {prefix}comment.timestamp, {prefix}lang.code ';

    if ($withAuthor)
    {
      $query .= ', {prefix}user.firstname, {prefix}user.lastname ';
    }

    if ($withContent)
    {
      $query .= ', {prefix}comment.content ';
    }

    $query .=
    'FROM   {prefix}comment '.
    'JOIN   {prefix}content on {prefix}content.id={prefix}comment.content_fk '.
    'JOIN   {prefix}lang on {prefix}lang.id={prefix}content.lang_fk '.
    'JOIN   {prefix}user on {prefix}user.id={prefix}comment.user_fk '.
    'WHERE  {prefix}content.title='."'".$title."' ".
    'AND    {prefix}lang.code='."'".$lang."' ".
    'ORDER BY timestamp DESC '.
    'LIMIT '.$limit.' OFFSET '.$offset;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief return a list of page that may link to page
   *
   * in fact, the search is naive as we return the list of content that contains
   * title
   *
   * \param title the title to search for
   * \param offset what portion of the list to return
   * \param limit max number of elements to return
   *
   * \return array[array[title, lang]]
   */
  function whatLinksHere($title, $offset, $limit)
  {
    $title  = $this->con->escapeString($title);
    $offset = $this->con->escapeString($offset);
    $limit  = $this->con->escapeString($limit);

    $query = <<<SQL
      SELECT {prefix}content.title, {prefix}lang.code
      FROM {prefix}content
      JOIN {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      WHERE {prefix}content.content LIKE '%$title%'
      LIMIT $limit OFFSET $offset
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);

    return $this->con->select($query);
  }

  /**
   * \brief check if an acl exist for a node
   *
   * \param id database id of node we want to look at
   * \param acl_type type of acl to look for
   */
  function existAclOnIdAndType($id, $acl_type)
  {
    $id       = $this->con->escapeString($id);
    $acl_type = $this->con->escapeString($acl_type);

    $query = <<<SQL
      SELECT content_fk
      FROM {prefix}content_group
      WHERE content_fk='$id'
      AND   acl_type='$acl_type'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $r = $this->con->select($query);

    return !empty($r);
  }

  /**
   * \brief check if an acl exist for a node
   *
   * \param title title of page to look at
   * \param lang lang of page to look at
   * \param acl_type type of acl to look for
   */
  function existAclOnTitleLangAndType($title, $lang, $acl_type)
  {
    $title    = $this->con->escapeString($title);
    $lang     = $this->con->escapeString($lang);
    $acl_type = $this->con->escapeString($acl_type);

    $query = <<<SQL
      SELECT {prefix}content.id
      FROM {prefix}content
      JOIN {prefix}content_group on {prefix}content_group.content_fk={prefix}content.id
      JOIN {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      WHERE {prefix}content.title='$title'
      AND   {prefix}lang.code='$lang'
      AND   {prefix}content_group.acl_type='$acl_type'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $r = $this->con->select($query);

    return !empty($r);
  }

  /**
   * \brief add a list of groups having acl_type access to page
   *
   * \param id database id of node we want to write protect
   * \param groups array[group_id]
   * \param acl_type type of acl to apply
   * \param recApply wether to recursively apply acls to sub groups too
   */
  function addACLOnGroups($id, $groups, $acl_type, $recApply)
  {
    $id       = $this->con->escapeString($id);
    $acl_type = $this->con->escapeString($acl_type);

    $query = <<<SQL
      INSERT INTO {prefix}content_group
      (content_fk, group_fk, acl_type) VALUES
SQL;

    if ($recApply)
    {
      $ids = $this->getSubNodesId_($id);
    }
    else
    {
      $ids = array();
      $ids[] = $id;
    }

    $first = true;
    foreach ($ids as $sid)
    {
      foreach ($groups as $gid)
      {
        $gid = $this->con->escapeString($gid);

        if (!$first)
        {
          $query .= ',';
        }

        $query .= "('".$sid."', '".$gid."', '".$acl_type."')";
        $first = false;
      }
    }

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();
  }

  /**
   * \brief add a list of users having acl_type access to page
   *
   * \param id database id of node we want to write protect
   * \param $users array[user_id]
   * \param acl_type type of acl to apply
   * \param recApply wether to recursively apply acls to sub groups too
  */
  function addACLOnUsers($id, $users, $acl_type, $recApply)
{
    $id       = $this->con->escapeString($id);
    $acl_type = $this->con->escapeString($acl_type);

    $query = <<<SQL
      INSERT INTO {prefix}content_group
      (content_fk, user_fk, acl_type) VALUES
SQL;

    if ($recApply)
    {
      $ids = $this->getSubNodesId_($id);
    }
    else
    {
      $ids = array();
      $ids[] = $id;
    }

    $first = true;
    foreach ($ids as $sid)
    {
      foreach ($users as $uid)
      {
        $uid = $this->con->escapeString($uid);

        if (!$first)
        {
          $query .= ',';
        }

        $query .= "('".$sid."', '".$uid."', '".$acl_type."')";
        $first = false;
      }
    }

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();
  }

  /**
   * \brief remove a list of groups from the groups that have acl_type access to page
   *
   * \param id database id of node to protect
   * \param groups array[group_id]
   * \param acl_type type of acl to apply
   * \param recApply wether to recursively apply modification to sub nodes too
   */
  function removeACLOnGroups($id, $groups, $acl_type, $recApply)
  {
    $id       = $this->con->escapeString($id);
    $acl_type = $this->con->escapeString($acl_type);

    if ($recApply)
    {
      $ids = $this->getSubNodesId_($id);
    }
    else
    {
      $ids = array();
      $ids[] = $id;
    }

    $query = <<<SQL
      DELETE FROM {prefix}content_group
      WHERE content_fk IN (
SQL;

    $first = true;
    foreach ($ids as $sid)
    {
      if (!$first)
      {
        $query .= ',';
      }

      $query .= $sid;
      $first = false;
    }

    $query .= ') AND group_fk IN (';

    $first = true;
    foreach ($groups as $gid)
    {
      $gid = $this->con->escapeString($gid);

      if (!$first)
      {
        $query .= ',';
      }

      $query .= $gid;
      $first = false;
    }
    $query .= ") AND acl_type='".$acl_type."'";

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  /**
   * \brief remove a list of users from the users that have acl_type access to page
   *
   * \param id database id of node to protect
   * \param users array[user_id]
   * \param acl_type type of acl to apply
   * \param recApply wether to recursively apply modification to sub nodes too
  */
  function removeACLOnUsers($id, $users, $acl_type, $recApply)
  {
    $id       = $this->con->escapeString($id);
    $acl_type = $this->con->escapeString($acl_type);

    if ($recApply)
    {
      $ids = $this->getSubNodesId_($id);
    }
    else
    {
      $ids = array();
      $ids[] = $id;
    }

    $query = <<<SQL
      DELETE FROM {prefix}content_group
      WHERE content_fk IN (
SQL;

    $first = true;
    foreach ($ids as $sid)
    {
      if (!$first)
      {
        $query .= ',';
      }

      $query .= $sid;
      $first = false;
    }

    $query .= ') AND user_fk IN (';

    $first = true;
    foreach ($users as $uid)
    {
      $uid = $this->con->escapeString($uid);

      if (!$first)
      {
        $query .= ',';
      }

      $query .= $uid;
      $first = false;
    }
    $query .= ") AND acl_type='".$acl_type."'";

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  /**
   * \brief duplicate a set of acl from a node to another one
   *
   * \param src_id page id to duplicate
   * \param dst_id the page to modify
   */
  function duplicateACL($src_id, $dst_id)
  {
    $src_id = $this->con->escapeString($src_id);
    $dst_id = $this->con->escapeString($dst_id);

    $query = <<<SQL
      SELECT acl_type, group_fk, user_fk FROM {prefix}content_group
      WHERE content_fk='$src_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    if (empty($result))
    {
      return;
    }

    $query = <<<SQL
      INSERT INTO {prefix}content_group
      (content_fk, acl_type, group_fk, user_fk)
      VALUES
SQL;

    $first = true;
    foreach ($result as $r)
    {
      if (!$first)
      {
        $query .= ',';
      }

      $query .= "('".$dst_id."','".$r[0]."','".$r[1]."','".$r[2]."')";

      $first = false;
    }

    $query = str_replace('{prefix}', $this->prefix, $query);
    $this->con->execute($query);
  }

  /**
   * \brief get the list of emails that have subscribed to page
   *
   * \param content_id database id of node
   * \param userIdExclusionList list of user to exclude from result
   *
   * \return array[array[email]]
   */
  function getEmails($content_id, $userIdExclusionList=array())
  {
    $content_id = $this->con->escapeString($content_id);

    $query = <<<SQL
      SELECT email
      FROM {prefix}user
      JOIN {prefix}content_user_email on {prefix}content_user_email.user_fk={prefix}user.id
      WHERE {prefix}content_user_email.content_fk='$content_id'
SQL;

    if (!empty($userIdExclusionList))
    {
      $query .= ' AND {prefix}content_user_email.user_fk not in (';

      $first = true;
      foreach ($userIdExclusionList as $userId)
      {
        $userId = $this->con->escapeString($userId);

        if (!$first)
        {
          $query .= ',';
        }
        $query .= "'".$userId."'";
        $first = false;
      }

      $query .= ')';
    }

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }

  /**
   * \brief get list of node we have subscribed to
   *
   * \param user_id id of user we're looking
   * \return array[array[id, title, lang]]
   */
  function getWatches($user_id)
  {
    $user_id = $this->con->escapeString($user_id);

    $query = <<<SQL
      SELECT {prefix}content.id, {prefix}content.title, {prefix}lang.code
      FROM {prefix}content
      JOIN {prefix}content_user_email on {prefix}content_user_email.content_fk={prefix}content.id
      JOIN {prefix}lang on {prefix}lang.id={prefix}content.lang_fk
      WHERE {prefix}content_user_email.user_fk='$user_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }

  /**
   * \brief subscribe a user to a node
   *
   * \param node_id node to subscribe
   * \param user_id user to subscribe
   */
  function addWatch($node_id, $user_id)
  {
    $node_id = $this->con->escapeString($node_id);
    $user_id = $this->con->escapeString($user_id);

    $query = <<<SQL
      INSERT INTO {prefix}content_user_email
      VALUES ('$node_id', '$user_id');
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result =  $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief remove a user subscription to a node
   *
   * \param node_id node to unsubscribe
   * \param user_id user to unsubscribe
   */
  function removeWatch($node_id, $user_id)
  {
    $node_id = $this->con->escapeString($node_id);
    $user_id = $this->con->escapeString($user_id);

    $query = <<<SQL
      DELETE FROM {prefix}content_user_email
      WHERE content_fk='$node_id'
      AND user_fk='$user_id';
SQL;

    global $cacheLite;
    $cacheLite->wcClean();

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  /**
   * \brief subscribe a user to a node and all subnodes
   *
   * \param node_id node to subscribe
   * \param user_id user to subscribe
   */
  function addWatchRecursively($node_id, $user_id)
  {
    $node_id = $this->con->escapeString($node_id);
    $user_id = $this->con->escapeString($user_id);

    $query = <<<SQL
      INSERT INTO {prefix}content_user_email
      VALUES
SQL;

    $idTitles = $this->getWatches($user_id); // nodes already watched by user
    $ids = $this->getSubNodesId_($node_id);  // nodes we're about to add to user's watches

    $idsToRemove = array();
    $indexIdsToRemove = array();
    // for each nodes already watched
    foreach ($idTitles as $idTitle)
    {
      // search in watch we're about to add
      $nbIds = sizeof($ids);
      for ($i = 0; $i < $nbIds; ++$i)
      {
        if ($ids[$i] == $idTitle[0])
        {
          // user is already watching this node : remember the index we want to remove
          $indexIdsToRemove[] = $i;
          break;
        }
      }
    }

    // remove the index now, because php will not reindex the array
    // we will have "hole" into the index space of the array
    foreach ($indexIdsToRemove as $i)
    {
      unset($ids[$i]);
    }

    $first = true;
    foreach ($ids as $sid)
    {
      if (!$first)
      {
        $query .= ',';
      }

      $query .= "('".$sid."', '".$user_id."')";
      $first = false;
    }

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  /**
   * \brief remove all subscription of a user
   *
   * \param user_id database id of user we want to clear subscription
   */
  function removeAllWatch($user_id)
  {
    $user_id = $this->con->escapeString($user_id);

    $query = <<<SQL
      DELETE FROM {prefix}content_user_email
      WHERE user_fk='$user_id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->execute($query);
  }

  /**
   * \brief get the list of defined metadatas
   *
   * \return list of metadata object
   */
  function getMetaDatas()
  {
    $query =<<<SQL
      SELECT id, name, type
      FROM {prefix}metadata
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $sqlResult = $this->con->select($query);

    $result = array();
    foreach ($sqlResult as $r)
    {
      $result[] = new metadata($r[0], $r[1], $r[2]);
    }

    return $result;
  }

  /**
   * \brief get metadata by name
   *
   * \return metadata object
   */
  function getMetaDataByName($name)
  {
    $query =<<<SQL
      SELECT id, name, type
      FROM {prefix}metadata
      WHERE name='$name'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $r = $this->con->select1($query);

    if ($r === false)
    {
      return false;
    }

    return new metadata($r[0], $r[1], $r[2]);
  }

  /**
   * \brief get the list of authorized values for a closed metadata
   *
   * \param id database id of metadata
   * \return map<id, value>
   */
  function getClosedMetadataValues($id)
  {
    $id = $this->con->escapeString($id);

    $query =<<<SQL
      SELECT id, value
      FROM {prefix}metadata_value
      WHERE metadata_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $sqlResult = $this->con->select($query);

    $result = array();
    foreach ($sqlResult as $r)
    {
      $result[$r[0]] = $r[1];
    }

    return $result;
  }

  /**
   * \brief add an opened metadata value to a node
   *
   * add a metadata to a node with an open value
   *
   * \param node_id database id of node
   * \param metadata_id database id of metadata
   * \param value value of open metadata
   */
  function addContentMetaDataValue($node_id, $metadata_id, $value)
  {
    $node_id = $this->con->escapeString($node_id);
    $metadata_id = $this->con->escapeString($metadata_id);
    $value = $this->con->escapeString($value);

    $query = <<<SQL
      INSERT INTO {prefix}metadata_content
      (content_fk, metadata_fk, value)
      VALUES ('$node_id', '$metadata_id', '$value')
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief add a closed metadata value to a node
   *
   * add a metadata to a node with a closed value
   *
   * \param node_id database id of node
   * \param metadata_id database id of metadata
   * \param metadata_value_id value of closed metadata
   */
  function addContentMetadataClosedValue($node_id, $metadata_id, $metadata_value_id)
  {
    $node_id = $this->con->escapeString($node_id);
    $metadata_id = $this->con->escapeString($metadata_id);
    $metadata_value_id = $this->con->escapeString($metadata_value_id);

    $query = <<<SQL
      INSERT INTO {prefix}metadata_content
      (content_fk, metadata_fk, metadata_value_fk)
      VALUES ('$node_id', '$metadata_id', '$metadata_value_id')
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief returns the list of metadatas attached to a node
   *
   * As an optimisation we return open and closed metadata in one request
   * you will have to compare open_value and closed_value_id to see what kind
   * of metadata you're looking at
   *
   * \param id database id of node
   * \return array[array[id, name, value_id, open_value, closed_value_id]]
   */
  function getMetaDataInfoByContentId($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      SELECT {prefix}metadata.id, {prefix}metadata.name,
             {prefix}metadata_content.id, {prefix}metadata_content.value,
             {prefix}metadata_content.metadata_value_fk
      FROM   {prefix}metadata_content
      JOIN   {prefix}metadata on {prefix}metadata.id={prefix}metadata_content.metadata_fk
      WHERE  {prefix}metadata_content.content_fk='$id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }

  /**
   * \brief delete a metadata value
   *
   * \param id database id of metadata value
   */
  function delContentMetaDataValue($id)
  {
    $id = $this->con->escapeString($id);

    $query = <<<SQL
      DELETE FROM {prefix}metadata_content
      WHERE id='$id';
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->execute($query);

    global $cacheLite;
    $cacheLite->wcClean();

    return $result;
  }

  /**
   * \brief get the list of closed metadata values
   *
   * \param closed_metadatas_ids list of closed metadata ids
   * \return map[metadata_id, map<value_id, value>>
   */
  function getClosedMetadataValuesByIds($closed_metadatas_ids)
  {
    $idList = '';

    $first = true;
    foreach ($closed_metadatas_ids as $id)
    {
      if (!$first)
      {
        $idList .= ',';
      }

      $id = $this->con->escapeString($id);

      $idList .= "'".$id."'";

      $first = false;
    }

    $query = <<<SQL
      SELECT {prefix}metadata_value.id, {prefix}metadata_value.metadata_fk,
             {prefix}metadata_value.value
      FROM   {prefix}metadata_value
      WHERE  {prefix}metadata_value.metadata_fk IN ($idList)
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select($query);

    $metadataIdToIdValue = array();
    foreach ($result as $r)
    {
      $mId = $r[1];
      $vId = $r[0];
      $v   = $r[2];

      if (empty($metadataIdToIdValue[$mId]))
      {
        $idValues = array();
        $metadataIdToIdValue[$mId] = $idValues;
      }

      $metadataIdToIdValue[$mId][$vId] = $v;
    }

    return $metadataIdToIdValue;
  }

  /**
   * \brief add a new ip to the list of banned ip
   *
   * \param ip the ip to ban
   * \return true if the ip was added, false if it was already present
   */
  function addBan($ip)
  {
    $ip = $this->con->escapeString($ip);

    // verify this IP is not already banned
    if (!$this->isBanned($ip))
    {
      // not banned, ban it
      $query = <<<SQL
        INSERT INTO {prefix}ban
        (ip)
        VALUES ('$ip');
SQL;

      $query = str_replace('{prefix}', $this->prefix, $query);
      return $this->con->execute($query);
    }

    return false;
  }

  /**
   * \brief return a list of already banned ips from an array of ip
   *
   * \param ips the ips to look for
   * \return array of ips that are in ips and are already banned
   */
  function getAlreadyBannedIps($ips)
  {
    $query = <<<SQL
      SELECT ip
      FROM {prefix}ban
      WHERE ip IN (
SQL;

    $first = true;
    foreach ($ips as $ip)
    {
      if (!$first)
      {
        $query .= ',';
      }

      $ip = $this->con->escapeString($ip);

      $query .= "'".$ip."'";
      $first = false;
    }

    $query .= ')';

    $query = str_replace('{prefix}', $this->prefix, $query);
    $results = $this->con->select($query);

    $r = array();
    foreach ($results as $result)
    {
      $r[] = $result[0];
    }

    return $r;
  }

  /**
   * \brief ban several ips at once
   *
   * \param ips the ips to ban
   * \return true if one ip was added, false if none was added
   */
  function addBans($ips)
  {
    $bannedIps = $this->getAlreadyBannedIps($ips);
    $ipsToBan = array_diff($ips, $bannedIps);

    if (!empty($ipsToBan))
    {
      $query = <<<SQL
        INSERT INTO {prefix}ban
        (ip)
        VALUES
SQL;

      $first = true;
      foreach ($ipsToBan as $ip)
      {
        if (!$first)
        {
          $query .= ',';
        }

        $ip = $this->con->escapeString($ip);

        $query .= "('".$ip."')";
        $first = false;
      }

      $query = str_replace('{prefix}', $this->prefix, $query);
      $this->con->execute($query);
    }

    return !empty($ipsToBan);
  }

  /**
   * \brief look if ip is banned
   *
   * \param ip the ip to look
   * \return true if the ip was banned, false otherwise
   */
  function isBanned($ip)
  {
    $ip = $this->con->escapeString($ip);

    $query = <<<SQL
      SELECT ip
      FROM {prefix}ban
      WHERE ip='$ip'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $result = $this->con->select1($query);

    return !empty($result);
  }

  /**
   * \brief look if one ip of a list of ip is banned
   *
   * \param ips the ip to look
   * \return true if the ip was banned, false otherwise
   */
  function containsBannedIp($ips)
  {
    $bannedIps = $this->getAlreadyBannedIps($ips);
    return !empty($bannedIps);
  }

  /**
  * \brief add a new attachment to a page
  *
  * \param file_name file to attach to page
  * \param node_id id of page
  * \param user_id id of user attaching to page
  */
  function attachFileToPage($file_name, $node_id, $user_id)
  {
    $file_name = $this->con->escapeString($file_name);
    $node_id   = $this->con->escapeString($node_id);
    $user_id   = $this->con->escapeString($user_id);

    $ip = implode(',', auth::getIPs());
    $time = time();

    $query =<<<SQL
    INSERT INTO {prefix}attachments
    SET file_name='$file_name', timestamp='$time', ip='$ip', content_fk='$node_id', user_fk='$user_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $sqlResult = $this->con->execute($query);
  }

  /**
  * \brief delete an attachment
  *
  * \param file_name name of attachment to delete
  * \param node_id id of node from which we want to delete the attachement
  */
  function deleteAttachments($file_name, $node_id)
  {
    $file_name = $this->con->escapeString($file_name);
    $node_id   = $this->con->escapeString($node_id);

    $query =<<<SQL
      DELETE FROM {prefix}attachments
      WHERE file_name='$file_name' AND content_fk='$node_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    $sqlResult = $this->con->execute($query);
  }

  /**
  * \brief get all attachments linked to a page
  *
  * \param node_id id of node from which to get attachments
  * \return array<file_name>
  */
  function getAttachments($node_id)
  {
    $node_id = $this->con->escapeString($node_id);

    $query =<<<SQL
      SELECT {prefix}attachments.file_name, {prefix}attachments.timestamp, {prefix}attachments.ip,
            {prefix}user.login, {prefix}user.firstname, {prefix}user.lastname
      FROM {prefix}attachments
      JOIN {prefix}user on {prefix}user.id={prefix}attachments.user_fk
      WHERE content_fk='$node_id'
SQL;

    $query = str_replace('{prefix}', $this->prefix, $query);
    return $this->con->select($query);
  }
}
?>
