<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

require wc_inc_path.'/inc/lib/error.lib.php';
require wc_inc_path.'/inc/classes/form.class.php';
require wc_inc_path.'/inc/classes/metadata.class.php';
require wc_inc_path.'/inc/lib/format.lib.php';
require wc_inc_path.'/inc/lib/login.lib.php';
require wc_inc_path.'/inc/lib/register.lib.php';
require wc_inc_path.'/inc/lib/content.lib.php';
require wc_inc_path.'/inc/lib/comment.lib.php';
require wc_inc_path.'/inc/lib/history.lib.php';
require wc_inc_path.'/inc/lib/lang_helper.lib.php';
require wc_inc_path.'/inc/lib/user_preferences.lib.php';
require wc_inc_path.'/inc/lib/search.lib.php';
require wc_inc_path.'/inc/lib/recentchanges.lib.php';
require wc_inc_path.'/inc/lib/whatlinkshere.lib.php';
require wc_inc_path.'/inc/lib/edit_acl.lib.php';
require wc_inc_path.'/inc/lib/watch.lib.php';
require wc_inc_path.'/inc/lib/metadata.lib.php';
require wc_inc_path.'/inc/lib/feed.lib.php';
require wc_inc_path.'/inc/lib/attachment.lib.php';
require wc_inc_path.'/inc/lib/ban.lib.php';

/**
 * \file
 * \brief this page includes all other native features and provide minimal functionalities
 *
 * Important functions are :
 * <ul>
 *   <li>wikiMenu() for the generation of a basic wiki menu</li>
 *   <li>contentBox() for the displaying of main area</li>
 *   <li>handleUserAction() for handling user interaction</li>
 *   <li>jsLinks() for generating links to javascript libraries when needed</li>
 * </ul>
 *
 * 3 global variables are used for that :
 * <ul>
 *   <li>$javascriptLibs : this array contain the list of required javascript lib to include</li>
 *   <li>$contentBoxHandle : this map<mode=>callback> allows easy redirection to a particular page from a given mode</li>
 *   <li>$userActionHandle : this map<mode=>callback> allows easy user interaction from a given mode</li>
 * </ul>
 */

/**
 * \brief generates javascript links so that a user can easily incorporates javascript defined in plugins
 * \ingroup template_functions
 *
 * \return html links to javascript suitable for inclusion in a HEADER section
 */
function jsLinks()
{
  global $app_path;
  global $javascriptLibs;

  if (sizeof($javascriptLibs) == 0)
  {
    return '';
  }

  $html = '<meta http-equiv="Content-Script-Type" content="text/javascript" />'."\n";

  // add support library first
  $html .= '<script type="text/javascript" src="'.$app_path.'tools/lib.js"></script>'."\n";
  foreach ($javascriptLibs as $jsLib)
  {
    $html .= '<script type="text/javascript" src="'.$app_path.$jsLib.'"></script>'."\n";
  }

  return $html;
}

/**
 * \brief builds a link to wiki root page
 * \ingroup template_functions
 *
 * generates an anchor link pointing to wiki root page, for use in template :
 *
 * \return html anchor
*/
function getRootNodeLink()
{
  global $wiki;
  global $node;

  $rootNode = $wiki->getRootNodeInLang(wc_default_lang);
  if (isset($node))
  {
    $lang = $node->lang->code;
  }
  else
  {
    $lang = wc_default_lang;
  }

  $rootTitle = '';
  if (isset($rootNode))
  {
    $rootTitle = $rootNode->title;
  }

  return '<a href="'.createURL($rootTitle, array('lang'=>$lang)).'">'.tr('Home Page').'</a>';
}

/**
 * \brief builds an access menu to different wiki parts
 */
function accessMenu()
{
  $blob = '<ul>';
  $blob .= '<li><a href="#content">'.tr('Go to content').' ('.tr('key').' 2)</a></li>';
  $blob .= '<li><a href="#comments">'.tr('Go to comments').' ('.tr('key').' 9)</a></li>';
  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief builds a complete wiki menu
 * \ingroup template_functions
 *
 * generates an unordered list of links depending of state (user logged, global rights ...) that contain :
 * - edit link
 * - add content link
 * - edit ACL link
 * - delete node link
 * - history link
 * - subscribe/unsubscribe link
 *
 * \return html unordered list
 */
function wikiMenu()
{
  global $mode;

  if (!empty($mode))
  {
    return '';
  }

  $blob = '<ul>';
  $editContentLink = editContentLink();
  if (!empty($editContentLink))
  {
    $blob .= '<li class="edit_content">'.$editContentLink.'</li>';
  }
  $addContentLink = addContentLink();
  if (!empty($addContentLink))
  {
    $blob .= '<li class="add_content">'.$addContentLink.'</li>';
  }
  $editACLLink = editACLLink();
  if (!empty($editACLLink))
  {
    $blob .= '<li class="edit_acl">'.$editACLLink.'</li>';
  }
  $deleteLink = deleteLink();
  if (!empty($deleteLink))
  {
    $blob .= '<li class="delete">'.$deleteLink.'</li>';
  }
  $makeRootLink = makeRootLink();
  if (!empty($makeRootLink))
  {
    $blob .= '<li class="make_root">'.$makeRootLink.'</li>';
  }
  $historyLink = historyLink();
  if (!empty($historyLink))
  {
    $blob .= '<li class="history">'.$historyLink.'</li>';
  }
  $emailSubscribeLink = emailSubscribeLink();
  if (!empty($emailSubscribeLink))
  {
    $blob .= '<li class="email_watch">'.$emailSubscribeLink.'</li>';
  }
  $emailUnsubscribeLink = emailUnsubscribeLink();
  if (!empty($emailUnsubscribeLink))
  {
    $blob .= '<li class="email_unwatch">'.$emailUnsubscribeLink.'</li>';
  }
  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief display main content area
 * \ingroup template_functions
 *
 * the main content area contains either the wiki content, or content based on mode (edition, search ...)
 * It uses the global map variable $contentBoxHandle and the mode to callback a registered method
 * that can handle the page.
 * If no mode is found, or if no function is registered to handle current mode, the special method
 * registered to the "default" mode is called.
 */
function contentBox()
{
  global $contentBoxHandle;
  global $mode;

  if (array_key_exists($mode, $contentBoxHandle))
  {
    return $contentBoxHandle[$mode]();
  }
  else
  {
    global $pluginManager;
    $contentBox = $pluginManager->contentBoxHandle($mode);
    if (!empty($contentBox))
    {
      return $contentBox;
    }
  }

  // no mode ? display default hook
  return $contentBoxHandle['default']();
}

/**
 * \brief handle user action
 *
 * called after the world is set up and before the them template builds up the page
 */
function handleUserAction()
{
  global $mode;

  if (empty($mode))
  {
    return;
  }

  global $userActionHandle;;
  if (array_key_exists($mode, $userActionHandle))
  {
    $userActionHandle[$mode]();
  }
  else
  {
    global $pluginManager;
    $pluginManager->handleUserAction($mode);
  }
}

/**
 * \brief handle browser title page customisation
 *
 * called after world setup and event handling
 */
function handleBrowserTitlePage()
{
  global $browserTitlePageHandle;
  global $mode;

  $result = '';

  if (array_key_exists($mode, $browserTitlePageHandle))
  {
    $result = $browserTitlePageHandle[$mode]();
  }
  else
  {
    global $pluginManager;
    $browserTitlePage = $pluginManager->browserTitlePageHandle($mode);
    if (!empty($browserTitlePage))
    {
      $result = $browserTitlePage;
    }
    else
    {
      // no mode ? use default hook
      $result = $browserTitlePageHandle['default']();
    }
  }

  return formatHtml($result);
}

/**
 * \brief load page info from database
 *
 * called after world setup and before event handling
 * if this is the main page, then gives a page to $mainPageHandler to intercept the call
 * (usefull to avoid loading the root node if you want to display something else)
 * else, it loads the page from GET variable title and current $user_lang
 */
function loadCurrentPage()
{
  global $con;
  if (!$con->connected())
  {
    return;
  }

  global $mainPage;
  if ($mainPage == true)
  {
    global $mainPageHandler;

    if (!empty($mainPageHandler)) 
    {
      $mainPageHandler();
    }
  }
  else
  {
    global $title;
    global $node;
    global $wiki;
    global $user_lang;

    if (isset($_GET['title']))
    {
      // because this can be a SpecialPage
      $title = rawurldecode(varGet('title'));
      $node  = $wiki->getNodeByTitleAndLang($title, $user_lang);
    }
  }
}
?>
