<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls comments
 *
 * This file creates several hooks :
 * <ul>
 *   <li>comment : to add a new comment</li>
 *   <li>comment_del : to delete a comment</li>
 * </ul>
 *
 * Edition is done via a form that contains several fields :
 * <ul>
 *   <li>a title : the title of the comment</li>
 *   <li>the comment : written in plain text without formatting</li>
 * </ul>
 *
 * This page contains also one helper functions for the user to create template :
 * <ul>
 *   <li>commentLink</li>
 * </ul>
 */

$contentBoxHandle['comment']     = 'commentAdd';

$userActionHandle['comment']     = 'onSubmitComment';
$userActionHandle['comment_del'] = 'onDeleteComment';

$browserTitlePageHandle['comment'] = 'browserCommentAdd';

/**
 * \brief customize browser title when adding comment
 *
 * \return browser title
 */
function browserCommentAdd()
{
  return wc_wiki_name.' - '.tr('Adding comment');
}

/**
 * \brief creates a comment link
 * \ingroup template_functions
 *
 * \return html link
 */
function commentLink()
{
  global $mode;

  if (!empty($mode))
  {
    return '';
  }

  global $node;

  if (!empty($node))
  {
    global $user;

    $canComment = auth::canComment($user, $node);
    if ($canComment)
    {
      return '<a href="'.createURL($node->title, array('mode'=>'comment', 'lang'=>$node->lang->code)).'">'.tr('Post comment').'</a>';
    }
    else
    {
      return '<p>'.tr('Comments are closed').'</p>';
    }
  }
}

/**
 * \brief display comments
 * \ingroup template_functions
 *
 * \return html blob
 */
function commentBox()
{
  global $node;
  global $user;
  global $mode;

  if (!empty($mode))
  {
    return '';
  }

  $comments = array();
  if (!empty($node))
  {
    global $wiki;
    $comments = $wiki->getNodeComments($node->id);
  }

  $nbComments = sizeof($comments);
  if ($nbComments == 0)
  {
    return '<p class="post-info-co"><a href="#comments" accesskey="9"></a>'.tr('No comments yet').'</p>';
  }

  $blob = '<h1><a href="#comments" accesskey="9"></a>'.tr('Comments').'</h1>';

  for ($i = 0; $i < $nbComments; ++$i)
  {
    $comment = &$comments[$i];
    $id = $comment->id;

    if ($user->isModerator() || $user->isAdmin())
    {
      $blob .= '<div class="admin">';
      $blob .= '<a class="delete" href="'.createURL($node->title, array('mode'=>'comment_del', 'id'=>$id, 'lang'=>$node->lang->code)).'">'.tr('Delete').'</a>';
      $blob .= '&nbsp;';
      $blob .= '<a class="ban" href="'.createURL($node->title, array('mode'=>'ban_add', 'ip'=>$comment->ip, 'lang'=>$node->lang->code, 'anchor'=>'co'.$id)).'">'.tr('Ban').'</a>';
      $blob .= '</div>';
    }

    $blob .= displayComment($comment->id, $comment->name, $comment->email, $comment->timestamp, $comment->title, $comment->content);
  }

  $blob .= '<div class="spacer">&nbsp;</div>';

  return $blob;
}

/**
 * \brief display one comment
 *
 * \return html blob
 */
function displayComment($id, $author, $email, $timestamp, $title, $comment)
{
  $comment = formatHtml($comment);
  $comment = str_replace("\n", '<br />', $comment);

  return
  '<div class="comment">'.
    '<div class="comment_meta">'.
      '<h1><a name="co'.$id.'" id="co'.$id.'" href="#co'.$id.'">'.formatHtml($title).'</a></h1>'.
      '<p>'.formatDate($timestamp).', '.tr('by').' '.formatAuthor($author, $email).'</p>'.
    '</div>'.
    $comment.
  '</div>';
}

/**
 * \brief display a form to add a comment
 *
 * \return html blob
 */
function commentAdd()
{
  global $node;
  global $user;

  if (!auth::canComment($user, $node))
  {
    return;
  }

  $author            = "";
  $email             = "";
  $comment_title     = "";
  $comment           = "";
  $remember_settings = false;

  if (existVarCookie('comment_settings'))
  {
    $comment_settings = &varCookie('comment_settings');
    $author = $comment_settings['author'];
    $email  = $comment_settings['email'];
    $remember_settings = true;
  }

  $submitDisabled = (wc_wiki_force_preview=='true');
  if (isset($_POST['check']))
  {
    if (isset($_POST['author']))
    {
      $author = varPost('author');
    }
    else
    if (!$user->isAnonymous())
    {
      $author = $user->first_name.' '.$user->last_name;
    }

    if (isset($_POST['email']))
    {
      $email = varPost('email');
    }
    else
    {
      $email = $user->email;
    }

    $comment_title = trim(varPost('comment_title'));
    $comment       = varPost('comment');

    if (isset($_POST['remember_settings']))
    {
      $remember_settings = true;
    }

    if ($author == '')
    {
      $error = tr('You must enter a pseudo');
    }
    else
    if (empty($comment_title))
    {
      $error = tr('You must enter a title');
    }
    else
    if (empty($comment))
    {
      $error = tr('You must enter a comment');
    }
    else
    if (existVarSession('comment_error'))
    {
      $error = varSession('comment_error');
      unsetVarSession('comment_error');
    }

    if (empty($error))
    {
      $submitDisabled = false;
    }
  }

  $blob = '';

  if (!empty($error))
  {
    $blob = '<h1>'.$error.'</h1>';
  }

  // preview
  if (!empty($comment))
  {
    $blob .= displayComment(0, $author, $email, time(), $comment_title, $comment);
  }

  // form
  $form = new form(createURL($node->title, array('mode'=>'comment', 'lang'=>$node->lang->code)), 'post', 'comment');
  $form->addFieldset();
  $form->addLegend(tr('Comment'));

  if ($user->isAnonymous())
  {
    $form->addHtml('<p class="field">');
    $form->addLabel(tr('Pseudo'), 'fauthor');
    $form->addInput('author', 'Author', formatHtml($author), 'fauthor');
    $form->addHtml('</p>');

    $form->addHtml('<p class="field">');
    $form->addLabel(tr('Email'), 'femail');
    $form->addInput('email', 'Email', formatHtml($email), 'femail');
    $form->addHtml('</p>');
  }

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Title'), 'ftitle');
  $form->addInput('comment_title', 'Title', formatHtml($comment_title), 'ftitle');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addTextarea('comment', 20, 20, formatHtml($comment), 'fcomment');
  $form->addHtml('</p>');

  if ($user->isAnonymous())
  {
    $form->addHtml('<p class="field">');
    $form->addCheckbox('remember_settings', $remember_settings);
    $form->addLabel(tr('Remember my information'));
    $form->addHtml('</p>');
  }

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('preview', tr('Preview'), 'submit');
  $form->addButton('submit', tr('Publish'), 'submit', $submitDisabled);
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  return $blob.$form->toHtml();
}

/**
 * \brief called when user submit a new comment
 */
function onSubmitComment()
{
  global $user;
  global $wiki;
  global $node;

  if (!auth::canComment($user, $node) || !isset($_POST['check']) || !isset($_POST['submit']))
  {
    return;
  }
  if ($user->isAnonymous() && auth::banned())
  {
    // banned
    setVarSession('comment_error', tr('Banned'));
    return;
  }

  $author = '';
  if (isset($_POST['author']))
  {
    $author        = varPost('author');
  }

  $email = '';
  if (isset($_POST['email']))
  {
    $email         = varPost('email');
  }

  $comment_title = varPost('comment_title');
  $comment       = varPost('comment');

  if ((empty($author) && $user->isAnonymous()) || empty($comment_title) || empty($comment))
  {
    return;
  }

  global $pluginManager;

  $data = array($comment_title, $comment);
  $reason = '';
  if ($pluginManager->validatePluginCallback('on_new_comment', $data, $reason) === false)
  {
    // plugin rejected content
    setVarSession('comment_error', $reason);
    return;
  }

  $comment_id = 0;
  if ($user->isAnonymous())
  {
    $comment_id = $wiki->anonymousComment($node->id, $author, $email, $comment_title, $comment);
  }
  else
  {
    $comment_id = $wiki->comment($node->id, $user, $comment_title, $comment);
  }

  $pluginManager->onPluginCallback('on_content_edit', $data);

  if (isset($_POST['remember_settings']))
  {
    $expire = time() + 60*60*24*30; // expire in one month
    setVarCookie('comment_settings[author]', $author, $expire);
    setVarCookie('comment_settings[email]',  $email,  $expire);
  }

  $message =
  tr('A new comment has been posted ').' '.tr('on page').' '.$node->title."\n".
  createHeaderURL($node->title, array('lang'=>$node->lang->code, 'anchor'=>'co'.$comment_id))."\n";

  $email_title = '['.wc_wiki_name.']'.tr('New comment');

  $emails = $wiki->getEmails($node->id, array($user->id));
  foreach ($emails as $email)
  {
    $result = mail_helper::send(
      wc_mail, $email[0], wc_charset, 
      $email_title,
      $message
    );
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code)));
}

/**
 * \brief called when user delete a comment
 */
function onDeleteComment()
{
  global $user;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET['id']))
  {
    return;
  }

  global $wiki;
  global $node;

  $id = varGet('id');

  $wiki->deleteComment($id);

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code, 'anchor'=>'comments')));
}
?>