<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls the main functionality : content edition
 *
 * This file creates several hooks :
 * <ul>
 *   <li>edit : to edit the content</li>
 *   <li>add  : to add a new content</li>
 *   <li>delete : to delete a content</li>
 * </ul>
 *
 * Edition is done via a form that contains several fields :
 * <ul>
 *   <li>a title : the title of the node (will control the urls)</li>
 *   <li>an alternate title : will be displayed instead of the title if non empty</li>
 *   <li>a parent section : indicate the hiearchical parent of the page being edited/created</li>
 *   <li>the content : written in a wiki syntax</li>
 *   <li>a previsualisation button</li>
 *   <li>a publish button</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>editContentLink</li>
 *   <li>addContentLink</li>
 *   <li>deleteLink</li>
 *   <li>subContentBox</li>
 *   <li>parentContentBox</li>
 * </ul>
 */

$mainPageHandler                    = 'loadMainPage';

$contentBoxHandle['default']        = 'contentDisplay';
$contentBoxHandle['edit']           = 'contentEdit';
$contentBoxHandle['add']            = 'contentEdit';
$contentBoxHandle['delete']         = 'contentDelete';
$contentBoxHandle['source']         = 'sourceDisplay';

$userActionHandle['edit']           = 'onContentEdit';
$userActionHandle['add']            = 'onContentEdit';
$userActionHandle['delete_confirm'] = 'onContentDelete';
$userActionHandle['make_root']      = 'onContentMakeRoot';

$browserTitlePageHandle['default']  = 'browserContentDisplay';
$browserTitlePageHandle['edit']     = 'browserContentEdit';
$browserTitlePageHandle['add']      = 'browserContentAdd';

/**
 * \brief customize browser title for page content
 *
 * \param $text text to add with wiki name and page title
 *
 * \return browser title
 */
function browserContentCustomize($text)
{
  global $node;

  if (empty($node))
  {
    global $title;

    return wc_wiki_name.' - '.$text.$title.'('.tr('Empty').')';
  }

  $title = empty($node->alternate_title) ? $node->title : $node->alternate_title;
  return wc_wiki_name.' - '.$text.formatHtml($title).'['.$node->lang->code.']';
}

/**
 * \brief customize browser title when displaying content
 *
 * \return browser title
 */
function browserContentDisplay()
{
  return browserContentCustomize('');
}

/**
 * \brief customize browser title when editing content
 *
 * \return browser title
 */
function browserContentEdit()
{
  return browserContentCustomize(tr('Edit').' ');
}

/**
 * \brief customize browser title when adding content
 *
 * \return browser title
 */
function browserContentAdd()
{
  $title = '';
  if (isset($_POST['content_title']))
  {
    $title = ' '.varPost('content_title');
  }

  return wc_wiki_name.' - '.tr('Add new page').formatHtml($title);
}

/**
 * \brief loads main page
 */
function loadMainPage()
{
  global $node;
  global $user_lang;
  global $wiki;
  global $title;
  $node  = $wiki->getRootNodeInLang($user_lang);

  if (!empty($node))
  {
    $title = $node->title;
    $url = createHeaderURL($node->title, array('lang'=>$node->lang->code));
    header('Location: '.$url);
  }
}

/**
 * \brief creates edit content link
 * \ingroup template_functions
 *
 * \return html link
 */
function editContentLink()
{
  global $node;

  $node_title = '';
  $node_lang = '';

  if (empty($node))
  {
    global $title;
    global $user_lang;

    $node_title = $title;
    $node_lang  = wc_default_lang; // empty node => we can only edit it in default lang
  }
  else
  {
    $node_title = $node->title;
    $node_lang  = $node->lang->code;
  }

  global $user;

  $canEdit = auth::canEdit($user, $node);

  if ($canEdit)
  {
    return '<a href="'.createURL($node_title, array('mode'=>'edit', 'lang'=>$node_lang)).'">'.tr('Edit content').'</a>';
  }
}

/**
 * \brief creates add content link
 * \ingroup template_functions
 *
 * \return html link
 */
function addContentLink()
{
  global $node;

  if (!empty($node))
  {
    $node_id    = $node->id;
    $node_lang  = $node->lang->code;
  }
  else
  {
    global $wiki;

    $rootNode = $wiki->getRootNodeInLang(wc_default_lang);

    $node_id    = empty($rootNode) ? 0 : $rootNode->id;
    $node_lang  = wc_default_lang;
  }

  global $user;

  $canEdit = auth::canEdit($user, $node);

  if ($canEdit && $node_lang == wc_default_lang)
  {
    return '<a href="'.createURL('', array('mode'=>'add', 'parent_node_id'=>$node_id)).'">'.tr('Post new content').'</a>';
  }
}

/**
 * \brief creates delete link
 * \ingroup template_functions
 *
 * \return html link
 */
function deleteLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  global $user;

  if (($user->isModerator() || $user->isAdmin()) && // we have the rights and
      $node->isRootNode() == false                  // this is not the root node
     )
  {
    return '<a href="'.createURL($node->title, array('mode'=>'delete', 'lang'=>$node->lang->code)).'">'.tr('Delete').'</a>';
  }
}

/**
 * \brief creates a "make root" link
 * \ingroup template_functions
 *
 * \return html link
 */
function makeRootLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  global $user;

  if (($user->isModerator() || $user->isAdmin()) && // we have the rights and
      $node->isRootNode() == false  &&              // this is not the root node
      $node->isMasterNode() == true
     )
  {
    return '<a href="'.createURL($node->title, array('mode'=>'make_root', 'lang'=>$node->lang->code)).'">'.tr('Make root').'</a>';
  }
}

/**
 * \brief creates a print link
 * \ingroup template_functions
 *
 * \return html link
 */
function printLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  return '<a href="'.createURL($node->title, array('mode'=>'print', 'lang'=>$node->lang->code)).'">'.tr('Print').'</a>';
}

/**
 * \brief creates a print link
 * \ingroup template_functions
 *
 * \return html link
 */
function sourceLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  return '<a href="'.createURL($node->title, array('mode'=>'source', 'lang'=>$node->lang->code)).'">'.tr('Source').'</a>';
}

/**
 * \brief display list of sub nodes as an unordered list
 * \ingroup template_functions
 *
 * \return html unordered list
 */
function subContentBox()
{
  global $node;
  global $user_lang;
  global $user;

  $links = array();
  if (!empty($node))
  {
    global $wiki;
    $links = $wiki->getNodeSubLinks($node->related_content_id, $node->lang->id, $user->id);
  }

  $nbLinks = sizeof($links);
  if ($nbLinks == 0)
  {
    return '';
  }

  $blob = '<ul>';
  foreach ($links as $link)
  {
    $subnode_title   = $link[1];
    $alternate_title = $link[2];
    $node_lang       = $link[3];
    if (empty($alternate_title))
    {
      $alternate_title = $subnode_title;
    }

    $blob .= '<li><a href="'.createURL($subnode_title, array('lang'=>$node_lang)).'">'.formatHtml($alternate_title).'</a></li>';
  }

  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief display list of parent nodes as li element
 * \ingroup template_functions
 *
 * \return html li
 */
function parentElemContentBox()
{
  global $node;
  global $user_lang;

  if (empty($node))
  {
    return '';
  }

  global $wiki;
  $parents = $wiki->getNodeParents($node->related_content_id, $node->lang->id);

  $blob = '';

  $class = ' class="first"';
  foreach ($parents as $parent)
  {
    $pid        = $parent[0];
    $ptitle     = $parent[1];
    $palt_title = $parent[2];

    $title = empty($palt_title) ? $ptitle : $palt_title;

    $blob .= '<li'.$class.'><a href="'.createURL($ptitle, array('lang'=>$user_lang)).'">'.formatHtml($title).'</a></li>';
    $class = '';
  }

  return $blob;
}

/**
 * \brief display title of the current node
 * \ingroup template_functions
 *
 * \return html li
 */
function currentElemContentBox()
{
  global $node;
  global $user_lang;

  if (empty($node))
  {
    return '';
  }

  $title = empty($node->alternate_title) ? $node->title : $node->alternate_title;
  return '<a href="'.createURL($node->title, array('lang'=>$user_lang)).'">'.formatHtml($title).'</a>';
}


/**
 * \brief display list of parent nodes as an unordered list
 * \ingroup template_functions
 *
 * \return html unordered list
 */
function parentContentBox()
{
  global $node;
  global $user_lang;

  if (empty($node))
  {
    return '<ul><li class="first">'.tr('At top level').'</li></ul>';
  }

  $blob = '<ul>';

  $class = ' class="first"';
  $parents = parentElemContentBox();
  if ($parents != '')
  {
    $class = '';
  }
  $blob .= $parents;

  $title = currentElemContentBox();
  $blob .= '<li'.$class.'>'.$title.'</li>';

  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief display time taken by script during whole execution
 * \ingroup template_functions
 *
 * \return html paragraph
 */
function phpExecutionTimeBox()
{
  global $php_start_time;
  $php_end_time = chrono::now();
  $seconds = $php_end_time - $php_start_time;

  return '<p>'.tr('PHP execution time').' : '.chrono::formatSeconds($seconds).' '.tr('seconds').'</p>';
}

/**
 * \brief display time taken by sql queries during whole execution
 * \ingroup template_functions
 *
 * \return html paragraph
 */
function sqlExecutionTimeBox()
{
  global $con;

  return '<p>'.tr('SQL execution time').' : '.chrono::formatSeconds($con->cumulated_sql_execution_time).' '.tr('seconds').'</p>';
}

/**
 * \brief display content
 * \ingroup template_functions
 *
 * \param showLangs indicates if we must show lang flags or not
 *
 * \return html blob
 */
function contentDisplay($showLangs = true)
{
  global $node;
  global $wiki;
  global $user;
  global $user_lang;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return errorMessageACLRead();
  }

  $allLangs = array();
  if ($showLangs)
  {
    $allLangs = $wiki->getLangs();
  }

  $canEdit =  auth::canEdit($user, $node);

  if (empty($node))
  {
    global $title;

    $my_node = new node(
      0, $title, '', tr('This page does not exist yet. Use the Edit link to add content.'), '', major_value, '',
      time(), new lang(0, wc_default_lang, 'english', 'flag_en.png'),
      0, 0, 0, 0,
      0, $user->login, $user->first_name, $user->last_name, $user->email
    );
    $my_node->langs = array();

    // search for default lang and keep only that one
    $nbLangs = sizeof($allLangs);
    for ($i = 0; $i < $nbLangs; ++$i)
    {
      if ($allLangs[$i]->code != wc_default_lang)
      {
        unset($allLangs[$i]);
      }
    }
  }
  else
  {
    $my_node = $node;
    setVarSession('last_visited_node_id', $node->id);
  }

  $html = '<a href="#content" accesskey="2"></a>';
  if (!empty($node) && $node->lang->code != $user_lang)
  {
    $html .= errorMessage(tr('This page is not available in requested lang, reverting to default lang'));
  }

  global $showTOC;
  global $editBySection;
  $html .= $my_node->toHtml($showTOC, $editBySection && $canEdit, $allLangs, $canEdit);

  return $html;
}

/**
 * \brief display source
 * \ingroup template_functions
 *
 * \return html blob
 */
function sourceDisplay()
{
  global $node;
  global $wiki;
  global $user;
  global $user_lang;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return '<p>'.tr('Sorry, you do not have enough rights to see this page').'</p>';
  }

  if (empty($node))
  {
    global $title;

    $my_node = new node(
      0, $title, '', tr('This page does not exist yet. Use the Edit link to add content.'), '', major_value, '',
      time(), new lang(0, wc_default_lang, 'english', 'flag_en.png'),
      0, 0, 0, 0,
      0, $user->login, $user->first_name, $user->last_name, $user->email
    );
  }
  else
  {
    $my_node = $node;
  }

  $html = formatHtml($node->content);
  $html = str_replace("\n", '<br/>', $html);

  return $html;
}

/**
 * \brief this method extract from a node wanted section
 *
 * \param node node to extract section from
 * \param section the section to extract, -1 to extract everything
 * \return extracted section
 */
function getContentFilteredBySection(&$node, $section)
{
  if ($section == -1)
  {
    // we're interested in whole content
    return $node->content;
  }
  else
  {
    // filter content  by retrieving only wanted section
    global $wikiRenderer;
    // parse content once to retrieve all sections and lines
    $wikiRenderer->render($node->content);
    // split by lines
    $contentsByLine = explode("\n", $node->content);
    // extract start line
    $lineStart = 0;
    if (sizeof($wikiRenderer->config->titles) > $section)
    {
      $lineStart = $wikiRenderer->config->titles[$section][3];
    }
    // extract end line
    $lineEnd = sizeof($contentsByLine);
    if (sizeof($wikiRenderer->config->titles) > $section + 1)
    {
      $lineEnd = $wikiRenderer->config->titles[$section + 1][3] - 1;
    }
    // reset WikiRenderer's config
    $wikiRenderer->config->reset();
    // and extract wanted lines
    $contentsByLine = array_slice($contentsByLine, $lineStart, $lineEnd - $lineStart + 1);
    return implode("\n", $contentsByLine);
  }
}

/**
 * \brief this method merge a node by replacing section with given content
 *
 * this is the "pendant" of method getContentFilteredBySection
 *
 * \param content the content to merge with
 * \param node node to merge with
 * \param section the section to merge, -1 to merge everything
 * \return merged content
 */
function mergeWithNodeAndSection($content, &$node, $section)
{
  if ($section == -1)
  {
    // we're interested in whole content
    return $content;
  }
  else
  {
    // filter content  by retrieving only wanted section
    global $wikiRenderer;
    // parse content once to retrieve all sections and lines
    $wikiRenderer->render($node->content);
    // split by lines
    $nodeContentsByLine = explode("\n", $node->content);
    // extract start line
    $lineStart = 0;
    if (sizeof($wikiRenderer->config->titles) > $section)
    {
      $lineStart = $wikiRenderer->config->titles[$section][3];
    }
    // extract end line
    $lineEnd = sizeof($nodeContentsByLine);
    if (sizeof($wikiRenderer->config->titles) > $section + 1)
    {
      $lineEnd = $wikiRenderer->config->titles[$section + 1][3] - 1;
    }
    // reset WikiRenderer's config
    $wikiRenderer->config->reset();
    // and merge wanted section
    $contentsByLine = explode("\n", $content);
    array_splice($nodeContentsByLine, $lineStart, $lineEnd - $lineStart + 1, $contentsByLine);
    return implode("\n", $nodeContentsByLine);
  }
}

function generateContentForm(
  $mode,
  $nodeLang, $content_title, $alternate_title, $parent_node_id, $related_node_id,
  $content, $commit_msg, $minor_major, $inherit_acl,
  $timestamp,
  $hasParentDropDown, $edit_section, $submitDisabled
)
{
  global $wiki;
  global $title;
  global $user;

  $formUrlOptions = array('mode'=>$mode, 'lang'=>$nodeLang->code);
  if ($edit_section != -1)
  {
    $formUrlOptions['edit_section'] = $edit_section;
  }
  $form = new form(createURL($title, $formUrlOptions), 'post', 'edit_content');
  $form->addFieldset();
  $form->addLegend(tr('Edit content'));

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Title'), 'ftitle');
  $content_title = formatHtml($content_title);
  $form->addInput('content_title', 'Title', $content_title, 'ftitle');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Alternate title'), 'falternate_title');
  $alternate_title = formatHtml($alternate_title);
  $form->addInput('alternate_title', 'Alternate title', $alternate_title, 'falternate_title');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  if ($hasParentDropDown)
  {
    $nodesId = $wiki->getAllNodesIdByLang($nodeLang->id);

    $form->addLabel(tr('Parent section'), 'fparent');

    $options = array();
    foreach ($nodesId as $nodeId)
    {
      if ($nodeId[1] != $content_title)
      {
        $options[formatHtml($nodeId[1])] = $nodeId[0];
      }
    }

    $form->addDropDown($options, $parent_node_id, 'parent_node_id', 'fparent');
  }
  else
  {
    $form->addHidden('parent_node_id', $parent_node_id);
  }
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHtml('<a href="help.php?h=editing_help&amp;lang='.$user->ui_lang.'" onclick="'."window.open(this.href, 'wc_popup', 'alwaysRaised=yes,toolbar=no,height=420,width=500,menubar=no,resizable=yes,scrollbars=yes,status=no'); return false;".'">'.tr('Editing quick help').'</a>');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('lang_id', $nodeLang->id);
  $form->addHtml('</p>');

  $content = formatHtml($content);
  $form->addTextarea('content', 20, 20, $content, 'fcontent');

  $form->addHtml('<p class="field" id="commit_msg">');
  $form->addLabel(tr('Commit message'), 'fcommit_msg');
  $form->addHtml('</p>');
  $commit_msg = formatHtml($commit_msg);
  $form->addTextarea('commit_msg', 3, 20, $commit_msg, 'fcommit_msg');

  if ($mode == 'add')
  {
    $form->addHtml('<p class="field">');
    $form->addHidden('minor_major', major_value);
    $form->addHtml('</p>');
  }
  else
  {
    $form->addHtml('<p class="field">');
    $form->addCheckbox('minor_major', $minor_major == major_value, 'fminor_major');
    $form->addLabel(tr('Major change'), 'fminor_major');
    $form->addHtml('</p>');
  }

  // acls
  if ($mode == 'add')
  {
    $form->addHtml('<p class="field"/>');

    $form->addHtml('<p class="field">');
    $form->addCheckbox('inherit_acl', $inherit_acl, 'finherit_acl');
    $form->addLabel(tr('Inherit acl'), 'finherit_acl');
    $form->addHtml('</p>');
  }

  $form->addHtml('<p class="field">');
  $form->addHidden('related_node_id', $related_node_id);
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('timestamp', $timestamp);
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('preview', tr('Preview'), 'submit');
  $form->addButton('submit', tr('Publish'), 'submit', $submitDisabled);
  $form->addButton('cancel', tr('Cancel'), 'submit');
  $form->addHtml('</p>');

  global $pluginManager;
  $pluginManager->pluginCallback('content_edit_form', $form);

  $form->addHtml('<p class="field"/>');

  return $form->toHtml();
}

function getLangFromNodeAndUrl($node, $mode)
{
  $langCode = wc_default_lang;

  // allow the user to enter nodes in a different lang only when editing
  if ($mode == 'edit')
  {
    // get lang from node
    if (!empty($node))
    {
      $langCode = $node->lang->code;
    }
    // except if user wants a different node from the URL
    if (isset($_GET['lang']))
    {
      $langCode = varGet('lang');
    }
  }

  return $langCode;
}

/**
 * \brief edit content
 *
 * \return html blob
 */
function contentEdit()
{
  global $node;
  global $user;

  if (!auth::canEdit($user, $node))
  {
    return;
  }
  if (empty($node) && isset($_GET['lang']) && varGet('lang') != wc_default_lang)
  {
    // do not allow edition of an empty node in a different lang than the default
    return;
  }

  global $wiki;
  global $title;
  global $mode;
  global $browser_title_page;

  // get default values
  $content_title    = $title;
  $alternate_title  = '';
  $content          = '';
  $commit_msg       = '';
  $minor_major      = major_value;
  $inherit_acl      = false;
  $parent_node_id   = -1;
  $related_node_id  = 0;
  $timestamp        = 0;
  $edit_section     = -1;
  $submitDisabled   = (wc_wiki_force_preview == 'true');
  $first_time       = (isset($_POST['check']) == false);
  $translatedLangs  = array();

  if (isset($_GET['related_content_id']))
  {
    $related_node_id = varGet('related_content_id');
  }
  if (isset($_GET['edit_section']))
  {
    $edit_section = varGet('edit_section');
  }

  // get all langs
  $allLangs = $wiki->getLangs();

  if ($first_time)
  {
    // get lang id from lang code obtained from node and url
    $langCode = getLangFromNodeAndUrl($node, $mode);
    $langId = lang::getIdFromCode($allLangs, $langCode);
    unset($langCode);

    // now get more values from what can be guessed
    if ($mode == 'edit' && empty($node))
    {
      // followed a link to an empty page
      $mode = 'add';
    }

    if ($mode == 'add')
    {
      if (isset($_GET['parent_node_id']))
      {
        $parent_node_id = varGet('parent_node_id');
      }
      else
      if (existVarSession('last_visited_node_id'))
      {
        $parent_node_id = varSession('last_visited_node_id');
      }
      else
      {
        $rootNode = $wiki->getRootNodeInLang(wc_default_lang);
        $parent_node_id = empty($rootNode) ? 0 : $rootNode->id;
      }
    }
    else
    if ($mode == 'edit' && $node->lang->id != $langId)
    {
      // followed a link to an empty translated page
      $parent_node_id  = $node->parent_id;
      $content_title   = $node->title;
      $alternate_title = $node->alternate_title;
      $related_node_id = $node->related_content_id;
      $timestamp       = $node->timestamp;

      // we want to translate a node
      unset($node);
      $mode = 'add';
    }
    else // ($mode == 'edit' && $node->lang->id == $langId)
    {
      // followed a link to an existing page (master or translation)
      $parent_node_id  = $node->parent_id;
      $content_title   = $node->title;
      $alternate_title = $node->alternate_title;
      $related_node_id = $node->related_content_id;
      $timestamp       = $node->timestamp;
      $content         = getContentFilteredBySection($node, $edit_section);
    }
  }
  else
  {
    // get values from form
    $content_title   = varPost('content_title');
    $alternate_title = varPost('alternate_title');
    $content         = varPost('content');
    $commit_msg      = varPost('commit_msg');
    $minor_major     = !isset($_POST['minor_major']) ? minor_value : major_value;
    if ($mode == 'add')
    {
      $inherit_acl     = isset($_POST['inherit_acl']);
    }
    $parent_node_id  = varPost('parent_node_id');
    $langId          = varPost('lang_id');
    $related_node_id = varPost('related_node_id');
    $timestamp       = varPost('timestamp');

    if (!empty($node) && $node->lang->id != $langId)
    {
      // we want to translate a node
      // we have retrieved enough information
      unset($node);
    }
  }

  if (!empty($node))
  {
    $translatedLangs = $node->langs;
  }

  // check values
  if ($first_time == false)
  {
    $conflictDetected = false;
    if ($mode == 'edit' && !empty($node) && $node->timestamp != $timestamp)
    {
      $conflictDetected = true;
    }

    if ($content_title == '')
    {
      $errors[] = tr('You must choose a title');
    }
    if ($mode == 'add' && $wiki->contentExists($content_title, $langId))
    {
      $errors[] = tr('Error, a node with title').' '.$content_title.' '.tr('already exist');
    }
    if ($mode == 'edit' && !empty($node) && $content_title != $node->title && $wiki->contentExists($content_title, $langId))
    {
      $errors[] = tr('Error, a node with title').' '.$content_title.' '.tr('already exist');
    }
    if ($content == '')
    {
      $errors[] = tr('Error, you must enter some content');
    }
    if ($conflictDetected)
    {
      $errors[] = tr('Conflict detected : the content has been modified while you were editing it');
    }
    if (existVarSession('content_error'))
    {
      $errors[] = varSession('content_error');
      unsetVarSession('content_error');
    }

    if (empty($errors))
    {
      $submitDisabled = false;
    }

    if ($conflictDetected)
    {
      // retrieve last version
      $diff = new diff(formatHtml($content), formatHtml($node->content));
      $content = $diff->toText();
      $timestamp = $node->timestamp;
    }
  }

  // prepare html output
  $blob = '';

  if (!empty($errors))
  {
    $blob_tmp = '<ul class="error">';
    foreach ($errors as $error)
    {
      $blob_tmp .= '<li>'.$error.'</li>';
    }
    $blob_tmp .= '</ul>';
    $blob .= errorMessage($blob_tmp);
  }

  $nodeLang = $wiki->getLangById($langId);

  // preview
  if (!$first_time && !empty($content))
  {
    $nodeToDisplay = new node(
      0, $content_title, $alternate_title, $content, $commit_msg, $minor_major, '', time(), $nodeLang,
      0, 0, 0, 0,
      $user->id, $user->login, $user->first_name, $user->last_name, $user->email
    );
    $nodeToDisplay->langs = $translatedLangs;
    $blob .= $nodeToDisplay->toHtml(false, false, $allLangs);

    global $wikiRenderer;
    if ($wikiRenderer->errors)
    {
      $blob .= '<p class="wiki_error">'.tr('Wiki error found on lines : ').'</p>';
      $blob .= '<ul>';
      foreach ($wikiRenderer->errors as $lineNumber=>$line)
      {
        $blob .= '<li>'.tr('Line').' '.$lineNumber.' : '.formatHtml($line).'</li>';
      }
      $blob .= '</ul>';
    }
  }

  // form
  // add drop down for parent
  $hasParentDropDown = false;
  if ($nodeLang->code == wc_default_lang)
  {
    if ($mode == 'add')
    {
      $hasParentDropDown = true;
    }
    else
    if ($node->isRootNode() == false)
    {
      $hasParentDropDown = true;
    }
  }

  $blob .= generateContentForm(
    $mode,
    $nodeLang, $content_title, $alternate_title, $parent_node_id, $related_node_id,
    $content, $commit_msg, $minor_major, $inherit_acl,
    $timestamp,
    $hasParentDropDown, $edit_section, $submitDisabled
  );

  // help
  $help = new translation(wc_inc_path.'/help/', $user->ui_lang, 'en', '.help');

  $blob .=
  '<p class="help">'.tr('Editing quick help').'</p>'.
  $help->get('editing_help');

  return $blob;
}

/**
 * \brief called when user submit content
 */
function onContentEdit()
{
  global $user;
  global $wiki;
  global $node;
  global $mode;

  $content = '';

  if (isset($_POST['cancel']))
  {
    header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code)));
  }

  if (!auth::canEdit($user, $node) || !isset($_POST['check']) || !isset($_POST['submit']))
  {
    // rights
    return;
  }
  if ($user->isAnonymous() && auth::banned())
  {
    // banned
    setVarSession('content_error', tr('Banned'));
    return;
  }

  // retrieve data from form
  $content_title   = varPost('content_title');
  $alternate_title = varPost('alternate_title');
  $content         = varPost('content');
  $commit_msg      = varPost('commit_msg');
  $minor_major     = !isset($_POST['minor_major']) ? minor_value : major_value;
  $inherit_acl     = isset($_POST['inherit_acl']);
  $parent_node_id  = varPost('parent_node_id');
  $lang_id         = varPost('lang_id');
  $related_node_id = varPost('related_node_id');
  $timestamp       = varPost('timestamp');
  $edit_section    = -1;
  if (isset($_GET['edit_section']))
  {
    $edit_section = varGet('edit_section');
  }

  /////////////////////////////////////////////////////////////////////////////
  // sanity checks BEGIN
  /////////////////////////////////////////////////////////////////////////////
  if (empty($content_title))
  {
    // no title
    return;
  }

  if ($mode == 'edit' && empty($node))
  {
    // fix mode when following an url on an empty page
    $mode = 'add';
  }

  if ($mode == 'edit' && $node->lang->id != $lang_id)
  {
    // fix mode when editing an empty translation node
    $mode = 'add';
  }

  $lang = $wiki->getLangById($lang_id);
  if ($mode == 'add')
  {
    // check lang relationships
    if ($lang->code == wc_default_lang && $related_node_id == 0)
    {
      // OK : we're adding a new node in default lang
    }
    else
    if ($lang->code != wc_default_lang && $related_node_id != 0 && $wiki->isMasterNode($related_node_id))
    {
      // OK : we're adding a new node in a translation lang that is linked to another node
      // and that node is a master node
    }
    else
    {
      // all other cases forbidden
      return;
    }

    // check a node does not already exist
    if ($wiki->contentExists($content_title, $lang_id))
    {
      return;
    }

    // check parent node does exist
    if (!$wiki->contentExistsById($parent_node_id))
    {
      return;
    }
  }

  if ($mode == 'edit')
  {
    if ($parent_node_id != $node->parent_id && !$wiki->contentExistsById($parent_node_id))
    {
      // parent node was changed, but parent_id do not exist (someone is trying to hack the form)
      return;
    }

    if ($content_title != $node->title && $wiki->contentExists($content_title, $lang_id))
    {
      // user has modified the title and another page exist with same name
      // it is forbidden
      return;
    }

    if ($node->timestamp != $timestamp)
    {
      // conflict : saved timestamp from form is different from database's timestamp : someone edited
      // it at the same time and validated before us
      return;
    }
  }

  // data validated
  $content = mergeWithNodeAndSection($content, $node, $edit_section);

  // plugin validation
  global $pluginManager;

  $debug_validate_plugin = strpos($commit_msg, 'debug_spam_score') !== false; // hack
  $data = array($content_title, $alternate_title, $content, $commit_msg);
  $reason = '';
  if ($pluginManager->validatePluginCallback('on_content', $data, $reason, $debug_validate_plugin) === false)
  {
    // plugin rejected content
    setVarSession('content_error', $reason);
    return;
  }
  if ($debug_validate_plugin)
  {
    setVarSession('content_error', 'Debugging spam score');
    return;
  }

  /////////////////////////////////////////////////////////////////////////////
  // sanity checks END
  /////////////////////////////////////////////////////////////////////////////

  // everything's validated at last
  // it's about time to save data
  if ($mode == 'edit')
  {
    $wiki->saveHistory($node);

    $wiki->editContent(
      $node->id, $node->parent_id, $parent_node_id, $content_title, $alternate_title, $content,
      $commit_msg, $minor_major,
      $node->related_content_id,
      $user,
      $node->left, $node->right
    );

    $pluginManager->onPluginCallback('on_content_edit', $data);
  }
  else
  {
    $newNodeId = $wiki->addContent(
      $parent_node_id,
      $content_title,
      $alternate_title,
      $content,
      $commit_msg,
      $minor_major,
      $lang_id,
      $related_node_id,
      $user
    );

    if ($inherit_acl)
    {
      if ($related_content_id == 0)
      {
        $wiki->duplicateACL($parent_node_id, $newNodeId);
      }
      else
      {
        $wiki->duplicateACL($related_content_id, $newNodeId);
      }
    }

    $pluginManager->onPluginCallback('on_content_add', $data);
  }

  $content_url = createHeaderURL($content_title, array('lang'=>$lang->code));
  if (!empty($node) && $minor_major == major_value)
  {
    // send an email
    $message =
    tr('Content has been modified ').' '.tr('on page').' '.$content_title."\n".
    $content_url."\n";

    $email_title = '['.wc_wiki_name.']'.tr('Content modified');

    $emails = $wiki->getEmails($node->id, array($user->id));
    foreach ($emails as $email)
    {
      $result = mail_helper::send(
        wc_mail, $email[0], wc_charset,
        $email_title,
        $message
      );
    }
  }

  // and redirect
  header('Location: '.$content_url);
}

/**
 * \brief ask user for confirmation before deleting content
 * By Steven Whitbread <wiclear@crazyvibe.co.uk>
 *
 * \return html blob
 */
function contentDelete()
{
  global $user;
  global $user_lang;
  global $wiki;
  global $node;
  global $wikirender;

  if (!$user->isModerator() && !$user->isAdmin())
  {
    return;
  }

  $html  = '<h1>'.tr('Confirm delete').'</h1>';
  $html .= '<p>'.tr('This will permanently delete the page and history from the wiki, you will not be able to undo this action').'.</p>';
  $html .= '<p>'.tr('Please confirm you wish to delete page named').' "'.formatHtml($node->title).'"</p>';
  $delete_link = '<a href="'.createURL($node->title, array('mode'=>'delete_confirm', 'lang'=>$node->lang->code)).'">'.
                tr('Yes (delete)').'</a>';
  $cancel_link = '<a href="'.createURL($node->title, array('lang'=>$node->lang->code)).'">'.tr('No (take me back)').'</a>';
  $html .= '<p>'.$delete_link.'&nbsp;|&nbsp;'.$cancel_link.'</p>';

  return $html;
}

/**
 * \brief called when user delete content
 */
function onContentDelete()
{
  global $user;
  global $user_lang;
  global $wiki;
  global $node;

  if (!$user->isModerator() && !$user->isAdmin())
  {
    return;
  }

  // prevent deletion of root node
  if ($node->isRootNode() == false)
  {
    $wiki->deleteContentById($node->id);
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$user_lang)));
}

/**
 * \brief called when user choose a page as new root for the site
 */
function onContentMakeRoot()
{
  global $user;
  global $user_lang;
  global $wiki;
  global $node;

  if (!$user->isModerator() && !$user->isAdmin())
  {
    return;
  }

  // do not do this on root node or translation node
  if ($node->isRootNode() == false && $node->isMasterNode() == true)
  {
    $wiki->makeNewRootPage($node->id);
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$user_lang)));
}

/**
* \brief display navigation box
* \ingroup template_functions
* \author Kai Dietrich
* \return html navigation box (ul/li)
*/
function navigationBox()
{
  global $node;
  global $user;
  global $wiki;

  if (empty($node))
  {
    return;
  }

  //this is the path in the page-tree array<link>
  $trail = $wiki->getNodeParents($node->id, $node->lang->id);
  $trail[] = array(0=>$node->id, 1=>$node->title, 2=>$node->alternate_title);

  //this are the sublinks for each level in the page tree
  //results in a list of linklists
  //first level of the array is the path in the page-tree up to the root,
  //second level is a list of sublinks, third level are the links
  $sublinks = array();
  foreach($trail as $t)
  {
    $sublinks[] = $wiki->getNodeSubLinks($t[0], $node->lang->id, $user->id);
  }

  //create the html ul/li recursively
  $blob = createNavLevel($node, $trail, $sublinks, 0);

  return $blob;
}
/**
 * \brief recursive helper function for navigationBox
 *
 * creates one level in the html tree, then calls itself for the next level
 *
 * \author Kai Dietrich
 */
//use call by reference here, to save memory and cpu cycles? don't know if this works out
function createNavLevel($node, &$trail, &$sublinks, $level)
{
  $blob = '<ul>';

  foreach ($sublinks[$level] as $link)
  {
    $blob .= '<li>'.createNavLink($node, $link);

    //next level?
    if (!empty($sublinks[$level+1]) && $link[0]== $trail[$level+1][0])
    {
      $blob .= createNavLevel($node, $trail, $sublinks, $level+1);
    }

    $blob .= '</li>';
  }

  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief helper function for navigationBox
 *
 * creates a navigation link
 *
 * \author Kai Dietrich
 */
function createNavLink($node, $link)
{
  // global $node;
  global $user_lang;
  $blob = '';
  $subnode_title = $link[1];
  $alternate_title = $link[2];

  if (empty($alternate_title))
  {
    $alternate_title = $subnode_title;
  }

  if ($node->id == $link[0])
  {
    $blob .= $alternate_title;
  }
  else
  {
    $blob.='<a href="'.createURL($subnode_title, array('lang'=>$user_lang)).'">'.$alternate_title.'</a>';
  }

  return $blob;
}

/**
 * \brief  construct the list langs
 *
 * \return all available langs
 *
 */

function langBox()
{
  global $node;
  global $wiki;
  global $user;
  global $user_lang;

  if (empty($node))
  {
    return '';
  }

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return '';
  }

  $allLangs    = $wiki->getLangs();
  $canEdit     = auth::canEdit($user, $node);
  $currentLang = '';
  $langList    = '';

  foreach ($allLangs as $lang)
  {
    if ($node->lang->id == $lang->id)
    {
      $options = array('lang'=>$lang->code);
      $url  = createUrl($node->title, $options);
      $langList .= '<li><strong><a href="'.$url.'">'.$lang->code.'</a></strong></li>';
    }
    else
    {
      $options = array('lang'=>$lang->code);

      if ($canEdit)
      {
        // try to find the lang in the list of already translated lang
        $translated = false;
        foreach ($node->langs as $tlangid=>$tlangcode)
        {
          if ($tlangid == $lang->id)
          {
            // already translated
            $translated = true;
            break;
          }
        }

        if ($translated == false)
        {
          // provide a link to edit the node
          $options['mode']               = 'edit';
          $options['related_content_id'] = $node->related_content_id;
        }
      }

      // create URL
      $url  = createUrl($node->title, $options);
      $html = '<a href="'.$url.'">'.$lang->code.'</a>';
      $langList .= '<li>'.$html.'</li>';
    }
  }

  if ($langList != '')
  {
    $langList = '<ul class="flags">'.$langList.'</ul>';
  }

  return $langList;
}

?>
