<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls write rights by handling acl
 *
 * This file creates one hook :
 * <ul>
 *   <li>edit_acl : to edit acl of a particular page</li>
 * </ul>
 *
 * Edition is done via a form that contains several fields :
 * <ul>
 *   <li>a list of available groups : Those groups can be added to the acl</li>
 *   <li>a list of groups composing the current acl : Those groups can be removed from the acl</li>
 *   <li>a button to add more groups to the current acl</li>
 *   <li>a button to remove some groups from the current acl</li>
 *   <li>an optional "Apply recursively" checkbox. This checkbox allows the user to propagate the changement to all nodes that are hierarchically contained by this page</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>editACLLink</li>
 * </ul>
 */

$contentBoxHandle['edit_acl'] = 'editACL';

$userActionHandle['edit_acl'] = 'onEditACL';

$browserTitlePageHandle['edit_acl'] = 'browserEditAcl';

/**
 * \brief customize browser title when editing ACL
 *
 * \return browser title
 */
function browserEditAcl()
{
  return wc_wiki_name.' - '.tr('Editing ACL');
}

/**
 * \brief creates link to acl edition
 * \ingroup template_functions
 *
 * \return html link
 */
function editACLLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  global $user;

  if ($user->isModerator() || $user->isAdmin())
  {
    return '<a href="'.createURL($node->title, array('mode'=>'edit_acl', 'lang'=>$node->lang->code)).'">'.tr('Edit ACL').'</a>';
  }
}

function generateQuickSynthesis(&$node, $acl_desc, $acl_type)
{
  global $node;

  $url = createURL($node->title, array('mode'=>'edit_acl', 'lang'=>$node->lang->code, 'anchor'=>$acl_type));
  $blob = '<a href="'.$url.'">'.$acl_desc.'</a> : ';

  $acl_groups = array();
  if (!empty($node->groups[$acl_type]))
  {
    $acl_groups = $node->groups[$acl_type];
  }

  $acl_users  = array();
  if (isset($acl_groups[0]))
  {
    $acl_users = $acl_groups[0]->users;
    unset($acl_groups[0]);
  }

  if (count($acl_groups) > 0)
  {
    $first = true;
    foreach ($acl_groups as $group)
    {
      if ($first == false)
      {
        $blob .= ' + '; 
      }

      $blob .= tr('group').' '.$group->name;
      $first = false;
    }
  }
  else
  {
    $blob .= tr('no groups');
  }

  $blob .= '. ';

  if (count($acl_users) > 0)
  {
    $first = true;
    foreach ($acl_users as $user)
    {
      if ($first == false)
      {
        $blob .= ' + '; 
      }

      $blob .= tr('user').' '.$user;
      $first = false;
    }
  }
  else
  {
    $blob .= tr('no users');
  }

  $blob .= '.';

  return $blob;
}

function generateAclForm(&$all_groups, &$all_users, &$node, $acl_desc, $acl_type)
{
  $acl_groups = array();
  if (!empty($node->groups[$acl_type]))
  {
    $acl_groups = $node->groups[$acl_type];
  }

  $acl_users = array();
  if (isset($acl_groups[0]))
  {
    $acl_users = $acl_groups[0]->users;
    unset($acl_groups[0]);
  }

  // groups
  $available_groups = array();
  $used_groups = array();
  foreach ($all_groups as $group)
  {
    if (array_key_exists($group->id, $acl_groups))
    {
      $used_groups[$group->name] = $group->id;
    }
    else
    {
      $available_groups[$group->name] = $group->id;
    }
  }

  // users
  $available_users = array();
  $used_users = array();
  foreach ($all_users as $user)
  {
    if (array_key_exists($user[0], $acl_users))
    {
      $used_users[$user[1]] = $user[0];
    }
    else
    {
      $available_users[$user[1]] = $user[0];
    }
  }

  $form = new form(createURL($node->title, array('mode'=>'edit_acl', 'lang'=>$node->lang->code)), 'post', 'acl');
  $form->addFieldSet();
  $form->addCheckBox('rec_apply');
  $form->addHtml(tr('Recursively apply'));
  $form->addLegend(tr('Edit ACL').':'.$acl_desc);
  $form->addHtml('<div class="acl_available">');
  $form->addHtml('<h1>'.tr('Available users').'</h1><p>');
  $form->addListBox($available_users, array(), 'available_users', 10);
  $form->addHtml('</p><h1>'.tr('Available groups').'</h1><p>');
  $form->addListBox($available_groups, array(), 'available_groups', 10);
  $form->addHtml('</p>');
  $form->addHtml('</div>');
  $form->addHtml('<div class="acl_buttons">');
  $form->addButton('add', '&gt;&gt;');
  $form->addButton('remove', '&lt;&lt;');
  $form->addHtml('</div>');
  $form->addHtml('<div class="acl_used">');
  $form->addHtml('<h1>'.tr('Defined users').'</h1><p>');
  $form->addListBox($used_users, array(), 'used_users', 10);
  $form->addHtml('</p><h1>'.tr('Defined groups').'</h1><p>');
  $form->addListBox($used_groups, array(), 'used_groups', 10);
  $form->addHtml('</p>');
  $form->addHtml('</div>');
  $form->addHidden('acl_type', $acl_type);
  $form->addHidden('check', '1');
  $form->addHtml('<div class="spacer">&nbsp;</div>');

  return '<a name="'.$acl_type.'"></a>'.$form->toHtml();
}

/**
 * \brief display ACL form edition
 *
 * \return html blob
 */
function editACL()
{
  global $node;
  global $wiki;
  global $user;

  if (!$user->isAdmin() && !$user->isModerator())
  {
    return;
  }

  $all_groups = $wiki->getGroups();
  $all_users  = $wiki->getUsers();

  $blob  = '<h1>'.tr('Quick synthesis').'</h1><ul>';

  $blob .= '<li>'.generateQuickSynthesis($node, tr('Write'),   acl_write).'</li>';
  $blob .= '<li>'.generateQuickSynthesis($node, tr('Read'),    acl_read).'</li>';
  $blob .= '<li>'.generateQuickSynthesis($node, tr('Comment'), acl_comment).'</li>';

  global $pluginManager;
  $acl_plugins = $pluginManager->getPluginACLs();

  foreach ($acl_plugins as $acl_type=>$desc)
  {
    if ($acl_type >= acl_plugin_start)
    {
      $blob .= '<li>'.generateQuickSynthesis($node, $desc, $acl_type).'</li>';
    }
  }

  $blob .= '</ul>';

  $blob .= generateAclForm($all_groups, $all_users, $node, tr('Write'),   acl_write);
  $blob .= generateAclForm($all_groups, $all_users, $node, tr('Read'),    acl_read);
  $blob .= generateAclForm($all_groups, $all_users, $node, tr('Comment'), acl_comment);

  foreach ($acl_plugins as $acl_type=>$desc)
  {
    if ($acl_type >= acl_plugin_start)
    {
      $blob .= generateAclForm($all_groups, $all_users, $node, $desc, $acl_type);
    }
  }

  return $blob;
}

/**
 * \brief edit acl
 */
function onEditACL()
{
  global $node;
  global $wiki;
  global $user;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_POST['check']))
  {
    return;
  }

  if (!isset($_POST['acl_type']))
  {
    return;
  }

  $acl_type = varPost('acl_type');

  $recApply = isset($_POST['rec_apply']);

  if (isset($_POST['add']))
  {
    if (isset($_POST['available_users']))
    {
      $available_users = varPost('available_users');
      $wiki->addACLOnUsers($node->id, $available_users, $acl_type, $recApply);
    }
    if (isset($_POST['available_groups']))
    {
      $available_groups = varPost('available_groups');
      $wiki->addACLOnGroups($node->id, $available_groups, $acl_type, $recApply);
    }
  }
  else
  if (isset($_POST['remove']))
  {
    if (isset($_POST['used_users']))
    {
      $used_users = varPost('used_users');
      $wiki->removeACLOnUsers($node->id, $used_users, $acl_type, $recApply);
    }
    if (isset($_POST['used_groups']))
    {
      $used_groups = varPost('used_groups');
      $wiki->removeACLOnGroups($node->id, $used_groups, $acl_type, $recApply);
    }
  }

  header('Location: '.createHeaderURL($node->title, array('mode'=>'edit_acl', 'lang'=>$node->lang->code)));
}
?>