<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Several helper methods for page formatting
 *
 * This file creates no hook.
 */

/**
 * \brief format html so that l10n caracters (encoding) are properly displayed
 *
 * I'm searching for a simpler version because I guess this one is slow, but I didn't find any yet.
 *
 * \param string the string to format (both html entities and encoding)
 * \return formated string
 */
function formatHtml($string)
{
  $trans_tbl = get_html_translation_table(HTML_ENTITIES);
  $trans_tbl = array_flip($trans_tbl);

  $ret = strtr($string, $trans_tbl);

  return htmlspecialchars(
    preg_replace(
      '/&#(\d+);/me',
      "chr('\\1')",
      $ret
    )
  );
}

/**
 * \brief format a date
 *
 * \param timestamp the date to format
 * \return formatted string
 */
function formatDate($timestamp)
{
  $date = getdate($timestamp);

  $today = getdate();
  $yesterday = getdate(mktime(0, 0, 0, date('m') , date('d') - 1, date('Y')));

  if ($today['yday'] == $date['yday'] && $today['year'] == $date['year'])
  {
    $str = tr('Today');
  }
  else if ($yesterday['yday'] == $date['yday'] && $yesterday['year'] == $date['year'])
  {
    $str = tr('Yesterday');
  }
  else
  {
    $dayInWeek = ($date['wday'] + 6) % 7;
    $str = formatDay($dayInWeek).', '.$date['mday'].'. '.formatMonth($date['mon'] - 1).' '.$date['year'];
  }

  if ($date['hours'] != 0 || $date['minutes'] != 0 || $date['seconds'] != 0)
  {
    $str .= sprintf(' %02d:%02d:%02d', $date['hours'], $date['minutes'], $date['seconds']);
  }

  return $str;
}

/**
 * \brief format a link for author and email
 *
 * \param author
 * \param email (optional : maybe empty)
 * \return formatted link
 */
function formatAuthor($author, $email)
{
  if (!empty($email) && $email != '')
  {
    return '<a href="mailto:'.rawurlencode(formatEmail($email)).'">'.formatHtml($author).'</a>';
  }
  else
  {
    return formatHtml($author);
  }
}

/**
 * \brief format email
 *
 * \param email the email to encode
 * \return formatted email
 */
function formatEmail($email)
{
  $result = str_replace('.', ' __dot__ ', $email);
  $result = str_replace('@', ' __at__ ', $result);

  return $result;
}

/**
 * \brief format name
 *
 * \param login
 * \param first_name
 * \param last_name
 *
 * \return formatted information
 */
function formatName($login, $first_name, $last_name)
{
  if (empty($first_name) && empty($last_name))
    return $login;
  if (empty($first_name))
    return $last_name;
  if (empty($last_name))
    return $first_name;
  return $first_name." ".$last_name;
}

/**
 * \brief format a size
 *
 * \param size size to format
 * \return formatted size
 */
function formatSize($size)
{
  if ($size < 1024)
  {
    $unit = tr('bytes');
  }
  else
  {
    $size /= 1024;

    if ($size < 1024)
    {
      $unit = tr('kb');
    }
    else
    {
      $size /= 1024;
      $unit = tr('Mb');
    }
  }

  return sprintf('%.2f', $size).$unit;
}

/**
 * \brief format a month according to i18n
 *
 * \param month month to format 
 */
function formatMonth($month)
{
  $monthText = array(
    tr('January'),
    tr('February'),
    tr('March'),
    tr('April'),
    tr('May'),
    tr('June'),
    tr('July'),
    tr('August'),
    tr('September'),
    tr('October'),
    tr('November'),
    tr('December')
  );

  $month = $month % 12;

  return $monthText[$month];
}

/**
 * \brief format a day according to i18n
 *
 * \param day day to format 
 */
function formatDay($day)
{
  $dayText = array(
    tr('Monday'),
    tr('Tuesday'),
    tr('Wednesday'),
    tr('Thursday'),
    tr('Friday'),
    tr('Saturday'),
    tr('Sunday')
  );

  $day = $day % 7;

  return $dayText[$day];
}

/**
 * \brief format a day according to i18n with lesser letter
 *
 * \param day day to format
 */
function formatSmallDay($day)
{
  $dayText = array(
    tr('Mon'),
    tr('Tue'),
    tr('Wen'),
    tr('Thu'),
    tr('Fri'),
    tr('Sat'),
    tr('Sun')
  );

  $day = $day % 7;

  return $dayText[$day];
}
?>
