<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls history viewing/diffing
 *
 * This file creates several hooks :
 * <ul>
 *   <li>history : to display list of available history</li>
 *   <li>history_view : to view a particular version by history id</li>
 *   <li>history_view_by_date : to view a particular version by date (the precision is the day)</li>
 *   <li>history_diff : to diff a version against another</li>
 *   <li>history_del : to delete a particular version. If the current version is deleted, then the latest version is restored as current content</li>
 * </ul>
 *
 * History viewing is done via a table. Each line is a particular version.
 * At the end of each line there is a radio button that allows the user to select 2 different versions.
 * <ul>
 * <li>If the user clicks on the version, or on the title, then the version is shown by history id.</li>
 * <li>If the user clicks on the date, then the version is shown by date (and the url used is a permalink).</li>
 * <li>If the user clicks on the "Show diff" button, then the two versions selected are diffed, which allows the user to see differences that happened between the versions</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>historyLink</li>
 * </ul>
 */

$contentBoxHandle['history']                    = 'history';
$contentBoxHandle['history_view']               = 'historyView';
$contentBoxHandle['history_view_by_date']       = 'historyViewByDate';
$contentBoxHandle['history_diff']               = 'historyDiff';
$contentBoxHandle['history_del']                = 'historyDelete';
$contentBoxHandle['history_del_all']            = 'historyDeleteAll';

$userActionHandle['history_del_confirm']        = 'onHistoryDelete';
$userActionHandle['history_del_all_confirm']    = 'onHistoryDeleteAll';

$browserTitlePageHandle['history']              = 'browserHistory';
$browserTitlePageHandle['history_view']         = 'browserHistoryView';
$browserTitlePageHandle['history_view_by_date'] = 'browserHistoryViewByDate';
$browserTitlePageHandle['history_diff']         = 'browserHistoryDiff';

/**
 * \brief customize browser title when viewing history
 *
 * \return browser title
 */
function browserHistory()
{
  global $node;
  return wc_wiki_name.' - '.$node->title.' - '.tr('History');
}

/**
 * \brief customize browser title when viewing a version of the content
 *
 * \return browser title
 */
function browserHistoryView()
{
  global $node;

  if (!isset($_GET['id']))
  {
    return;
  }

  $id = varGet('id');
  return wc_wiki_name.' - '.$node->title.' - '.tr('Version').' '.$id;
}

/**
 * \brief customize browser title when viewing history at a date
 *
 * \return browser title
 */
function browserHistoryViewByDate()
{
  global $node;

  if (!isset($_GET['year']) || !isset($_GET['month']) || !isset($_GET['day']))
  {
    return;
  }

  $year  = varGet('year');
  $month = varGet('month');
  $day   = varGet('day');

  return wc_wiki_name.' - '.$node->title.' - '.tr('at').' '.$year.'/'.$month.'/'.$day;
}

/**
 * \brief customize browser title when diffing versions
 *
 * \return browser title
 */
function browserHistoryDiff()
{
  global $node;

  if (!isset($_POST['diff1']) || !isset($_POST['diff2']))
  {
    return;
  }

  $id1 = varPost('diff1');
  $id2 = varPost('diff2');
  if ($id2 == -1)
  {
    $id2 = 'current';
  }

  return wc_wiki_name.' - '.$node->title.' - '.tr('Diff between version').' '.$id1.' '.tr('and version').' '.$id2;
}

require wc_inc_path.'/inc/classes/diff.class.php';

/**
 * \brief creates a link to history
 * \ingroup template_functions
 *
 * \return html link
 */
function historyLink()
{
  global $mode;

  if (!empty($mode))
  {
    return '';
  }

  global $node;

  if (empty($node))
  {
    return '';
  }

  return '<a href="'.createURL($node->title, array('mode'=>'history', 'lang'=>$node->lang->code)).'">'.tr('View history').'</a>';
}

/**
 * \brief display history versions
 *
 * \return html blob
 */
function history()
{
  global $node;
  global $wiki;
  global $user;
  global $tableRenderer;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return errorMessageACLRead();
  }

  $history = array();
  $history[] = $node;
  $history = array_merge($history, $wiki->getHistory($node->id));

  $form = new form(createURL($node->title, array('mode'=>'history_diff', 'lang'=>$node->lang->code)), 'post', 'diff');
  $form->addHtml('<div id="diff">');
  $form->addHtml('<p>');
  //sdw - add format of diff selection
  $form->addLabel(tr('Diff format'), 'fformat');
  $form->addHtml('&nbsp;');
  $default = 'inline';
  $options = array(
    tr('Inline comparison')       => 'inline',
    tr('Unified comparison')      => 'unified',
  );
  if (wc_table_rendering == 'table')
  {
    $options[tr('Side by side comparison')] = 'table';
    $default = 'table';
  }
  $form->addDropDown($options, $default , 'format', 'fformat');
  $form->addButton('diff', tr('Diff'), 'submit');
  $form->addHtml('</p>');

  if ($user->isAdmin() || $user->isModerator())
  {
    $deleteAllUrl = '<a href="'.createUrl($node->title, array('mode'=>'history_del_all', 'id'=>$node->id, 'lang'=>$node->lang->code)).'">'.tr('Delete All').'</a>';
    $form->addHtml('<p>'.$deleteAllUrl.'</p>');
  }

  $form->addHtml('<hr />');

  $columns = array();
  {
    $columns[] = tr('Title');
    $columns[] = tr('Version');
    $columns[] = tr('Author');
    $columns[] = tr('Date');
    $columns[] = tr('Major/Minor');
    $columns[] = tr('Comment');

    if ($user->isAdmin() || $user->isModerator())
    {
      $columns[] = tr('Delete').'/'.tr('Ban');
    }

    $columns[] = tr('Diff from');
    $columns[] = tr('Diff to');
  }

  $table = new Table($columns);

  $even = true;
  $i = 0;
  $count = sizeof($history);

  foreach ($history as $n)
  {
    if ($i == 0)
    {
      $class = 'first';
      $version = tr('Current');
    }
    else
    {
      $version = $count - $i;

      if ($even)
      {
        $class = 'even';
      }
      else
      {
        $class = 'uneven';
      }
    }

    $select1checked = false;
    $select2checked = false;
    $id = $n->id;
    if ($i == 0)
    {
      $id = -1;
      $url = createURL($node->title, array('lang'=>$node->lang->code));
      $select2checked = true;
    }
    else
    {
      if ($i == 1)
      {
        $select1checked = true;
      }
      $url = createURL($node->title, array('mode'=>'history_view', 'id'=>$id, 'lang'=>$node->lang->code));
    }

    if ($user->isAdmin() || $user->isModerator())
    {
      if ($count > 1)
      {
        $deleteUrl = '<a href="'.createUrl($node->title, array('mode'=>'history_del', 'id'=>$id, 'lang'=>$node->lang->code)).'">'.tr('Delete').'</a>';
      }
      else
      {
        $deleteUrl = '';
      }

      $banUrl = '<a href="'.createUrl($node->title, array('mode'=>'ban_add', 'ip'=>$n->ip, 'lang'=>$node->lang->code, 'old_mode'=>'history')).'">'.tr('Ban').' '.$n->ip.'</a>';
    }

    $year   = date('Y', $n->timestamp);
    $month  = date('m', $n->timestamp);
    $day    = date('d', $n->timestamp);

    $urlByDate = createURL(
      $node->title,
      array(
        'mode'=>'history_view_by_date',
        'lang'=>$node->lang->code,
        'year'=>$year,
        'month'=>$month,
        'day'=>$day,
      )
    );

    $row = array();

    $row[] = '<a href="'.$url.'">'.formatHtml($n->title).'</a>';
    $row[] = '<a href="'.$url.'">'.$version.'</a>';
    $row[] = formatName($n->login, $n->first_name, $n->last_name);
    $row[] = '<a href="'.$urlByDate.'">'.formatDate($n->timestamp).'</a>';
    $row[] = ($n->minor_major==minor_value?tr('minor'):tr('major'));
    $row[] = formatHtml($n->commit_msg);

    if ($user->isAdmin() || $user->isModerator())
    {
      $row[] = (empty($deleteUrl) ? '' : $deleteUrl.'/').$banUrl;
    }

    if ($i != 0)
    {
      $row[] = $form->generateRadioButton('diff1', $n->id, $select1checked);
    }
    else
    {
      $row[] = '';
    }

    $row[] = $form->generateRadioButton('diff2', $id, $select2checked);

    $table->push_row($row, $class);

    $even =  !$even;
    ++$i;
  }

  $form->addHtml($tableRenderer->render($table));
  $form->addHtml('</div>');

  return $form->toHtml();
}

/**
 * \brief display a particular history version by id
 *
 * \return html blob
 */
function historyView()
{
  global $wiki;
  global $node;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return errorMessageACLRead();
  }

  if (!isset($_GET['id']))
  {
    return;
  }

  $id = varGet('id');

  $old_node = $wiki->getHistoryVersion($id);
  return $old_node->toHtml(false, false);
}

/**
 * \brief display a particular history version by date
 *
 * the precision is the date. The displayed version is the last version of the day.
 * It is possible to go beyond subsecond precision, but it will need to add more arguments
 * to url (at least hour, min, sec)
 *
 * \return html blob
 */
function historyViewByDate()
{
  global $wiki;
  global $node;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return errorMessageACLRead();
  }

  if (!isset($_GET['year']) || !isset($_GET['month']) || !isset($_GET['day']))
  {
    return;
  }

  $year  = varGet('year');
  $month = varGet('month');
  $day   = varGet('day');

  $timestamp = mktime(23, 59, 59, $month, $day, $year);

  $old_node = $wiki->getHistoryByDate($node->id, $timestamp);
  return $old_node->toHtml(false, false);
}

/**
 * \brief create title to display differences
 *
 * \param node the node to create title for
 * \return title
 */
function createTitle($node)
{
  return
  '<p>'.tr('Revision as of').' '.formatDate($node->timestamp).'</p>'.
  '<p>'.tr('By').' '.formatName($node->login, $node->first_name, $node->last_name).'</p>';
}

/**
 * \brief perform diff and display it
 *
 * \return html blob
 */
function historyDiff()
{
  global $wiki;
  global $node;
  global $user;

  $canRead =  auth::canRead($user, $node);
  if ($canRead == false)
  {
    return errorMessageACLRead();
  }

  if (!isset($_POST['diff1']) || !isset($_POST['diff2']))
  {
    return;
  }

  //steve - choose view type
  $type = isset($_POST['format']) ? $_POST['format'] : 'table';

  $id1 = varPost('diff1');
  $id2 = varPost('diff2');
  $html = '';

  $node1 = $wiki->getHistoryVersion($id1);
  $title1 = createTitle($node1);

  if ($id2 == -1)
  {
    $node2 = $node;
    $title2 =
    '<p>'.tr('Current version').'</p>'.
    '<p>'.tr('By').' '.formatName($node->login, $node->first_name, $node->last_name).'</p>';
  }
  else
  {
    $node2 = $wiki->getHistoryVersion($id2);
    $title2 = createTitle($node2);
  }

  $previous = explode("\n", formatHtml($node1->content));
  $current = explode("\n", formatHtml($node2->content));

  //steve - new diff process

  //include all required files from pear Text_Diff
  require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff.php';
  require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff/Engine/native.php';
  require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff/Renderer.php';

  $diff = new Text_Diff($previous, $current);

   //Uncomment to view diff array
   /*
    echo '<pre>';
    print_r($diff);
    echo '</pre>';
   */

  if ($type == 'inline')
  {
    require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff/Renderer/inline.php';
    require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff/Renderer/inline2.php';
    $renderer = new Text_Diff_Renderer_inline2();
    $html .= '<h1>'.tr('Diff').' - '.tr('Inline comparison').'</h1>'."\n";
    $html .= '<div class="difftitle">'.$title2.'</div>'."\n";
    $html .= '<div class="difftitle">'.$title1.'</div>'."\n";
    $html .= $renderer->render($diff);
  }

  // This choice is not available when useTable is turn off
  if ($type == 'table')
  {
    require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff/Renderer/table.php';
    $html .= '<h1>'.tr('Diff').' - '.tr('Side by side comparison').'</h1>'."\n";
    $renderer = new Text_Diff_Renderer_table();
    $html .= $renderer->render($diff);
    //replace tiles in returned rendering of diff with the titles generated by wiclear
    $html = str_replace('<th>New</th>','<th>'.$title2.'</th>', $html);
    $html = str_replace('<th>Old</th>','<th>'.$title1.'</th>', $html);
  }

  if ($type == 'unified')
  {
    require wc_inc_path.'/inc/classes/PEAR/Text_Diff/Diff/Renderer/unified.php';
    $renderer = new Text_Diff_Renderer_unified();
    $html .= '<h1>'.tr('Diff').' - '.tr('Unified').'</h1>'."\n";
    $html .= '<div class="difftitle">'.$title2.'</div>'."\n";
    $html .= '<div class="difftitle">'.$title1.'</div>'."\n";
    $html .= '<pre>';
    $html .= $renderer->render($diff);
    $html .= '</pre>';
  }

  if ($diff->isEmpty())
  {
    $html .= '<p>'.tr('There is no differences between the 2 versions').'</p>'."\n";
  }

  return $html;
}

/**
 * \brief Confirm delete history
 *
 * \return html blob
 */
function historyDelete()
{
  global $wiki;
  global $node;
  global $user;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET['id']))
  {
    return;
  }
  $id = varGet('id');

  $html  = '<h1>'.tr('Confirm delete history').'</h1>';
  $html .= '<p>'.tr('This will permentally delete this version of the page from the wiki, you will not be able to undo this action.').'</p>';
  $html .= '<p>'.tr('Please confirm you wish to delete this version of').' "'.formatHtml($node->title).'"</p>';
  $delete_link = '<a href="'.createURL($node->title, array('mode'=>'history_del_confirm', 'lang'=>$node->lang->code, 'id'=>$id)).'">'.
                tr('Yes (delete)').'</a>';
  $cancel_link = '<a href="'.createURL($node->title, array('mode'=>'history', 'lang'=>$node->lang->code)).'">'.tr('No (take me back)').'</a>';
  $html .= '<p>'.$delete_link.'&nbsp;|&nbsp;'.$cancel_link.'</p>';
  return $html;
}

/**
 * \brief delete history from history form
 */
function onHistoryDelete()
{
  global $user;
  global $node;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET['id']))
  {
    return;
  }

  global $wiki;

  $id = varGet('id');

  if ($id == -1)
  {
    // this means we want to restore last version and delete current one
    $history = $wiki->getHistory($node->id);
    if (sizeof($history) > 0)
    {
      $lastVersion = $history[0];

      $fake_user = new user(
        $lastVersion->user_id,
        $lastVersion->login,
        '', /*password*/
        user_value,
        $lastVersion->first_name,
        $lastVersion->last_name,
        $lastVersion->email,
        $lastVersion->lang->id,
        $lastVersion->lang->id,
        time()
      );

      $wiki->editContent(
        $node->id, $node->parent_id, $lastVersion->parent_id,
        $lastVersion->title, $lastVersion->alternate_title, $lastVersion->content,
        $lastVersion->commit_msg, $lastVersion->minor_major,
        $node->related_content_id,
        $fake_user,
        $node->left, $node->right
      );

      // overload title so that redirection correctly works
      $node->title = $lastVersion->title;

      $id = $lastVersion->id;
    }
    else
    {
      $id = -1;
    }
  }

  $wiki->deleteHistory($id);

  header('Location: '.createHeaderURL($node->title, array('mode'=>'history', 'lang'=>$node->lang->code)));
}

/**
 * \brief Confirm delete all history
 *
 * \return html blob
 */
function historyDeleteAll()
{
  global $wiki;
  global $node;
  global $user;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET['id']))
  {
    return;
  }
  $id = varGet('id');

  $html  = '<h1>'.tr('Confirm delete all history').'</h1>';
  $html .= '<p>'.tr('This will permentally delete all previous versions of the page from the wiki, you will not be able to undo this action.').'</p>';
  $html .= '<p>'.tr('Please confirm you wish to delete all previous versions of').' "'.formatHtml($node->title).'"</p>';
  $delete_link = '<a href="'.createURL($node->title, array('mode'=>'history_del_all_confirm', 'lang'=>$node->lang->code, 'id'=>$id)).'">'.
                tr('Yes (delete)').'</a>';
  $cancel_link = '<a href="'.createURL($node->title, array('mode'=>'history', 'lang'=>$node->lang->code)).'">'.tr('No (take me back)').'</a>';
  $html .= '<p>'.$delete_link.'&nbsp;|&nbsp;'.$cancel_link.'</p>';
  return $html;
}

/**
 * \brief delete all history from history form
 */
function onHistoryDeleteAll()
{
  global $user;
  global $node;

  if ((!$user->isAdmin() && !$user->isModerator()) || !isset($_GET['id']))
  {
    return;
  }

  global $wiki;

  $id = varGet('id');

  // button delete all history has been used
  $wiki->deleteAllHistory($id);
  header('Location: '.createHeaderURL($node->title, array('mode'=>'history', 'lang'=>$node->lang->code)));
}
?>
