<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls logging in/logging out
 *
 * This file creates several hooks :
 * <ul>
 *   <li>login : to log'in</li>
 *   <li>logoff : to log'out</li>
 * </ul>
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>loginLink</li>
 *   <li>registerLink</li>
 *   <li>preferencesLink</li>
 *   <li>logoffLink</li>
 *   <li>userMenu</li>
 * </ul>
 */

$contentBoxHandle['login']  = 'loginBox';

$userActionHandle['login']  = 'onLogin';
$userActionHandle['logoff'] = 'onLogoff';

$browserTitlePageHandle['login'] = 'browserLogin';

/**
 * \brief customize browser title when logging in
 *
 * \return browser title
 */
function browserLogin()
{
  return wc_wiki_name.' - '.tr('Login');
}

/**
 * \brief creates a login link
 * \ingroup template_functions
 *
 * \return html link
 */
function loginLink()
{
  global $user;
  global $title;

  if ($user->isAnonymous())
  {
    return '<a href="'.createURL($title, array('mode'=>'login')).'">'.tr('Login').'</a>';
  }
}

/**
 * \brief creates a register link
 * \ingroup template_functions
 *
 * \return html link
 */
function registerLink()
{
  global $user;
  global $title;

  if ($user->isAnonymous() && wc_wiki_open_registration == 'true')
  {
    return '<a href="'.createURL($title, array('mode'=>'register')).'">'.tr('Register').'</a>';
  }
}

/**
 * \brief creates a preferences link
 * \ingroup template_functions
 *
 * \return html link
 */
function preferencesLink()
{
  global $user;
  global $title;

  if (!$user->isAnonymous())
  {
    return '<a href="'.createURL($title, array('mode'=>'user_pref')).'">'.tr('Preferences').'</a>';
  }
}

/**
 * \brief creates a logoff link
 * \ingroup template_functions
 *
 * \return html link
 */
function logoffLink()
{
  global $user;
  global $title;

  if (!$user->isAnonymous())
  {
    return '<a href="'.createURL($title, array('mode'=>'logoff')).'">'.tr('Logoff').'</a>';
  }
}

/**
 * \brief creates a user link as an unordered list
 * \ingroup template_functions
 *
 * the menu may contain (depending on global configuration, user login ...)
 * - login link
 * - logged in information
 * - preferences link
 * - register link
 * - logoff link
 *
 * \return html unordered list
 */
function userMenu()
{
  $blob = '<ul>';

  $loginLink = loginLink();
  if (!empty($loginLink))
  {
    $blob .= '<li class="login">'.$loginLink.'</li>';
  }
  else
  {
    global $user;
    $blob .= '<li class="logged">'.tr('Welcome').' '.$user->login.'</li>';

    $preferencesLink = preferencesLink();
    $blob .= '<li class="preferences">'.$preferencesLink.'</li>';
  }

  $registerLink = registerLink();
  if (!empty($registerLink))
  {
    $blob .= '<li class="register">'.$registerLink.'</li>';
  }

  $logoffLink = logoffLink();
  if (!empty($logoffLink))
  {
    $blob .= '<li class="logoff">'.$logoffLink.'</li>';
  }

  $blob .= '</ul>';

  return $blob;
}

/**
 * \brief display a login form
 *
 * \return html blob
 */
function loginBox()
{
  global $wiki;
  global $title;

  $login    = '';
  $password = '';

  if (isset($_POST['check']))
  {
    $login    = varPost('login');
    $password = varPost('password');
  }

  $form = new form(createURL($title, array('mode'=>'login')), 'post', 'login');
  $form->addFieldset();
  $form->addLegend(tr('Login'));

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('login'), 'flogin');
  $form->addInput('login', 'Login', $login, 'flogin');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addLabel(tr('Password'), 'fpassword');
  $form->addPassword('password', $password, 'fpassword');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addCheckbox('remember_login');
  $form->addLabel(tr('Remember login'));
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $ips = implode(',', auth::getIPs());
  setVarSession('login_ips', $ips);

  $form->addHtml('<p class="field">');
  $form->addHidden('ips', $ips);
  $form->addHtml('</p>');

  $random = auth::generateRandomText(255);
  setVarSession('login_random', $random);

  setVarSession('login_date', time());

  $form->addHtml('<p class="field">');
  $form->addHidden('random', $random);
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('submit', tr('Login'));
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  return $form->toHtml();
}

/**
 * \brief called when the user submit the login form
 */
function onLogin()
{
  global $wiki;
  global $title;
  global $user_lang;

  if (!isset($_POST['check']) || !isset($_POST['ips']) || !isset($_POST['random']) ||
      !isset($_POST['login']) || !isset($_POST['password'])
     )
  {
    return;
  }

  $form_checked_ok = true;
  $ips = varPost('ips');
  if (!existVarSession('login_ips') || $ips != varSession('login_ips'))
  {
    $form_checked_ok = false;
  }

  $random = varPost('random');
  if (!existVarSession('login_random') || $random != varSession('login_random'))
  {
    $form_checked_ok = false;
  }

  $date = time();
  if (!existVarSession('login_date') || ($date - varSession('login_date')) > wc_login_timeout)
  {
    // only 10 sec allowed to login
    $form_checked_ok = false;
  }

  unsetVarSession('login_ip');
  unsetVarSession('login_random');
  unsetVarSession('login_date');
  if ($form_checked_ok == true)
  {
    $login    = varPost('login');
    $password = varPost('password');

    $user = $wiki->findUser($login, $password);
    $user_lang = $user->lang;

    if (!$user->isAnonymous())
    {
      // logged in
      // set the required info in session
      setVarSession('user_id', $user->id);
      setVarSession('user_password', $user->password);

      if (isset($_POST['remember_login']))
      {
        $expire = time() + 60*60*24*30; // expire in one month
        setVarCookie('login[user_id]',       $user->id,       $expire);
        setVarCookie('login[user_password]', $user->password, $expire);
      }

      session_write_close();
    }
  }

  header('Location: '.createHeaderURL($title, array('lang'=>$user_lang)));
}

/**
 * \brief called when the user click the logoff link
 */
function onLogoff()
{
  global $title;
  global $user_lang;

  unsetVarSession('user_id');
  unsetVarSession('user_password');

  setVarCookie('login[user_id]', false);
  setVarCookie('login[user_password]', false);

  session_write_close();
  header('Location: '.createHeaderURL($title, array('lang'=>$user_lang)));
}
?>
