<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls metadata handling
 *
 * This file creates several hooks :
 * <ul>
 *   <li>metadata_view : to view the list of defined metadata</li>
 *   <li>metadata_add : to define a new metadata value</li>
 *   <li>metadata_delete : to delete a metadata</li>
 * </ul>
 *
 * Metadatas must first have been defined in the admin panel.
 * Once defined, they can be displayed, added, removed on particular page.
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>metadataBox</li>
 * </ul>
 */

$contentBoxHandle['metadata_view']   = 'metadataDisplay';

$userActionHandle['metadata_add']    = 'onMetadataAdd';
$userActionHandle['metadata_delete'] = 'onMetadataDelete';

$browserTitlePageHandle['metadata_view'] = 'browserMetadataDisplay';

/**
 * \brief customize browser title when viewing metadatas
 *
 * \return browser title
 */
function browserMetadataDisplay()
{
  global $node;
  return wc_wiki_name.' - '.$node->title.' - '.tr('Metadatas');
}

/**
 * \brief display a link to metadata
 * \ingroup template_functions
 *
 * \return html link
 */
function metadataBox()
{
  global $title;
  global $user;
  global $node;

  if (!empty($node))
  {
    $canEdit = auth::canEdit($user, $node);
    if ($canEdit)
    {
      return '<a href="'.createURL($title, array('mode'=>'metadata_view')).'">'.tr('Metadatas').'</a>';
    }
  }
}

/**
 * \brief display list of metadata and a form to add new metadatas
 *
 * \return html blob
 */
function metadataDisplay()
{
  global $title;
  global $wiki;
  global $node;
  global $user;

  $canEdit = auth::canEdit($user, $node);
  if (!$canEdit)
  {
    return;
  }

  // list of metadata available
  $metadatas = $wiki->getMetaDatas();

  $open_metadata = array();
  $closed_metadatas_ids = array();
  $closed_metadatas_names = array();

  // separate list of open and closed metadatas
  foreach ($metadatas as $m)
  {
    if ($m->type == metadata_open)
    {
      $open_metadata[$m->name] = $m->id;
    }
    else
    {
      $closed_metadatas_ids[] = $m->id;
      $closed_metadatas_names[$m->id] = $m->name;
    }
  }

  // retrieve possible values for each closed metadatas
  // list<metadata_id, list<metadata_value_id, value>>
  $closed_metadata_values = $wiki->getClosedMetadataValuesByIds($closed_metadatas_ids);

  // retrieve list of values defined on this node
  $nodeMetadatas = $wiki->getMetaDataInfoByContentId($node->id);

  if (empty($nodeMetadatas))
  {
    $blob = tr('No metadata on this node at the moment');
  }
  else
  {
    $blob =
    '<table>'.
    '<tr><th>'.tr('Name').'</th><th>'.tr('Value').'</th>';

    global $user;
    if ($user->isModerator() || $user->isAdmin())
    {
      $blob .= '<th>'.tr('Delete').'</th>';
    }

    $blob .= '</tr>';

    foreach ($nodeMetadatas as $idNameIdValue)
    {
      $id       = $idNameIdValue[0]; // metadata.id
      $name     = $idNameIdValue[1]; // metadata.name
      $valueId  = $idNameIdValue[2]; // metadata_content.id
      $value    = $idNameIdValue[3]; // metadata_content.value
      $value_fk = $idNameIdValue[4]; // metadata_value.id

      if (!empty($closed_metadatas_names[$id]))
      {
        $value = $closed_metadata_values[$id][$value_fk];
      }

      $blob .= '<tr>';
      $blob .= '<td>'.$name.'</td><td>'.$value.'</td>';

      if ($user->isModerator() || $user->isAdmin())
      {
        $url = createURL($title, array('mode'=>'metadata_delete', 'id'=>$valueId));
        $blob .= '<td><a href="'.$url.'">'.tr('Delete').'</a></td>';
      }

      $blob .= '</tr>';
    }
    $blob .= '</table>';
  }

  $blob .= '<hr/>';

  $form = new form(createURL($title, array('mode'=>'metadata_add')), 'post');
  $form->addHtml($blob);

  $form->addFieldSet();
  $form->addLegend(tr('Add metadata'));

  foreach ($open_metadata as $name=>$id)
  {
    $form->addHtml('<p class="field">');
    $form->addLabel($name, 'fmetadata_name_'.$name);
    $form->addInput('metadata_value_open_'.$id, 'Value', '', 'fmetadata_name_'.$name);
    $form->addHtml('</p>');
  }

  foreach ($closed_metadata_values as $id=>$valueList)
  {
    $name = $closed_metadatas_names[$id];

    $valueList[0] = '--';

    $valueList = array_flip($valueList);

    $form->addHtml('<p class="field">');
    $form->addLabel($name, 'fmetadata_'.$name);
    $form->addDropDown($valueList, '0', 'metadata_value_close_id_'.$id, 'fmetadata_'.$name);
    $form->addHtml('</p>');
  }

  $form->addHtml('<p class="field">');
  $form->addHidden('check', '1');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field">');
  $form->addButton('add_meta', tr('Add metadata'), 'action');
  $form->addHtml('</p>');

  $form->addHtml('<p class="field"/>');

  return $form->toHtml();
}

/**
 * \brief called when the user add a new metadata in metadataDisplay() form
 */
function onMetadataAdd()
{
  global $node;
  global $user;

  $canEdit = auth::canEdit($user, $node);
  if (!$canEdit)
  {
    return;
  }

  if (!isset($_POST['check']))
  {
    return;
  }

  global $wiki;
  global $title;

  $metadatas = $wiki->getMetaDatas();
  foreach ($metadatas as $m)
  {
    if ($m->type == metadata_close)
    {
      $closed_id = varPost('metadata_value_close_id_'.$m->id);
      if ($closed_id != 0)
      {
        $wiki->addContentMetadataClosedValue($node->id, $m->id, $closed_id);
      }
    }
    else
    {
      $open_value = varPost('metadata_value_open_'.$m->id);
      if (!empty($open_value))
      {
        $wiki->addContentMetaDataValue($node->id, $m->id, $open_value);
      }
    }
  }

  header('Location: '.createHeaderURL($title, array('mode'=>'metadata_view')));
}

 /**
 * \brief called when the user delete a new metadata in metadataDisplay() form
 */
function onMetadataDelete()
{
  global $node;
  global $user;

  $canEdit = auth::canEdit($user, $node);
  if (!$canEdit)
  {
    return;
  }

  if (!isset($_GET['id']))
  {
    return;
  }

  if ($user->isModerator() || $user->isAdmin())
  {
    $id = varGet('id');

    global $wiki;
    global $title;
    $wiki->delContentMetadataValue($id);
  }

  header('Location: '.createHeaderURL($title, array('mode'=>'metadata_view')));
}
?>