<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief handles url with different modes
 *
 * mode is defined in wiki global configuration. It can be get, query_string or path_info
 * urls get decoded so that the _GET global array is populated just as only 'get' url were used.
 * This way all remaining code don't see urls.
 */

/**
 * \brief create path to web application
 *
 * \param path path to application from web's root
 * \return web url to application
 */
function createWebAppPath($path = wc_path)
{
  $protocol = 'http';
  $port = 80;

  if (isset($_SERVER['HTTPS']))
  {
    $protocol = 'https';
    $port = 443;
  }

  $portExt = '';
  if (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] != $port)
  {
    $portExt = ':'.$_SERVER['SERVER_PORT'];
  }

  $path = $protocol.'://'.$_SERVER['SERVER_NAME'].$portExt.$path;
  if (!preg_match('!.*/$!', $path))
  {
    // no terminal '/' : add one
    $path .= '/';
  }

  return $path;
}

/**
 * \brief urlArg a list of argument separated by '/'
 *
 * all arguments will be stored in global array _GET
 */
function decodeArguments($urlArg)
{
  // format supported :
  // title
  // lang/title
  // year/month/day/title
  // lang/year/month/day/title
  // count number of arguments

  if (empty($urlArg))
  {
    return false;
  }

  $args = explode('/', $urlArg);
  $nbArgs = sizeof($args);
  for ($i = 0; $i < $nbArgs; ++$i)
  {
    $args[$i] = rawurldecode($args[$i]);
  }

  // other depends on number
  $langRe  = '[a-z]{2,3}';
  $yearRe  = '[0-9]{4}';
  $monthRe = '[0-9]{2}';
  $dayRe   = '[0-9]{2}';
  $catRe   = '[a-zA-Z0-9_-]+';

  $compatible = false;
  // title
  if ($nbArgs == 1 && strpos($urlArg, '&') === false && strpos($urlArg, '=') === false)
  {
    $_GET['title'] = $args[0];
    $compatible = true;
  }
  else
  // lang/title
  if ($nbArgs == 2 && preg_match('!'.$langRe.'/.*!', $urlArg, $match))
  {
    $_GET['lang']  = $args[0];
    $_GET['title'] = $args[1];
    $compatible = true;
  }
  else
  // year/month/day/title
  if ($nbArgs == 4 && preg_match('!'.$yearRe.'/'.$monthRe.'/'.$dayRe.'/.*!', $urlArg, $match))
  {
    $_GET['year']  = $args[0];
    $_GET['month'] = $args[1];
    $_GET['day']   = $args[2];
    $_GET['title'] = $args[3];
    $_GET['mode']  = 'history_view_by_date';
    $compatible = true;
  }
  else
  // lang/year/month/day/title
  if ($nbArgs == 5 && preg_match('!'.$langRe.'/'.$yearRe.'/'.$monthRe.'/'.$dayRe.'/.*!', $urlArg, $match))
  {
    $_GET['lang']  = $args[0];
    $_GET['year']  = $args[1];
    $_GET['month'] = $args[2];
    $_GET['day']   = $args[3];
    $_GET['title'] = $args[4];
    $_GET['mode']  = 'history_view_by_date';
    $compatible = true;
  }
/*
  else
  // lang/categorie/year/month/day/title
  if ($nbArgs == 6 && preg_match('!'.$langRe.'/'.$catRe.'/'.$yearRe.'/'.$monthRe.'/'.$dayRe.'/.*!', $urlArg, $match))
  {
    $_GET['lang']  = $args[0];
    $_GET['cat']   = $args[1];
    $_GET['year']  = $args[2];
    $_GET['month'] = $args[3];
    $_GET['day']   = $args[4];
    $_GET['mode']  = 'history_view_by_date';
  }
*/
/*
  if (isset($_GET['lang']))  echo '<pre>lang='.$_GET['lang'].'</pre>';
  if (isset($_GET['cat']))   echo '<pre>cat='.$_GET['cat'].'</pre>';
  if (isset($_GET['year']))  echo '<pre>year='.$_GET['year'].'</pre>';
  if (isset($_GET['month'])) echo '<pre>month='.$_GET['month'].'</pre>';
  if (isset($_GET['day']))   echo '<pre>day='.$_GET['day'].'</pre>';
*/

  return $compatible;
}

/**
 * \brief try to decode url as a pathinfo url
 */
function decodePathInfoUrl()
{
  // if url start with configured path
  $url = $_SERVER['REQUEST_URI'];
  // do not rawurldecode here, let decodeArguments do it because url may contain both
  // '/' and '%2F' which when decoded will lead to '/' too
  // if we rawurldecode here we won't be able to make the difference between a '/' for the date
  // for example and a url-encoded '/' (%2F) in the title
  if (preg_match('/^'.preg_quote(wc_path, '/').'(.*)$/', $url, $match))
  {
    // remove first '/' and last '/' if any
    $url = preg_replace('!^[/]?(.*)[/]?$!', '$1', $match[1]);
    // remove index.php/ if needed (wc_app_path defaults to 'index.php')
    if (wc_url_format == 'path_info')
    {
      if (!preg_match('!^('.preg_quote(wc_app_path, '!').')?/(.*)$!', $url, $match))
      {
        // url may contain index.php? : it is not in pathinfo format : just return
        return;
      }
      else
      {
        $url = $match[2];
      }
    }

    decodeArguments($url);
  }
}

/**
 * \brief try to decode url as a querystring url
 */
function decodeQueryStringUrl()
{
  // if url start with configured path
  $url = $_SERVER['QUERY_STRING'];
  // do not rawurldecode here, let decodeArguments do it because url may contain both
  // '/' and '%2F' which when decoded will lead to '/' too
  // if we rawurldecode here we won't be able to make the difference between a '/' for the date
  // for example and a url-encoded '/' (%2F) in the title
  if (!decodeArguments($url))
  {
    // try to extract pair of key=value
    $kvs = explode('&', $url);
    foreach ($kvs as $kv1)
    {
      $kv    = explode('=', $kv1);
      if (sizeof($kv) == 2)
      {
        $key   = $kv[0];
        $value = rawurldecode($kv[1]);
        $_GET[$key] = $value;
      }
    }
  }
}

/**
 * \brief decode url depending on global configuration
 */
function decodeUrl()
{
  // rewrite the url if format is not a get format
  if (wc_url_format == 'get')
  {
    // default
  }
  else
  if (wc_url_format == 'path_info' || wc_url_format == 'path_info_mod_rewrite')
  {
    decodePathInfoUrl();
  }
  else
  if (wc_url_format == 'query_string' && isset($_SERVER['QUERY_STRING']))
  {
    decodeQueryStringUrl();
  }

  if (isset($_GET['title']))
  {
    $title = varGet('title');
    if (preg_match('/^SpecialPage\:(.*)$/', $title, $match))
    {
      $_GET['mode'] = $match[1];
      unset($_GET['title']);
    }
  }
}

/**
 * \brief creates a "raw" url
 *
 * depending if the url will be used in a header() or in a html link
 * the amp separator will be called with '&amp;' or just '&'
 *
 * \param title title of node
 * \param options map<key, value> options to encode in url depending on format
 * \param amp the separator to use
 *
 * \return url
 */
function createURLRaw($title, &$options, $amp)
{
  global $app_path;
  global $node;

  // title may have a '#' in it : do not escape it.
  $parts = explode('#', $title);
  $nbParts = sizeof($parts);
  for ($i = 0; $i < $nbParts; ++$i)
  {
    $parts[$i] = rawurlencode($parts[$i]);
  }
  $title = implode('#', $parts);

  $anchor = null;
  // Get anchor if it exist and keep till later;
  if (isset($options['anchor']))
  {
     $anchor = '#'.$options['anchor'];
     unset($options['anchor']);
  }

  // display url in path info form when they are of the form
  // title, or
  // lang/title
  if (wc_url_format == 'path_info' || wc_url_format == 'query_string' || wc_url_format == 'path_info_mod_rewrite')
  {
    // build base url depending of format
    $base_url = $app_path;
    if (wc_url_format == 'path_info')
    {
      $base_url .= wc_app_path; // for example 'index.php'.'/'
      if (!preg_match('!.*/$!', $base_url))
      {
        // no terminal '/' : add one
        $base_url .= '/';
      }
    }
    else
    if (wc_url_format == 'path_info_mod_rewrite')
    {
      $base_url .= wc_mod_rewrite_app_path;
      if (!preg_match('!.*/$!', $base_url))
      {
        // no terminal '/' : add one
        $base_url .= '/';
      }
    }
    else
    if (wc_url_format == 'query_string')
    {
      $base_url .= '?';
    }

    $nbOptions = sizeof($options);
    // now try to build arguments based on recognised patterns
    // title
    if ($nbOptions == 0)
    {
      $url = $base_url.$title;
    }
    else
    // lang/title
    if ($nbOptions == 1 && isset($options['lang']))
    {
      $lang = $options['lang'];
      $url = $base_url.$lang.'/'.$title;
    }
    else
    // lang/year/month/day/title
    if ($nbOptions == 5 &&
        isset($options['mode']) && ($options['mode'] == 'history_view_by_date') && // history.lib.php
        isset($options['lang']) &&
        isset($options['year']) &&
        isset($options['month']) &&
        isset($options['day'])
       )
    {
      $lang  = $options['lang'];
      $year  = $options['year'];
      $month = $options['month'];
      $day   = $options['day'];

      $url = $base_url.$lang.'/'.$year.'/'.$month.'/'.$day.'/'.$title;
    }

    // an url was constructed
    if (!empty($url))
    {
      //Final add anchor if on was set
      $url .=  $anchor;
      return $url;
    }
  }

  // revert to get format if options are too complex
  $nbArg = 0;
  $url = $app_path.wc_app_path;

  if (!empty($title) && $title != '')
  {
    $url .= '?title='.$title;
    $nbArg = 1;
  }

  if (!empty($options))
  {
    if ($nbArg == 0)
    {
      $url .= '?';
    }

    foreach ($options as $k=>$v)
    {
      if ($nbArg > 0)
      {
        $url .= $amp;
      }
      $url .= rawurlencode($k).'='.rawurlencode($v);
      ++$nbArg;
    }
  }
  //Final add anchor if on was set
  $url .=  $anchor;
  return $url;
}

/**
 * \brief creates an url suitable for an html link
 *
 * \param title title of node
 * \param options map<key, value> options to encode in url depending on format
 *
 * \return html url
 */
function createURL($title, $options = array())
{
  return createURLRaw($title, $options, '&amp;');
}

/**
 * \brief creates an url suitable for header()
 *
 * \param title title of node
 * \param options map<key, value> options to encode in url depending on format
 *
 * \return header url
 */
function createHeaderURL($title, $options = array())
{
  return createURLRaw($title, $options, '&');
}

/**
 * \brief creates an url to an icon depending of global configuration
 *
 * \param icon filename of icon
 * \param width optional desired width of icon
 * \param height optional desired height of icon
 *
 * \return full path to icon
 */
function createIconURL($icon, $width = -1, $height = -1)
{
  global $app_path;

  // for security reason, strip all '..' in $image
  $icon = str_replace("..", "", $icon);

  if ($width == -1 && $height == -1)
  {
    return $app_path.wc_icon_path.'/'.$icon;
  }
  else
  {
    return $app_path.'view_thumb.php?mode=icon&amp;width='.$width.'&amp;height='.$width.'&amp;thumb='.$icon;
  }
}

/**
 * \brief creates a path to an icon depending of global configuration
 *
 * \param icon filename of icon
 *
 * \return full path to icon
 */
function createIconPath($icon)
{
  // for security reason, strip all '..' in $image
  $icon = str_replace('..', '', $icon);

  return wc_inc_path.wc_icon_path.'/'.$icon;
}

/**
 * \brief creates an url to an image depending of global configuration
 *
 * \param image filename of image
 * \param width optional desired width of image
 * \param height optional desired height of image
 *
 * \return full path to image
 */
function createImageURL($image, $width = -1, $height = -1)
{
  global $app_path;

  // for security reason, strip all '..' in $image
  $image = str_replace('..', '', $image);

  if ($width == -1 && $height == -1)
  {
    return $app_path.wc_image_path.'/'.$image;
  }
  else
  {
    return $app_path.'view_thumb.php?width='.$width.'&amp;height='.$height.'&amp;thumb='.$image;
  }
}

/**
 * \brief creates a path to an image depending of global configuration
 *
 * \param image filename of image
 *
 * \return full path to image
 */
function createImagePath($image)
{
  // for security reason, strip all '..' in $image
  $image = str_replace('..', '', $image);

  return wc_inc_path.wc_image_path.'/'.$image;
}
