<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

/**
 * \file
 * \brief Controls the main functionality : content edition
 *
 * This file creates several hooks :
 * <ul>
 *   <li>watch_view : to see the list of subscription of a user</li>
 *   <li>watch  : to register to a particular page</li>
 *   <li>unwatch_by_id : to unregister from a particular page</li>
 *   <li>watch_rec : to register to a page and to all its sub pages</li>
  * </ul>
 *
 * The display page can be called from user preferences. The page contain only a list of links.
 * Links to stop watching a page, links to subscribe to all sub nodes of a page, and a link to unsubscribe to all.
 *
 * This page contains also several helper functions for the user to create template :
 * <ul>
 *   <li>emailSubscribeLink</li>
 *   <li>emailUnsubscribeLink</li>
 * </ul>
 */

$contentBoxHandle['watch_view']    = 'watchViewBox';

$userActionHandle['watch']         = 'onWatch';
$userActionHandle['unwatch']       = 'onUnwatch';
$userActionHandle['unwatch_by_id'] = 'onUnwatchById';
$userActionHandle['watch_rec']     = 'onWatchRecursively';
$userActionHandle['unwatch_all']   = 'onUnwatchAll';

$browserTitlePageHandle['watch_view'] = 'browserWatchView';

/**
 * \brief customize browser title when viewing watches
 *
 * \return browser title
 */
function browserWatchView()
{
  return wc_wiki_name.' - '.tr('Watches');
}

/**
 * \brief display link to subscribe to this node
 * \ingroup template_functions
 *
 * \return html link
 */
function emailSubscribeLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  global $user;

  if (!$user->isAnonymous() && !$user->isWatching($node->id))
  {
    return '<a href="'.createURL($node->title, array('mode'=>'watch', 'lang'=>$node->lang->code)).'">'.tr('Subscribe').'</a>';
  }
}

/**
 * \brief display link to unsubscribe to this node
 * \ingroup template_functions
 *
 * \return html link
 */
function emailUnsubscribeLink()
{
  global $node;

  if (empty($node))
  {
    return '';
  }

  global $user;

  if (!$user->isAnonymous() && $user->isWatching($node->id))
  {
    return '<a href="'.createURL($node->title, array('mode'=>'unwatch', 'lang'=>$node->lang->code)).'">'.tr('Unsubscribe').'</a>';
  }
}

/**
 * \brief display list of nodes this user has subscribed to
 *
 * \return html blob
 */
function watchViewBox()
{
  global $wiki;
  global $user;
  global $node;

  $idTitles = $wiki->getWatches($user->id);

  if (empty($idTitles))
  {
    return '<p>'.tr('No watches').'</p>';
  }

  $blob =
  '<p><a href="'.createUrl($node->title, array('mode'=>'unwatch_all')).'">'.tr('Unwatch all').'</a></p>'.
  '<table class="watch">'.
  '<tr>'.
  '<th>'.tr('Title').'</th>'.
  '<th>'.tr('Delete').'</th>'.
  '<th>'.tr('Watch recursively').'</th>'.
  '</tr>';

  foreach ($idTitles as $idTitle)
  {
    $blob .=
    '<tr>'.
    '<td><a href="'.createUrl($idTitle[1], array('lang'=>$idTitle[2])).'">'.$idTitle[1].' ['.$idTitle[2].']</a></td>'.
    '<td><a href="'.createUrl($node->title, array('mode'=>'unwatch_by_id', 'id'=>$idTitle[0])).'">'.tr('Unwatch').'</a></td>'.
    '<td><a href="'.createUrl($node->title, array('mode'=>'watch_rec', 'id'=>$idTitle[0])).'">'.tr('Apply').'</a></td>'.
    '</tr>';
  }
  $blob .= '</table>';

  return $blob;
}

/**
 * \brief called when user click on subscribe link
 */
function onWatch()
{
  global $wiki;
  global $node;
  global $user;

  if (!$user->isAnonymous())
  {
    $wiki->addWatch($node->id, $user->id);
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code)));
}

/**
 * \brief called when user click on unsubscribe link
 */
function onUnwatch()
{
  global $wiki;
  global $node;
  global $user;

  if (!$user->isAnonymous())
  {
    $wiki->removeWatch($node->id, $user->id);
  }

  header('Location: '.createHeaderURL($node->title, array('lang'=>$node->lang->code)));
}

/**
 * \brief called when user click on unsubscribe link in watchViewBox
 */
function onUnwatchById()
{
  global $wiki;
  global $user;
  global $node;

  if (!isset($_GET['id']))
  {
    return;
  }
  $id = varGet('id');

  if (!$user->isAnonymous())
  {
    $wiki->removeWatch($id, $user->id);
  }

  header('Location: '.createHeaderURL($node->title, array('mode'=>'watch_view')));
}

/**
 * \brief called when user click on subscribe link in watchViewBox
 */
function onWatchRecursively()
{
  global $wiki;
  global $user;
  global $node;

  if (!isset($_GET['id']) || $user->isAnonymous())
  {
    return;
  }
  $id = varGet('id');

  $wiki->addWatchRecursively($id, $user->id);

  header('Location: '.createHeaderURL($node->title, array('mode'=>'watch_view')));
}

/**
 * \brief called when user click on unsubscribe all link in watchViewBox
 */
function onUnwatchAll()
{
  global $wiki;
  global $user;
  global $node;

  if ($user->isAnonymous())
  {
    return;
  }

  $wiki->removeAllWatch($user->id);

  header('Location: '.createHeaderURL($node->title, array('mode'=>'watch_view')));
}
?>