#!/usr/bin/perl -w
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

use strict;

my %strings;
my %fuzzys;

sub analyse
{
  my $dir  = shift;
  my $file = shift;

  my $path = $dir.'/'.$file;
  open(FILE_HDL, "<$path") || die "Cannot open $path";

  while (my $line = <FILE_HDL>)
  {
    while ($line =~ /tr\(['"](.*?)['"]\)/mg)
    {
      $strings{$1} = "";
    }
  }

  close(FILE_HDL);
}

#
# recBrowsePath($dir)
#
sub recBrowsePath
{
  my $dir = shift;
  local *DIR_HDL;

  print "entering directory [$dir]\n";

  # first, let's find directory and recurse into it
  opendir DIR_HDL, $dir || die "cannot open '$dir'";
  while (my $file = readdir DIR_HDL)
  {
    if (-d $dir.$file)
    {
      if ($file eq "." || $file eq ".." || $file eq ".arch-ids")
      {
        next;
      }

      # recursively
      recBrowsePath($dir.$file."/");
    }
  }
  closedir(DIR_HDL);

  # now, only find php files
  opendir DIR_HDL, $dir || die "cannot open '$dir'";
  while (my $file = readdir DIR_HDL)
  {
    if (-f $dir.$file)
    {
      if ($file =~ /\.php$/)
      {
        analyse($dir, $file);
      }
    }
  }

  closedir(DIR_HDL);

  print "leaving directory [$dir]\n";
}

sub collectCurrentTranslation
{
  my $file = shift;

  print "Parsing file $file\n";
  open(FILE_HDL, "<$file") || die "Cannot open $file";

  my $state = 0;
  my $key;
  my $content;
  my %lang;

  while (my $line = <FILE_HDL>)
  {
    chomp($line);

    # do not read comments
    if ($line =~ /^#/)
    {
      next;
    }

    if ($state == 0)
    {
      # searching for a key
      if ($line =~ /^\[(.*)\]$/)
      {
        $key = $1;
        $content = "";
        $state = 1;
      }
    }
    elsif ($state == 1)
    {
      if ($line =~ /^$/)
      {
        $lang{$key} = $content;
        $state = 0;
      }
      else
      {
        $content .= $line;
      }
    }
  }

  if ($state == 1)
  {
    $lang{$key} = $content;
  }

  close(FILE_HDL);

  return %lang;
}

sub dumpHash
{
  my %hash = (@_);

  foreach my $key (keys %hash)
  {
    print $key."=>".$hash{$key}."\n";
  }
}

sub dumpArray
{
  my @array = (@_);

  foreach my $value (@array)
  {
    print $value."\n";
  }
}

sub updateTranslation
{
  my %currentTr = (@_);
  my %targetTr;
  my @fuzzyStrings = ();

  #print "FIRST PASS\n";
  #dumpHash(%currentTr);
  #print "---\n";
  #dumpHash(%strings);

  # first pass : find unchanged strings, remaining strings are pushed into fuzzyStrings
  foreach my $string (keys %strings)
  {
    if (exists $currentTr{$string})
    {
      $targetTr{$string} = $currentTr{$string};
      delete $currentTr{$string};
    }
    else
    {
      push @fuzzyStrings, $string;
    }
  }

  #print "SECOND PASS\n";
  #dumpHash(%currentTr);
  #print "---\n";
  #dumpArray(@fuzzyStrings);

  my @newStrings = ();
  # second pass, try to find fuzzy strings, remaining strings are pushed into newStrings
  foreach my $string (@fuzzyStrings)
  {
    my @words1 = split(/\s/, $string);

    my $bestKey;
    my $score = 0;
    foreach my $key (keys %currentTr)
    {
      # try to find a fuzzy string
      my @words2 = split(/\s/, $key);

      # count number of common words in $string and $key
      my $s = 0;
      foreach my $w1 (@words1)
      {
        foreach my $w2 (@words2)
        {
          if ($w1 =~ /^$w2$/)
          {
            ++$s;
            last;
          }
        }
      }

      $s = $s * 100 / ($#words1 + 1);
      if ($s > 70 && $s > $score)
      {
        $score = $s;
        $bestKey = $key;
      }
    }

    if ($score != 0)
    {
      # a fuzzy was found
      #print "Fuzzy found for [".$string."] is [".$bestKey."] with score ".$score."% words found\n";
      $targetTr{$string} = $currentTr{$bestKey};
      delete $currentTr{$bestKey};
      $fuzzys{$string} = "";
    }
    else
    {
      push @newStrings, $string;
    }
  }

  #print "THIRD PASS\n";
  #dumpHash(%currentTr);
  #print "---\n";
  #dumpArray(@newStrings);

  # third pass, add all remaining strings to target with empty translation
  foreach my $string (@newStrings)
  {
    $targetTr{$string} = "";
  }

  return %targetTr;
}

sub regenerateFile
{
  my ($file, %targetTr) = (@_);

  open(FILE_HDL, ">$file.new") || die "Cannot open $file.new";
  foreach my $key (keys %targetTr)
  {
    if (exists $fuzzys{$key})
    {
      print FILE_HDL "#Fuzzy\n";
    }

    print FILE_HDL "[".$key."]\n";
    print FILE_HDL $targetTr{$key}."\n";
    print FILE_HDL "\n";
  }
  close FILE_HDL;
}

# command line analysis
if ($#ARGV == -1)
{
  print "Error, I need at least one directory to browse\n";
  exit;
}

# for each path given on command line
for (my $i = 0; $i <= $#ARGV; ++$i)
{
  recBrowsePath($ARGV[$i]);
}

# for each file named .lang in current directory
opendir DIR_HDL, '.' || die "cannot open '.'";
while (my $file = readdir DIR_HDL)
{
  if (-f $file && $file =~ /^(.*)\.lang$/)
  {
    my %currentTr = collectCurrentTranslation($file);

    %fuzzys = ();
    my %targetTr = updateTranslation(%currentTr);

    regenerateFile($file, %targetTr);
  }
}
closedir(DIR_HDL);