#!/usr/bin/perl -w
# ***** BEGIN LICENSE BLOCK *****
# This file is part of WiClear.
# Copyright (c) 2004-2007 David Jobet. All rights
# reserved.
#
# WiClear is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# WiClear is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
#
# ***** END LICENSE BLOCK *****

use strict;

if ($#ARGV < 3)
{
  print "Usage : deploy.pl user password local_dir remote_dir [exclude_patterns]\n";
  print "Where : \n";
  print "  local_dir  : local directory to deploy\n";
  print "  remote_dir : where to put files\n";
  exit(1);
}

# extract arguments from command line
my $user       = $ARGV[0];
my $password   = $ARGV[1];
my $local_dir  = $ARGV[2];
my $remote     = $ARGV[3];
my @exclude_patterns = ();
for (my $i = 4; $i <= $#ARGV; ++$i)
{
  push @exclude_patterns, $ARGV[$i];
}

# add a final slash if needed
$local_dir  = addFinalSlash($local_dir);

my $remote_host = $remote;
my $remote_dir  = "";
if ($remote =~ /^ftp:\/\/(.*?)\/(.*)$/)
{
  $remote_host = $1;
  $remote_dir  = addFinalSlash($2);
}

printArgs();

# and transfer
my %filesPerDirectory = (); # dir=>array files
recBrowsePath($local_dir, \&accumulateFiles, 1);
transferFiles();

#
# print args
#
sub printArgs
{
  print "Transfering files from ".$local_dir." to ".$remote_host." in directory ".$remote_dir." with exclude patterns [";
  my $first = 1;
  foreach my $pattern (@exclude_patterns)
  {
    if ($first == 0)
    {
      print ", ";
    }
    else
    {
      $first = 0;
    }

    print $pattern;
  }
  print "]\n";
}

#
# add a final slash to argument if it is missing
#
sub addFinalSlash
{
  my $file = shift;

  if ($file =~ /^.*\/$/)
  {
    # end with a '/' : OK
  }
  else
  {
    # final '/' missing : add it
    $file = $file.'/';
  }

  return $file;
}

#
# recBrowsePath($dir, $callback)
# browse recursively in path $dir and callback $callback each time a file is found
#
sub recBrowsePath
{
  my $dir      = shift;
  my $function = shift;
  my $silent   = shift || 0;

  my @subdirs;

  if (-f $dir)
  {
    # if dir is a file
    $function->($dir);
  }
  elsif (-d $dir)
  {
    if ($silent == 0)
    {
      print STDERR "Entering directory [$dir]\n";
    }

    # first, let's find directory and recurse into it
    local *H_DIR;
    opendir H_DIR, $dir || die "cannot open '$dir'";
    while (my $file = readdir H_DIR)
    {
      #print "Testing [".$dir.$file."]\n";
      if (-d $dir.$file)
      {
        if ($file eq "." || $file eq "..")
        {
          # do nothing
          next;
        }
        else
        {
          # keep for recurse browsing
          push(@subdirs, $dir.$file."/");
        }
      }
      elsif (-f $dir.$file)
      {
        $function->($dir, $file);
      }
    }
    closedir(H_DIR);

    foreach my $subdir (@subdirs)
    {
      recBrowsePath($subdir, $function, $silent);
    }

    if ($silent == 0)
    {
      print STDERR "Leaving directory [$dir]\n";
    }
  }
}

#
# accumulate files
#
sub accumulateFiles
{
  my $dir  = shift;
  my $file = shift;

  my $full_file = $dir.$file;

  foreach my $pattern (@exclude_patterns)
  {
    if ($full_file =~ /$pattern/)
    {
      return;
    }
  }

  $filesPerDirectory{$dir} .= " ".$dir.$file;
}

#
# transfer files to an host
#
sub transferFiles
{
  $local_dir = quotemeta($local_dir);

  foreach my $dir (keys %filesPerDirectory)
  {
    my $files = $filesPerDirectory{$dir};
    my $relative_dir = $dir;
    $relative_dir =~ s/^($local_dir)(.*)$/$2/g;

    my $full_remote_dir = $remote_dir.$relative_dir;
    my $command = "ncftpput -u $user -p $password -m -r 3 $remote_host $full_remote_dir $files";
    #print $command\n";
    `$command`;
  }
}

